Package: evacpath
Title: Least-Cost Pedestrian Evacuation Modeling
Version: 0.1.0
Authors@R: c(
    person("Elvin", "Cordero", role = c("aut", "cre"), email = "elvin.cordero1@upr.edu")
  )
Description: Tools for road-constrained, least-cost pedestrian evacuation modeling.
    The package provides reusable functions for preparing hazard zones, generating
    road-based evacuation origin points, identifying escape/safety points, creating
    slope-based conductance surfaces, calculating least-cost distance to safety,
    and converting distance outputs into evacuation-time polygons. It is designed
    to support workflows like tsunami evacuation modeling while remaining
    adaptable to other regions and hazards. Tsunami-specific helpers support separate
    land-only hazard zones, water-combined escape zones, road-aware escape
    boundaries, and study-area inset cropping for quality assurance and quality
    control. Methods build on Cordero et al. (2025) <doi:10.1007/s44367-025-00018-y>,
    Lewis (2021) <doi:10.1007/s10816-021-09522-w>, and Joseph Lewis's
    'leastcostpath' package (2023) <https://CRAN.R-project.org/package=leastcostpath>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1)
Imports: terra, leastcostpath, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), devtools, fields
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-22 17:19:23 UTC; ec
Author: Elvin Cordero [aut, cre]
Maintainer: Elvin Cordero <elvin.cordero1@upr.edu>
Repository: CRAN
Date/Publication: 2026-05-28 18:50:02 UTC
Built: R 4.5.2; ; 2026-05-28 21:21:35 UTC; unix

Package: evacpath
Title: Least-Cost Pedestrian Evacuation Modeling
Version: 0.1.0
Authors@R: c(
    person("Elvin", "Cordero", role = c("aut", "cre"), email = "elvin.cordero1@upr.edu")
  )
Description: Tools for road-constrained, least-cost pedestrian evacuation modeling.
    The package provides reusable functions for preparing hazard zones, generating
    road-based evacuation origin points, identifying escape/safety points, creating
    slope-based conductance surfaces, calculating least-cost distance to safety,
    and converting distance outputs into evacuation-time polygons. It is designed
    to support workflows like tsunami evacuation modeling while remaining
    adaptable to other regions and hazards. Tsunami-specific helpers support separate
    land-only hazard zones, water-combined escape zones, road-aware escape
    boundaries, and study-area inset cropping for quality assurance and quality
    control. Methods build on Cordero et al. (2025) <doi:10.1007/s44367-025-00018-y>,
    Lewis (2021) <doi:10.1007/s10816-021-09522-w>, and Joseph Lewis's
    'leastcostpath' package (2023) <https://CRAN.R-project.org/package=leastcostpath>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1)
Imports: terra, leastcostpath, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), devtools, fields
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-22 17:19:23 UTC; ec
Author: Elvin Cordero [aut, cre]
Maintainer: Elvin Cordero <elvin.cordero1@upr.edu>
Repository: CRAN
Date/Publication: 2026-05-28 18:50:02 UTC
Built: R 4.6.0; ; 2026-05-28 19:44:58 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,evacpath_result)
S3method(print,evacpath_tsunami_zones)
export(calc_evac_time)
export(calc_min_distance_to_safety)
export(calculate_lc_path)
export(calculate_min_dist)
export(clean_roads)
export(crop_roads_to_inner_extent)
export(find_escape_points)
export(interpolate_distance_surface)
export(make_conductance_surface)
export(make_evac_grid)
export(make_evac_polygons)
export(make_region_area)
export(make_road_aware_escape_zone)
export(make_road_mask)
export(make_road_origins)
export(make_roads_in_zone)
export(prepare_evac_inputs)
export(prepare_hazard_zone)
export(prepare_tsunami_zones)
export(read_spatial)
export(run_evacpath)
export(write_evac_outputs)

# evacpath <img src="man/figures/logo.png" align="right" height="140" alt="evacpath logo" />

`evacpath` is an R package for road-constrained pedestrian evacuation modeling using least-cost path analysis. It takes generic hazard, road/pathway, and elevation inputs, then returns distance-to-safety and evacuation-time outputs for any projected study area.

The package is designed around a simple idea:

> hazard zone + roads/pathways + DEM + projected CRS -> distance-to-safety and evacuation-time outputs

The workflow builds on open-source least-cost path methods for evacuation planning (Cordero et al. 2025) and uses `leastcostpath` for least-cost path and movement-potential modeling (Lewis 2023; Lewis 2021).

## Current status

This is an initial package scaffold, not a polished CRAN release. It is ready for local development with `devtools::load_all()` and `devtools::document()`.

## Core workflow

```r
library(evacpath)

result <- run_evacpath(
  hazard_zone = "path/to/hazard_zone.tif",
  roads = "path/to/roads.gpkg",
  dem = "path/to/dem.tif",
  target_crs = "EPSG:XXXX",
  region_name = "Study area",
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,
  seed = 23401,
  walking_speed_mps = 1.22,
  clip_mode = "hazard",
  progress = TRUE
)

result$evac_polygons
result$distance_points

write_evac_outputs(
  result,
  output_dir = "outputs",
  prefix = "study_area"
)
```

## Main package functions

| Function | Purpose |
|---|---|
| `read_spatial()` | Reads either a file path or an existing `terra` object. |
| `prepare_hazard_zone()` | Converts an inundation raster into a hazard-zone raster or polygon. |
| `clean_roads()` | Removes roads/pathways based on user-defined attributes. |
| `prepare_evac_inputs()` | Reads and projects the hazard zone, roads, and DEM. |
| `make_evac_grid()` | Creates the evacuation grid over the hazard zone. |
| `find_escape_points()` | Finds candidate exits where roads cross the hazard boundary. |
| `make_road_mask()` | Buffers roads and escape points to constrain the DEM. |
| `make_road_origins()` | Creates road-based origin points inside the hazard zone. |
| `make_conductance_surface()` | Builds the slope-based conductance surface. |
| `calc_min_distance_to_safety()` | Calculates minimum least-cost distance to safety. |
| `calc_evac_time()` | Converts distance to evacuation time. |
| `make_evac_polygons()` | Creates Voronoi evacuation-distance/time polygons. |
| `run_evacpath()` | Runs the full workflow. |
| `write_evac_outputs()` | Writes the main outputs to disk. |

## Customizable assumptions

Most modeling assumptions are explicit parameters:

```r
run_evacpath(
  ...,
  target_crs = "EPSG:XXXX",
  grid_resolution = NULL,
  grid_resolution_factor = 5,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,
  dem_resolution = NULL,
  walking_speed_mps = 1.22,
  clip_mode = "hazard"
)
```

## Tsunami preprocessing

Some coastal workflows need two tsunami zones. The land-only inundation zone is used for origins and mapping. A second zone combines the land inundation footprint with water so that the coastline is not treated as an artificial escape boundary.

```r
library(terra)
library(evacpath)

tsunami <- rast("path/to/inundation.nc")
dem <- rast("path/to/topography.nc")
roads <- vect("path/to/roads.shp")

zones <- prepare_tsunami_zones(
  inundation = tsunami,
  dem = dem,
  target_crs = "EPSG:XXXX",
  inundation_threshold = 0,
  dem_sign_multiplier = 1
)

roads <- clean_roads(
  roads,
  exclude = list(field = "road_type", values = "pier"),
  target_crs = "EPSG:XXXX"
)

result <- run_evacpath(
  hazard_zone = zones$hazard_zone,  # land-only TEZ for origins/output
  escape_zone = zones$escape_zone,  # TEZ + water for true inland escape boundary
  roads = roads,
  dem = zones$dem,
  target_crs = "EPSG:XXXX",
  region_name = "Study area"
)
```

Do not use the land-only `hazard_zone` to find escape points in tsunami workflows unless you intentionally want the coastline to act as a boundary. In most tsunami cases, use `escape_zone = zones$escape_zone`.

## Local development

From the parent directory of the package:

```r
install.packages(c("devtools", "terra", "remotes"))
remotes::install_github("josephlewis/leastcostpath") # if leastcostpath is not on CRAN for your setup

devtools::load_all("evacpath")
devtools::document("evacpath")
devtools::check("evacpath")
```

## Notes

- Use a projected CRS in meters before distance modeling.
- Keep case-specific preprocessing outside the core package when possible.
- Do not assume every DEM needs to be multiplied by `-1`; make that decision per dataset.
- Do not assume every road layer has the same fields; use `clean_roads()` with region-specific settings.
- Use `max_origins` for exploratory runs and remove or increase it for final production runs.


## Time-grid output

By default, `run_evacpath()` now clips `result$evac_polygons` / `result$time_grid` to the full hazard zone, not just the buffered road network. Movement is still calculated using the road-constrained conductance surface, but the mapped time grid is a Voronoi-style surface over the hazard/inundation zone. Use `clip_mode = "road_hazard"` only when you want the older road-buffer-limited output.


## Example data and figures

Small example inputs are included in `inst/extdata/`:

```text
inst/extdata/dem.tif
inst/extdata/rds.gpkg
inst/extdata/tsunami_inundation_depth.tif
```

These files are from the Jakarta example dataset and are used only to demonstrate how to load package data:

```r
dem <- terra::rast(system.file("extdata/dem.tif", package = "evacpath"))
roads <- terra::vect(system.file("extdata/rds.gpkg", package = "evacpath"))
inundation <- terra::rast(system.file("extdata/tsunami_inundation_depth.tif", package = "evacpath"))
```

The detailed diagnostic example is available at `vignettes/diagnostic-example.Rmd` and `inst/examples/diagnostic-example.Rmd`. It walks through each major function separately, including tsunami-specific zone preparation, inset cropping of roads used for escape-point detection, road-aware escape-boundary generation, least-cost-path testing, and final time-grid mapping.

The README figures are generated with `terra` plotting code in `inst/scripts/make-readme-figures.R`. The figures use a focused projected window (`x = 669000` to `675000`, `y = 9223000` to `9225000`) so the examples stay readable.

![Example hazard zone, roads, and escape points](man/figures/readme-example-inputs.png)

![Example modeled evacuation time](man/figures/readme-example-time.png)

For tsunami workflows, the preferred escape-point pattern is:

```r
zones <- prepare_tsunami_zones(inundation, dem, target_crs = target_crs)

roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads,
  zone = zones$escape_zone,
  inset_x_m = 250,
  inset_y_m = 250
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 2,
  crop_buffer_m = 3
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)
```

Use `?evacpath` after running `devtools::document()` or installing the package to view the package-level help page.

## References

Cordero, E., Ruiz Vélez, R., Huérfano Moreno, V., Sherman, C., 2025. Enhancing tsunami evacuation strategies in Puerto Rico using open-source least-cost path analysis. J. Disaster Sci. Manag. 1, 18. https://doi.org/10.1007/s44367-025-00018-y

Lewis, J., 2023. leastcostpath: Modelling Pathways and Movement Potential Within a Landscape.

Lewis, J., 2021. Probabilistic Modelling for Incorporating Uncertainty in Least Cost Path Results: a Postdictive Roman Road Case Study. Journal of Archaeological Method and Theory 28, 911-924. https://doi.org/10.1007/s10816-021-09522-w

### Function: Escape Points
### This function intersects the roads network with the borders of the
### study area, then converts the intersection locations into points. 
### Essentially, it discovers escape/exit points

# requires:
# library: terra

# network <- roads
# area.region <- study region (greater area)

escape_points <- function(area.evac,network,area.study=NULL){
  area.evac <- aggregate(area.evac)
  area.evac <- as.lines(area.evac)
  network <- as.lines(network)
  escape <- terra::intersect(network,area.evac)
  # escape <- as.points(escape)
  
  # There is an issue with the conversion of lines to points.
  # Multipoint vector is created with certain points
  # containing points with multiple parts.
  escape <- crds(escape,df=TRUE)
  escape <- vect(escape, geom=c("x", "y"), crs=crs(area.evac))
  
  if (is.null(area.study) == FALSE){
    area.region <- region_area(area.evac,area.study)
    # crop to region extent
    escape <- crop(escape,area.region,ext=TRUE)
  }
  
  return(escape)
}

### Function: Calculate Minimum Distance
### This function calculates the minimum distance from a list of line-type vectors
### generated from calculate_lc_path

# requires:
# library: terra, leastcostpath

# lc_paths_list <- list of line vectors 

calculate_min_dist <- function(lc_paths_list){
    lc_paths <- vect(lc_paths_list)
    lc_paths_perim <- perim(lc_paths) # return vector of distance
    lc_paths_perim <- lc_paths_perim[!is.infinite(lc_paths_perim)]
    lc_paths_perim <- lc_paths_perim[lc_paths_perim != 0]
    min_dist <- min(lc_paths_perim) 
    return(min_dist)
}

# Modular evacpath workflow --------------------------------------------------
# Use this when you want full control over each geoprocessing step.

library(terra)
library(evacpath)

zones <- prepare_tsunami_zones(
  inundation = "_data/tsunami_inundation_depth.tif",
  dem = "_data/dem.tif",
  target_crs = "EPSG:32748"
)

roads <- clean_roads(
  "_data/rds.gpkg",
  exclude = list(field = "man_made", values = "pier"),
  target_crs = "EPSG:32748"
)

roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads,
  zone = zones$escape_zone,
  inset_x_m = 250,
  inset_y_m = 250
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 2,
  crop_buffer_m = 3
)

grid <- make_evac_grid(zones$hazard_zone, resolution = terra::res(zones$dem) * 50)

escape <- find_escape_points(escape_boundary_zone, roads_for_escape)

mask_parts <- make_road_mask(
  roads = roads,
  escape_points = escape,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  return_components = TRUE
)

origins <- make_road_origins(
  evac_grid = grid,
  roads_buffer = mask_parts$roads_buffer,
  seed = 23401
)

cs <- make_conductance_surface(
  dem = zones$dem,
  road_mask = mask_parts$mask,
  resolution = terra::res(zones$dem) * 50
)

dist_pts <- calc_min_distance_to_safety(
  cs = cs,
  origins = origins,
  destinations = escape,
  progress = TRUE,
  progress_every = 1,
  check_locations = FALSE
)

polys <- make_evac_polygons(
  distance_points = dist_pts,
  clip_area = zones$hazard_zone,
  walking_speed_mps = 1.22,
  region_name = "Study area"
)

### Function: Region Area
### This function creates a buffer around the evacuation zone
### in order to include evacuation points outside the municipality
### for a more accurate distance analysis.

# requires:
# library: terra

### area.evac <- the entire evacuation zone (zono de desalojo)
### area.study <- the evacuation zone for the municipality

region_area <- function(area.evac, area.study){
  area.region <- aggregate(area.evac,dissolve=TRUE)
  buff <- buffer(area.study, 5000)
  buff <- aggregate(buff,dissolve=TRUE)
  area.region <- crop(area.region,buff)
  area.region <- erase(area.region)
  return(area.region)
}

# Example workflow ---------------------------------------------------
# This is a compact version of the diagnostic workflow. For full QA/QC, see
# diagnostic-example.Rmd or vignettes/diagnostic-example.Rmd.

library(terra)
library(evacpath)

# ---- Raw data --------------------------------------------------------------
tsunami <- rast("_data/raw/inundation.nc")
dem <- rast("_data/raw/topo.nc")
roads <- vect("_data/raw/road.shp")

target_crs <- "EPSG:32748"

# ---- Example-specific preprocessing ---------------------------------------
# Change dem_sign_multiplier to -1 if your topobathymetry sign convention is
# reversed relative to land > 0 and water < 0.
zones <- prepare_tsunami_zones(
  inundation = tsunami,
  dem = dem,
  target_crs = target_crs,
  dem_sign_multiplier = 1,
  inundation_threshold = 0
)

roads <- clean_roads(
  roads,
  exclude = list(field = "man_made", values = "pier"),
  target_crs = target_crs
)

# Crop roads used only for escape-point detection to avoid artificial study-area
# edge intersections, then include buffered road corridors in the escape boundary.
roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads,
  zone = zones$escape_zone,
  inset_x_m = 250,
  inset_y_m = 250
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 2,
  crop_buffer_m = 3
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

# ---- Full evacuation workflow ---------------------------------------------
result <- run_evacpath(
  hazard_zone = zones$hazard_zone,
  escape_zone = zones$escape_zone,
  roads = roads,
  dem = zones$dem,
  target_crs = target_crs,
  region_name = "Example region",
  grid_resolution = terra::res(zones$dem) * 50,
  dem_resolution = terra::res(zones$dem) * 50,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,
  escape_roads_inset_x_m = 250,
  escape_roads_inset_y_m = 250,
  road_aware_escape_zone = TRUE,
  seed = 23401,
  walking_speed_mps = 1.22,
  clip_mode = "hazard",
  progress = TRUE
)

plot(result$time_grid, "EvacTimeAvg", border = NA)
plot(result$escape_points, add = TRUE, pch = 22, bg = "red", col = "black")

write_evac_outputs(
  result,
  output_dir = "_outputs/example",
  prefix = "example"
)

### Function: Calculate Least Cost Path
### This function calculates the least cost path if the destination can be
### reached from the origin via the conductance surface 

# requires:
# library: terra, leastcostpath

# cs <- leastcostpath conductance matrix
# origin <- origin point
# destination <- destination point

calculate_lc_path <- function(cs, origin, destination){
    lc_path <- tryCatch(
        {
        create_lcp(x = cs, origin = origin,
        destination = destination,check_locations = TRUE)
    },  error = function(e) {return(NULL)}
    )
    if (is.null(lc_path) == FALSE){
        return(lc_path)
    }
}

library(terra)

if (file.exists("DESCRIPTION") && requireNamespace("devtools", quietly = TRUE)) {
  devtools::load_all(".", quiet = TRUE)
} else {
  library(evacpath)
}

out_dir <- file.path("_outputs", "function-example-check")
dir.create(out_dir, recursive = TRUE, showWarnings = FALSE)

if (!interactive()) {
  pdf(file.path(out_dir, "function-example-check.pdf"), width = 10, height = 7)
  on.exit(dev.off(), add = TRUE)
}

pause <- function(label) {
  cat("\n\n---", label, "---\n")
  if (interactive()) readline("Press [enter] to continue...")
}

check_inside <- function(points, polygon) {
  inside <- relate(points, polygon, "intersects")
  c(total = nrow(points), inside = sum(inside), outside = sum(!inside))
}

oldpar <- par(no.readonly = TRUE)
on.exit(par(oldpar), add = TRUE)

# -------------------------------------------------------------------------
# 1. Build a small but realistic coastal example
# -------------------------------------------------------------------------

target_crs <- "EPSG:3857"

dem <- rast(
  nrows = 80,
  ncols = 120,
  xmin = 0,
  xmax = 6000,
  ymin = 0,
  ymax = 4000,
  crs = target_crs
)

xy <- crds(dem, df = TRUE)

# Offshore values are negative. Elevation rises inland with a low ridge and a
# shallow valley to make the least-cost surface visually meaningful.
values(dem) <-
  -2.5 +
  0.0011 * xy$x +
  0.00025 * xy$y +
  1.1 * exp(-((xy$x - 4200)^2 + (xy$y - 3000)^2) / 1200000) -
  0.8 * exp(-((xy$x - 2600)^2 + (xy$y - 1500)^2) / 600000)

inundation <- rast(dem)
values(inundation) <- ifelse(
  values(dem) > 0 & values(dem) < 2.8 & xy$x < 4300,
  1.2 + 0.0002 * (4300 - xy$x),
  0
)

roads <- vect(
  list(
    matrix(c(600, 800, 1900, 1200, 3200, 1500, 5200, 2200), ncol = 2, byrow = TRUE),
    matrix(c(800, 2600, 2100, 2300, 3600, 2500, 5600, 3300), ncol = 2, byrow = TRUE),
    matrix(c(1800, 500, 2100, 1500, 2150, 2600, 2300, 3800), ncol = 2, byrow = TRUE),
    matrix(c(3300, 600, 3350, 1600, 3500, 2600, 3800, 3700), ncol = 2, byrow = TRUE),
    matrix(c(500, 400, 800, 200), ncol = 2, byrow = TRUE)
  ),
  type = "lines",
  crs = target_crs
)
roads$name <- c("coastal road", "ridge road", "school access", "inland connector", "pier")
roads$kind <- c("road", "road", "road", "road", "pier")

# -------------------------------------------------------------------------
# Shared plotting extent and plotting helper
# -------------------------------------------------------------------------

# Every plot in this walkthrough uses the full synthetic DEM extent.
plot_extent <- ext(dem)
plot_xlim <- c(xmin(plot_extent), xmax(plot_extent))
plot_ylim <- c(ymin(plot_extent), ymax(plot_extent))

map_plot <- function(x, ..., add = FALSE) {
  if (add) {
    terra::plot(x, add = TRUE, ...)
  } else {
    terra::plot(
      x,
      ...,
      xlim = plot_xlim,
      ylim = plot_ylim,
      axes = FALSE,
      xlab = "",
      ylab = "",
      asp = 1
    )
  }
}

# -------------------------------------------------------------------------
# 1. Visualize input rasters
# -------------------------------------------------------------------------

par(
  mfrow = c(1, 2),
  mar = c(2, 2, 3, 5),
  las = 1,
  xaxs = "i",
  yaxs = "i"
)

map_plot(dem, main = "Synthetic Coastal DEM")
map_plot(inundation, main = "Synthetic Inundation Depth")

pause("Input rasters")

# -------------------------------------------------------------------------
# 2. prepare_hazard_zone()
# -------------------------------------------------------------------------

hazard_from_depth <- prepare_hazard_zone(inundation, threshold = 0, as_polygon = TRUE)
cat("Hazard polygons:", nrow(hazard_from_depth), "\n")

par(
  mfrow = c(1, 1),
  mar = c(2, 2, 3, 2),
  las = 1,
  xaxs = "i",
  yaxs = "i"
)

map_plot(
  hazard_from_depth,
  col = "#f28e2b99",
  border = NA,
  main = "prepare_hazard_zone()"
)
map_plot(roads, add = TRUE, col = "grey25", lwd = 1.2)

pause("prepare_hazard_zone")

# -------------------------------------------------------------------------
# 3. prepare_tsunami_zones()
# -------------------------------------------------------------------------

zones <- prepare_tsunami_zones(
  inundation = inundation,
  dem = dem,
  target_crs = target_crs,
  inundation_threshold = 0,
  dem_sign_multiplier = 1,
  as_polygon = TRUE,
  dissolve = TRUE
)

print(names(zones))

par(
  mfrow = c(2, 2),
  mar = c(2, 2, 3, 5),
  las = 1,
  xaxs = "i",
  yaxs = "i"
)

map_plot(zones$dem, main = "DEM")
map_plot(zones$land_mask, main = "Land Mask")
map_plot(
  zones$hazard_zone,
  col = "#f28e2b99",
  border = NA,
  main = "Land-Only Hazard Zone"
)
map_plot(
  zones$escape_zone,
  col = "grey90",
  border = "grey60",
  main = "Escape Zone: Hazard + Water"
)
map_plot(zones$hazard_zone, add = TRUE, col = "#f28e2b99", border = NA)

pause("prepare_tsunami_zones")

# -------------------------------------------------------------------------
# 4. clean_roads()
# -------------------------------------------------------------------------

roads_clean <- clean_roads(roads, exclude = list(field = "kind", values = "pier"))
cat("Roads before cleaning:", nrow(roads), "\n")
cat("Roads after cleaning:", nrow(roads_clean), "\n")

par(
  mfrow = c(1, 2),
  mar = c(2, 2, 3, 1),
  las = 1,
  xaxs = "i",
  yaxs = "i"
)

map_plot(
  roads,
  col = ifelse(roads$kind == "pier", "red", "grey30"),
  lwd = 2,
  main = "Before clean_roads()"
)
map_plot(
  roads_clean,
  col = "grey30",
  lwd = 2,
  main = "After clean_roads()"
)

pause("clean_roads")

# -------------------------------------------------------------------------
# 5. prepare_evac_inputs()
# -------------------------------------------------------------------------

inputs <- prepare_evac_inputs(
  hazard_zone = zones$hazard_zone,
  roads = roads_clean,
  dem = zones$dem,
  target_crs = target_crs
)

str(names(inputs))

pause("prepare_evac_inputs")

# -------------------------------------------------------------------------
# 6. make_evac_grid()
# -------------------------------------------------------------------------

evac_grid <- make_evac_grid(inputs$hazard_zone, resolution = 250)
cat("Evacuation grid cells:", nrow(evac_grid), "\n")

par(
  mfrow = c(1, 1),
  mar = c(2, 2, 3, 2),
  las = 1,
  xaxs = "i",
  yaxs = "i"
)

map_plot(
  inputs$hazard_zone,
  col = "grey95",
  border = "grey70",
  main = "make_evac_grid()"
)
map_plot(evac_grid, add = TRUE, border = "#4e79a7")
map_plot(inputs$roads, add = TRUE, col = "grey20", lwd = 1.1)

pause("make_evac_grid")

# -------------------------------------------------------------------------
# 7. find_escape_points(), crop_roads_to_inner_extent(), make_road_aware_escape_zone()
# -------------------------------------------------------------------------

roads_for_escape <- crop_roads_to_inner_extent(
  roads = inputs$roads,
  zone = zones$escape_zone,
  inset_x_m = 50,
  inset_y_m = 50
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 40,
  crop_buffer_m = 80
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

cat("Escape points:", nrow(escape_points), "\n")

map_plot(
  escape_boundary_zone,
  col = "grey92",
  border = "grey55",
  main = "Escape-Point Detection"
)
map_plot(zones$hazard_zone, add = TRUE, col = "#f28e2b80", border = NA)
map_plot(roads_for_escape, add = TRUE, col = "grey20", lwd = 1.1)
map_plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.9)

pause("escape-point functions")

# -------------------------------------------------------------------------
# 8. make_region_area()
# -------------------------------------------------------------------------

study_area <- crop(zones$hazard_zone, ext(800, 3600, 500, 3000))
region_area <- make_region_area(zones$escape_zone, study_area, buffer_m = 700)

map_plot(
  zones$escape_zone,
  col = "grey95",
  border = "grey70",
  main = "make_region_area()"
)
map_plot(region_area, add = TRUE, col = "#4e79a755", border = "#4e79a7")
map_plot(study_area, add = TRUE, col = "#f28e2b99", border = NA)

pause("make_region_area")

# -------------------------------------------------------------------------
# 9. make_road_mask() and make_road_origins()
# -------------------------------------------------------------------------

mask_parts <- make_road_mask(
  roads = inputs$roads,
  escape_points = escape_points,
  road_buffer_m = 45,
  escape_buffer_m = 70,
  return_components = TRUE
)

road_points <- make_road_origins(
  evac_grid = evac_grid,
  roads_buffer = mask_parts$roads_buffer,
  hazard_zone = inputs$hazard_zone,
  max_origins = 40,
  seed = 42
)

cat("Road origins:\n")
print(check_inside(road_points, inputs$hazard_zone))

map_plot(
  inputs$hazard_zone,
  col = "grey95",
  border = "grey65",
  main = "Road Origins Inside Hazard Zone"
)
map_plot(mask_parts$roads_buffer, add = TRUE, col = "#bab0ab80", border = NA)
map_plot(inputs$roads, add = TRUE, col = "grey20", lwd = 0.8)
map_plot(road_points, add = TRUE, pch = 20, col = "#4e79a7", cex = 0.8)

pause("make_road_mask / make_road_origins")

# -------------------------------------------------------------------------
# 10. make_conductance_surface(), calculate_lc_path(), calculate_min_dist()
# -------------------------------------------------------------------------

conductance <- make_conductance_surface(
  dem = inputs$dem,
  road_mask = mask_parts$mask,
  resolution = 100
)

one_path <- calculate_lc_path(conductance, road_points[1, ], escape_points[1, ])
path_distance <- calculate_min_dist(list(one_path))
cat("Example least-cost path distance:", round(path_distance, 1), "map units\n")

map_plot(inputs$dem, main = "Single Least-Cost Path")
map_plot(mask_parts$mask, add = TRUE, col = "#ffffff55", border = NA)
map_plot(inputs$roads, add = TRUE, col = "grey20", lwd = 0.8)
map_plot(one_path, add = TRUE, col = "#4e79a7", lwd = 2)
map_plot(road_points[1, ], add = TRUE, pch = 20, col = "black", cex = 1.1)
map_plot(escape_points[1, ], add = TRUE, pch = 22, bg = "red", col = "black", cex = 1.1)

pause("conductance and least-cost path")

# -------------------------------------------------------------------------
# 11. calc_min_distance_to_safety(), calc_evac_time(), make_evac_polygons()
# -------------------------------------------------------------------------

distance_points <- calc_min_distance_to_safety(
  cs = conductance,
  origins = road_points,
  destinations = escape_points,
  progress = TRUE,
  progress_every = 10,
  check_locations = FALSE
)

distance_points$minutes <- calc_evac_time(distance_points$distance, walking_speed_mps = 1.22)

evac_polygons <- make_evac_polygons(
  distance_points = distance_points,
  clip_area = inputs$hazard_zone,
  walking_speed_mps = 1.22,
  region_name = "Synthetic coast"
)

cat("Distance/time points:", nrow(distance_points), "\n")
print(summary(evac_polygons$EvacTimeAvg))

map_plot(
  evac_polygons,
  "EvacTimeAvg",
  main = "Evacuation Time Polygons"
)
map_plot(inputs$roads, add = TRUE, col = "grey20", lwd = 0.6)
map_plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.6)

pause("distance, time, polygons")

# -------------------------------------------------------------------------
# 12. run_evacpath() and write_evac_outputs()
# -------------------------------------------------------------------------

result <- run_evacpath(
  hazard_zone = inputs$hazard_zone,
  escape_zone = zones$escape_zone,
  roads = inputs$roads,
  roads_for_escape = roads_for_escape,
  dem = inputs$dem,
  target_crs = target_crs,
  region_name = "Synthetic coast",
  grid_resolution = 250,
  road_buffer_m = 45,
  escape_buffer_m = 70,
  final_road_buffer_m = 55,
  road_aware_escape_zone = TRUE,
  escape_zone_road_buffer_m = 40,
  escape_zone_crop_buffer_m = 80,
  max_origins = 40,
  max_destinations = 20,
  seed = 42,
  walking_speed_mps = 1.22,
  clip_mode = "hazard",
  progress = TRUE,
  progress_every = 10
)

cat("run_evacpath road origins:\n")
print(check_inside(result$road_points, result$hazard_zone))

paths <- write_evac_outputs(
  result,
  output_dir = out_dir,
  prefix = "synthetic_coast",
  include_inputs = TRUE,
  overwrite = TRUE
)

print(paths)

map_plot(
  result$time_grid,
  "EvacTimeAvg",
  main = "run_evacpath() Output"
)
map_plot(result$roads, add = TRUE, col = "grey20", lwd = 0.5)
map_plot(result$road_points, add = TRUE, pch = 20, col = "#4e79a7", cex = 0.65)
map_plot(result$escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.65)

cat("\nOutputs written to:\n", normalizePath(out_dir), "\n")

pause("run_evacpath / write_evac_outputs")

params <-
list(run_diagnostics = FALSE, run_lcp = FALSE, data_dir = "data-raw", 
    target_crs = "EPSG:32748", region_name = "Example region", 
    dem_sign_multiplier = 1L, inundation_threshold = 0L, land_threshold = 0L, 
    water_threshold = 0L, resample_method = "bilinear", road_exclude_field = "man_made", 
    road_exclude_values = "pier", grid_multiplier = 50L, dem_multiplier = 50L, 
    road_buffer_m = 2L, escape_buffer_m = 5L, final_road_buffer_m = 3L, 
    escape_roads_inset_x_m = 50L, escape_roads_inset_y_m = 50L, 
    road_aware_escape_zone = TRUE, walking_speed_mps = 1.22, 
    seed = 23401L, output_dir = "_outputs/example")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 6,
  dpi = 120
)

## ----render-command, eval = FALSE---------------------------------------------
# rmarkdown::render(
#   "vignettes/diagnostic-example.Rmd",
#   params = list(
#     run_diagnostics = TRUE,
#     run_lcp = TRUE,
#     data_dir = "data-raw",
#     target_crs = "EPSG:32748",
#     dem_sign_multiplier = 1,
#     grid_multiplier = 50,
#     dem_multiplier = 50,
#     escape_roads_inset_x_m = 50,
#     escape_roads_inset_y_m = 50,
#     road_aware_escape_zone = TRUE
#   )
# )

## ----render-fast, eval = FALSE------------------------------------------------
# rmarkdown::render(
#   "vignettes/diagnostic-example.Rmd",
#   params = list(
#     run_diagnostics = TRUE,
#     run_lcp = FALSE,
#     data_dir = "data-raw"
#   )
# )

## ----setup--------------------------------------------------------------------
# This loader works both when the package is installed and during local package development.
if (requireNamespace("evacpath", quietly = TRUE)) {
  library(evacpath)
} else if (requireNamespace("devtools", quietly = TRUE) && file.exists("DESCRIPTION")) {
  devtools::load_all(".")
} else if (requireNamespace("devtools", quietly = TRUE) && file.exists(file.path("evacpath", "DESCRIPTION"))) {
  devtools::load_all("evacpath")
} else {
  stop("Install evacpath or run this from the package root/parent directory.", call. = FALSE)
}

library(terra)

run_diagnostics <- isTRUE(params$run_diagnostics)
run_lcp <- isTRUE(params$run_lcp)

assert_true <- function(x, message) {
  if (!isTRUE(x)) {
    stop(message, call. = FALSE)
  }
  invisible(TRUE)
}

is_spatvector <- function(x) inherits(x, "SpatVector")
is_spatraster <- function(x) inherits(x, "SpatRaster")

count_features <- function(x) {
  if (inherits(x, "SpatRaster")) return(terra::ncell(x))
  if (inherits(x, "SpatVector")) return(nrow(x))
  NA_integer_
}

## ----switches-----------------------------------------------------------------
run_diagnostics
run_lcp

## ----locate-files-------------------------------------------------------------
find_input_file <- function(filename, data_dir = params$data_dir) {
  candidate_dirs <- unique(c(
    data_dir,
    file.path("evacpath", data_dir),
    file.path(getwd(), data_dir),
    file.path(getwd(), "evacpath", data_dir),
    system.file("extdata", package = "evacpath")
  ))

  candidate_dirs <- candidate_dirs[nzchar(candidate_dirs)]
  candidates <- file.path(candidate_dirs, filename)
  hits <- candidates[file.exists(candidates)]

  if (length(hits) == 0L) {
    return(NA_character_)
  }

  normalizePath(hits[1])
}

dem_file <- find_input_file("dem.tif")
roads_file <- find_input_file("rds.gpkg")
inundation_file <- find_input_file("tsunami_inundation_depth.tif")

input_files <- data.frame(
  layer = c("DEM", "roads", "inundation depth"),
  file = c(dem_file, roads_file, inundation_file),
  exists = file.exists(c(dem_file, roads_file, inundation_file))
)

input_files

example_data_available <- all(input_files$exists)
example_data_available

## ----read-inputs, eval = run_diagnostics && example_data_available------------
# dem_raw <- read_spatial(dem_file)
# roads_raw <- read_spatial(roads_file)
# inundation_raw <- read_spatial(inundation_file)
# 
# assert_true(is_spatraster(dem_raw), "DEM was not read as a SpatRaster.")
# assert_true(is_spatvector(roads_raw), "Roads were not read as a SpatVector.")
# assert_true(is_spatraster(inundation_raw), "Inundation input was not read as a SpatRaster.")
# 
# list(
#   dem_class = class(dem_raw),
#   roads_class = class(roads_raw),
#   inundation_class = class(inundation_raw)
# )

## ----raw-diagnostics, eval = run_diagnostics && example_data_available--------
# cat("DEM CRS:\n", terra::crs(dem_raw), "\n\n")
# cat("Roads CRS:\n", terra::crs(roads_raw), "\n\n")
# cat("Inundation CRS:\n", terra::crs(inundation_raw), "\n\n")
# 
# cat("DEM resolution:\n")
# print(terra::res(dem_raw))
# 
# cat("Inundation resolution:\n")
# print(terra::res(inundation_raw))
# 
# cat("DEM cell count:\n")
# print(terra::ncell(dem_raw))
# 
# cat("Road feature count:\n")
# print(nrow(roads_raw))
# 
# cat("Road attribute names:\n")
# print(names(roads_raw))
# 
# cat("DEM range:\n")
# print(terra::global(dem_raw, fun = range, na.rm = TRUE))
# 
# cat("Inundation range:\n")
# print(terra::global(inundation_raw, fun = range, na.rm = TRUE))

## ----plot-raw-inputs, eval = run_diagnostics && example_data_available, fig.height = 7----
# oldpar <- par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
# plot(dem_raw, main = "Raw DEM")
# plot(inundation_raw, main = "Raw inundation depth")
# par(oldpar)

## ----clean-roads, eval = run_diagnostics && example_data_available------------
# road_exclude <- NULL
# if (!is.null(params$road_exclude_field) && nzchar(params$road_exclude_field)) {
#   road_exclude <- list(
#     field = params$road_exclude_field,
#     values = params$road_exclude_values
#   )
# }
# 
# roads_clean <- clean_roads(
#   roads = roads_raw,
#   exclude = road_exclude,
#   target_crs = params$target_crs
# )
# 
# assert_true(is_spatvector(roads_clean), "clean_roads() did not return a SpatVector.")
# assert_true(nrow(roads_clean) > 0, "clean_roads() returned no roads.")
# 
# nrow(roads_raw)
# nrow(roads_clean)

## ----prepare-tsunami-zones, eval = run_diagnostics && example_data_available----
# zones <- prepare_tsunami_zones(
#   inundation = inundation_raw,
#   dem = dem_raw,
#   target_crs = params$target_crs,
#   inundation_threshold = params$inundation_threshold,
#   land_threshold = params$land_threshold,
#   water_threshold = params$water_threshold,
#   dem_sign_multiplier = params$dem_sign_multiplier,
#   resample_method = params$resample_method,
#   as_polygon = TRUE,
#   dissolve = TRUE
# )
# 
# zones
# 
# assert_true(is_spatvector(zones$hazard_zone), "zones$hazard_zone is not a SpatVector.")
# assert_true(is_spatvector(zones$escape_zone), "zones$escape_zone is not a SpatVector.")
# assert_true(is_spatraster(zones$hazard_raster), "zones$hazard_raster is not a SpatRaster.")
# assert_true(is_spatraster(zones$escape_raster), "zones$escape_raster is not a SpatRaster.")
# 
# cat("Land-only hazard-zone polygons:", nrow(zones$hazard_zone), "\n")
# cat("Water-combined escape-zone polygons:", nrow(zones$escape_zone), "\n")

## ----land-water-check, eval = run_diagnostics && example_data_available, fig.height = 7----
# oldpar <- par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
# plot(zones$land_mask, col = "grey85", legend = FALSE, main = "DEM-derived land mask")
# plot(zones$water_mask, col = "lightskyblue1", legend = FALSE, main = "DEM-derived water mask")
# par(oldpar)

## ----plot-tsunami-zones, eval = run_diagnostics && example_data_available, fig.height = 7----
# oldpar <- par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
# plot(zones$hazard_zone, col = "tomato", border = NA, main = "hazard_zone: land-only TEZ")
# plot(zones$escape_zone, col = "grey90", border = NA, main = "escape_zone: TEZ + water")
# plot(zones$hazard_zone, add = TRUE, col = "tomato", border = NA)
# par(oldpar)

## ----crop-roads-inner-extent, eval = run_diagnostics && example_data_available----
# roads_for_escape <- crop_roads_to_inner_extent(
#   roads = roads_clean,
#   zone = zones$escape_zone,
#   inset_x_m = params$escape_roads_inset_x_m,
#   inset_y_m = params$escape_roads_inset_y_m
# )
# 
# assert_true(is_spatvector(roads_for_escape), "roads_for_escape is not a SpatVector.")
# assert_true(nrow(roads_for_escape) > 0, "roads_for_escape has no features.")
# 
# cat("Clean roads:", nrow(roads_clean), "\n")
# cat("Roads used for escape-point detection:", nrow(roads_for_escape), "\n")

## ----plot-roads-inner-extent, eval = run_diagnostics && example_data_available, fig.height = 7----
# plot(zones$escape_zone, col = "grey92", border = "grey60", main = "Roads cropped to inset extent for escape detection")
# plot(roads_clean, add = TRUE, col = adjustcolor("red", alpha.f = 0.25), lwd = 0.35)
# plot(roads_for_escape, add = TRUE, col = "black", lwd = 0.45)
# legend(
#   "topright",
#   legend = c("Original cleaned roads", "Roads used for escape detection"),
#   col = c(adjustcolor("red", alpha.f = 0.5), "black"),
#   lty = 1,
#   lwd = 1,
#   bty = "n",
#   cex = 0.8
# )

## ----road-aware-escape-zone, eval = run_diagnostics && example_data_available----
# escape_boundary_zone <- make_road_aware_escape_zone(
#   escape_zone = zones$escape_zone,
#   roads = roads_for_escape,
#   road_buffer_m = params$road_buffer_m,
#   crop_buffer_m = params$final_road_buffer_m,
#   include_base_zone = TRUE
# )
# 
# assert_true(is_spatvector(escape_boundary_zone), "escape_boundary_zone is not a SpatVector.")
# assert_true(nrow(escape_boundary_zone) > 0, "escape_boundary_zone has no features.")
# 
# nrow(escape_boundary_zone)

## ----plot-road-aware-zone, eval = run_diagnostics && example_data_available, fig.height = 7----
# plot(escape_boundary_zone, col = "grey90", border = NA, main = "Road-aware escape-boundary zone")
# plot(zones$hazard_zone, add = TRUE, col = adjustcolor("tomato", alpha.f = 0.45), border = NA)
# plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.45), lwd = 0.35)

## ----compare-escape-points, eval = run_diagnostics && example_data_available----
# escape_wrong <- try(
#   find_escape_points(
#     hazard_zone = zones$hazard_zone,
#     roads = roads_clean
#   ),
#   silent = TRUE
# )
# 
# escape_water <- find_escape_points(
#   hazard_zone = zones$escape_zone,
#   roads = roads_for_escape
# )
# 
# escape_points <- find_escape_points(
#   hazard_zone = escape_boundary_zone,
#   roads = roads_for_escape
# )
# 
# comparison <- data.frame(
#   method = c(
#     "wrong: land-only hazard zone",
#     "better: TEZ + water, inset roads",
#     "preferred: road-aware escape zone, inset roads"
#   ),
#   escape_point_count = c(
#     if (inherits(escape_wrong, "try-error")) NA_integer_ else nrow(escape_wrong),
#     nrow(escape_water),
#     nrow(escape_points)
#   )
# )
# comparison
# 
# assert_true(nrow(escape_points) > 0, "Preferred escape-point calculation returned no points.")

## ----plot-escape-comparison, eval = run_diagnostics && example_data_available, fig.height = 8----
# oldpar <- par(mfrow = c(1, 3))
# 
# plot(zones$hazard_zone, col = "grey90", border = "grey40", main = "Wrong\nland-only boundary")
# plot(roads_clean, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
# if (!inherits(escape_wrong, "try-error")) {
#   plot(escape_wrong, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
# }
# 
# plot(zones$escape_zone, col = "grey90", border = "grey40", main = "Better\nTEZ + water")
# plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
# plot(escape_water, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
# 
# plot(escape_boundary_zone, col = "grey90", border = "grey40", main = "Preferred\nroad-aware boundary")
# plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
# plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
# 
# par(oldpar)

## ----assign-core-objects, eval = run_diagnostics && example_data_available----
# hazard_zone <- zones$hazard_zone
# escape_zone <- zones$escape_zone
# dem <- zones$dem
# roads <- roads_clean

## ----prepare-core-inputs, eval = run_diagnostics && example_data_available----
# inputs <- prepare_evac_inputs(
#   hazard_zone = hazard_zone,
#   roads = roads,
#   dem = dem,
#   target_crs = params$target_crs,
#   hazard_as_polygon = TRUE,
#   dissolve_hazard = TRUE,
#   road_exclude = NULL
# )
# 
# hazard_zone <- inputs$hazard_zone
# roads <- inputs$roads
# dem <- inputs$dem
# 
# assert_true(is_spatvector(hazard_zone), "Prepared hazard_zone is not a SpatVector.")
# assert_true(is_spatvector(roads), "Prepared roads are not a SpatVector.")
# assert_true(is_spatraster(dem), "Prepared DEM is not a SpatRaster.")

## ----make-grid, eval = run_diagnostics && example_data_available--------------
# grid_resolution <- terra::res(dem) * params$grid_multiplier
# 
# evac_grid <- make_evac_grid(
#   hazard_zone = hazard_zone,
#   resolution = grid_resolution
# )
# 
# assert_true(is_spatvector(evac_grid), "make_evac_grid() did not return a SpatVector.")
# assert_true(nrow(evac_grid) > 0, "make_evac_grid() returned no grid cells.")
# 
# cat("Grid resolution:", paste(grid_resolution, collapse = " x "), "\n")
# cat("Grid cells:", nrow(evac_grid), "\n")

## ----plot-grid, eval = run_diagnostics && example_data_available--------------
# plot(hazard_zone, col = "grey95", border = "grey50", main = "Evacuation grid over land-only TEZ")
# plot(evac_grid, add = TRUE, border = adjustcolor("black", alpha.f = 0.25), col = NA)
# plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.45), lwd = 0.35)

## ----make-road-mask, eval = run_diagnostics && example_data_available---------
# road_mask_parts <- make_road_mask(
#   roads = roads,
#   escape_points = escape_points,
#   road_buffer_m = params$road_buffer_m,
#   escape_buffer_m = params$escape_buffer_m,
#   return_components = TRUE
# )
# 
# road_mask <- road_mask_parts$mask
# roads_buffer <- road_mask_parts$roads_buffer
# escape_buffer <- road_mask_parts$escape_buffer
# 
# assert_true(is_spatvector(road_mask), "Road mask is not a SpatVector.")
# assert_true(is_spatvector(roads_buffer), "roads_buffer is not a SpatVector.")
# assert_true(is_spatvector(escape_buffer), "escape_buffer is not a SpatVector.")

## ----plot-road-mask, eval = run_diagnostics && example_data_available---------
# plot(hazard_zone, col = "grey95", border = "grey60", main = "Road-constrained movement mask")
# plot(road_mask, add = TRUE, col = adjustcolor("grey30", alpha.f = 0.35), border = NA)
# plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

## ----make-road-origins, eval = run_diagnostics && example_data_available------
# road_points <- make_road_origins(
#   evac_grid = evac_grid,
#   roads_buffer = roads_buffer,
#   hazard_zone = hazard_zone,
#   max_origins = params$max_origins,
#   seed = params$seed
# )
# 
# assert_true(is_spatvector(road_points), "make_road_origins() did not return a SpatVector.")
# assert_true(nrow(road_points) > 0, "make_road_origins() returned no origins.")
# 
# cat("Road origin points retained:", nrow(road_points), "\n")

## ----plot-road-origins, eval = run_diagnostics && example_data_available------
# plot(hazard_zone, col = "grey95", border = "grey60", main = "Road origin points inside land-only TEZ")
# plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
# plot(road_points, add = TRUE, pch = 21, bg = "dodgerblue", col = "black", cex = 0.45)
# plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

## ----make-conductance, eval = run_diagnostics && example_data_available-------
# dem_resolution <- terra::res(dem) * params$dem_multiplier
# 
# conductance <- make_conductance_surface(
#   dem = dem,
#   road_mask = road_mask,
#   resolution = dem_resolution
# )
# 
# cs_raster <- leastcostpath::rasterise(conductance)
# 
# assert_true(is_spatraster(cs_raster), "Conductance rasterization failed.")
# 
# cat("Conductance raster resolution:", paste(terra::res(cs_raster), collapse = " x "), "\n")
# cat("Conductance raster cells:", terra::ncell(cs_raster), "\n")

## ----plot-conductance, eval = run_diagnostics && example_data_available-------
# plot(cs_raster, main = "Road-constrained conductance surface")
# plot(hazard_zone, add = TRUE, border = "black", col = NA)
# plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 1), lwd = 0.35)
# plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

## ----sample-destinations, eval = run_diagnostics && example_data_available----
# escape_points_ltd <- escape_points
# if (!is.null(params$max_destinations) && params$max_destinations < nrow(escape_points_ltd)) {
#   set.seed(params$seed)
#   escape_points_ltd <- escape_points_ltd[sort(sample.int(nrow(escape_points_ltd), params$max_destinations)), ]
# }
# 
# cat("Escape points available:", nrow(escape_points), "\n")
# cat("Escape points used in diagnostic LCP:", nrow(escape_points_ltd), "\n")

## ----calc-distance, eval = run_diagnostics && example_data_available && run_lcp----
# distance_points <- calc_min_distance_to_safety(
#   cs = conductance,
#   origins = road_points,
#   destinations = escape_points_ltd,
#   include_destinations = TRUE,
#   progress = TRUE,
#   progress_every = 500,
#   check_locations = FALSE
# )
# 
# assert_true(is_spatvector(distance_points), "calc_min_distance_to_safety() did not return a SpatVector.")
# assert_true("distance" %in% names(distance_points), "distance_points does not contain a distance field.")
# 
# summary(distance_points$distance)
# table(distance_points$type)

## ----plot-distance-points, eval = run_diagnostics && example_data_available && run_lcp----
# plot(hazard_zone, col = "grey95", border = "grey60", main = "Minimum least-cost distance points")
# plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.30), lwd = 0.35)
# plot(distance_points, "distance", add = TRUE)

## ----calc-time, eval = run_diagnostics && example_data_available && run_lcp----
# example_distances <- c(0, 100, 500, 1000, 2000)
# data.frame(
#   distance_m = example_distances,
#   time_min = calc_evac_time(
#     distance_m = example_distances,
#     walking_speed_mps = params$walking_speed_mps,
#     units = "minutes"
#   )
# )

## ----make-polygons, eval = run_diagnostics && example_data_available && run_lcp----
# evac_polygons <- make_evac_polygons(
#   distance_points = distance_points,
#   clip_area = hazard_zone,
#   walking_speed_mps = params$walking_speed_mps,
#   region_name = params$region_name
# )
# 
# assert_true(is_spatvector(evac_polygons), "make_evac_polygons() did not return a SpatVector.")
# assert_true("EvacTimeAvg" %in% names(evac_polygons), "EvacTimeAvg field is missing.")
# 
# summary(evac_polygons$EvacTimeAvg)

## ----plot-evac-polygons, eval = run_diagnostics && example_data_available && run_lcp, fig.height = 7----
# plot(evac_polygons, "EvacTimeAvg", border = NA, main = "Evacuation-time polygons")
# plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.30), lwd = 0.35)
# plot(escape_points_ltd, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

## ----run-wrapper, eval = run_diagnostics && example_data_available && run_lcp, message=FALSE,echo=FALSE, eval = FALSE----
# result <- run_evacpath(
#   hazard_zone = zones$hazard_zone,
#   escape_zone = zones$escape_zone,
#   roads = roads_clean,
#   dem = zones$dem,
#   target_crs = params$target_crs,
#   region_name = params$region_name,
#   grid_resolution = terra::res(zones$dem) * params$grid_multiplier,
#   dem_resolution = terra::res(zones$dem) * params$dem_multiplier,
#   road_buffer_m = params$road_buffer_m,
#   escape_buffer_m = params$escape_buffer_m,
#   final_road_buffer_m = params$final_road_buffer_m,
#   escape_roads_inset_x_m = params$escape_roads_inset_x_m,
#   escape_roads_inset_y_m = params$escape_roads_inset_y_m,
#   road_aware_escape_zone = params$road_aware_escape_zone,
#   escape_zone_road_buffer_m = params$road_buffer_m,
#   escape_zone_crop_buffer_m = params$final_road_buffer_m,
#   max_origins = params$max_origins,
#   max_destinations = params$max_destinations,
#   seed = params$seed,
#   walking_speed_mps = params$walking_speed_mps,
#   clip_mode = "hazard",
#   progress = TRUE,
#   progress_every = 500,
#   lcp_check_locations = FALSE
# )
# 
# result
# 
# assert_true("escape_boundary_zone" %in% names(result), "Result does not contain escape_boundary_zone.")
# assert_true("roads_for_escape" %in% names(result), "Result does not contain roads_for_escape.")
# assert_true("time_grid" %in% names(result), "Result does not contain time_grid.")
# assert_true("EvacTimeAvg" %in% names(result$time_grid), "Result time_grid lacks EvacTimeAvg.")

## ----final-map, eval = run_diagnostics && example_data_available && run_lcp, fig.width = 9, fig.height = 7, eval = FALSE----
# evac_map <- result$time_grid
# roads_plot <- crop(result$roads, evac_map)
# escape_plot <- crop(result$escape_points, evac_map)
# 
# dem_bg <- result$dem
# if (!same.crs(dem_bg, evac_map)) {
#   dem_bg <- project(dem_bg, crs(evac_map))
# }
# 
# e <- ext(evac_map)
# xpad <- (xmax(e) - xmin(e)) * 0.06
# ypad <- (ymax(e) - ymin(e)) * 0.06
# e2 <- ext(xmin(e) - xpad, xmax(e) + xpad, ymin(e) - ypad, ymax(e) + ypad)
# 
# dem_bg <- crop(dem_bg, e2)
# land_water <- ifel(dem_bg > params$land_threshold, 2, 1)
# names(land_water) <- "land_water"
# 
# bg_cols <- c(
#   adjustcolor("lightskyblue1", alpha.f = 0.65),
#   adjustcolor("grey92", alpha.f = 1.00)
# )
# time_cols <- hcl.colors(100, "YlOrRd", rev = TRUE)
# 
# oldpar <- par(mar = c(4, 4, 3, 5), mgp = c(2.2, 0.7, 0), las = 1, bg = "white", xpd = FALSE)
# 
# plot(
#   land_water,
#   ext = e2,
#   col = bg_cols,
#   legend = FALSE,
#   axes = TRUE,
#   box = FALSE,
#   main = "Modeled Pedestrian Evacuation Time",
#   cex.main = 1.1
# )
# 
# plot(
#   evac_map,
#   "EvacTimeAvg",
#   add = TRUE,
#   col = time_cols,
#   border = NA,
#   plg = list(title = "Minutes", cex = 0.75, title.cex = 0.85, x = "right")
# )
# 
# plot(roads_plot, add = TRUE, col = adjustcolor("grey25", alpha.f = 0.30), lwd = 0.30)
# plot(escape_plot, add = TRUE, pch = 22, bg = adjustcolor("red", alpha.f = 0.75), col = "black", cex = 0.38, lwd = 0.35)
# 
# legend(
#   "topright",
#   legend = c("Escape / safety points", "Roads", "Land", "Water"),
#   pch = c(22, NA, 15, 15),
#   pt.bg = c("red", NA, "grey92", "lightskyblue1"),
#   col = c("black", adjustcolor("grey25", alpha.f = 0.5), "grey92", "lightskyblue1"),
#   lty = c(NA, 1, NA, NA),
#   lwd = c(NA, 1, NA, NA),
#   pt.cex = c(0.8, NA, 1.1, 1.1),
#   bty = "n",
#   cex = 0.7,
#   inset = 0.02
# )
# par(oldpar)

## ----write-outputs, eval = run_diagnostics && example_data_available && run_lcp, eval = FALSE----
# write_evac_outputs(
#   result,
#   output_dir = params$output_dir,
#   prefix = "example"
# )
# 
# list.files(params$output_dir, full.names = TRUE)

### Function: Evacuation Grid
### This function creates an evacuation grid from the evacuation zone

# requires:
# library: terra

### area.evac <- the evacuation zone (zono de desalojo)
### res <- the resoution of each evacuation grid

evacuation_grid <- function(area.evac, res){
  grid.r <- rast(ext=ext(area.evac),crs=crs(area.evac),resolution=res)
  grid.p <- as.polygons(grid.r)
  
  # municipalities <- sort(unique(area.evac$Municipio)) 
  # grid <- crop(grid.p, area.evac,ext=TRUE)
  grid <- mask(grid.p, area.evac)

  return(grid)
}

library(terra)
tsunami <- rast("_data/raw/inundation.nc")
dem <- rast("_data/raw/topo.nc")
roads <- vect("_data/raw/road.shp")


dem <- -dem
land <- ifel(dem > 0, 1, NA)
water <- ifel(dem < 0, 1, NA)
tsunami <- resample(tsunami, land)
tsunami <- mask(tsunami, land)
tez <- ifel(tsunami > 0, 1, NA)
tez_vector <- as.polygons(tez)

tez <- mosaic(tez,water)

plot(water, col = "lightblue", legend=FALSE,box=FALSE,axes=FALSE)
plot(land, col="darkgreen",add=TRUE, legend=FALSE)
plot(tez_vector, col="red", add=TRUE, legend=FALSE)
plot(roads,add=TRUE)

roads <- roads[!(roads$man_made %in% "pier"),]
# roads_locs <- as.points(roads)

writeRaster(tsunami, "_data/tsunami_inundation_depth.tif", overwrite=TRUE)
writeRaster(tez, "_data/tsunami_inundation_zone.tif", overwrite=TRUE)
writeRaster(dem,"_data/dem.tif", overwrite=TRUE)
writeRaster(land,"_data/land.tif", overwrite=TRUE)
writeVector(roads,"_data/rds.gpkg", overwrite=TRUE)
# writeVector(roads_locs,"_data/rds_locs.gpkg", overwrite=TRUE)
writeVector(tez_vector,"_data/tez.gpkg",overwrite=TRUE)

# Generate simple README figures from the packaged example data.
#
# The packaged rasters are stored in lon/lat, but the README figures use the
# projected workflow CRS. The requested display window is interpreted as UTM
# easting/northing coordinates:
#   x: 669000 to 675000
#   y: 9223000 to 9225000

library(terra)

if (file.exists("DESCRIPTION") && requireNamespace("devtools", quietly = TRUE)) {
  devtools::load_all(".", quiet = TRUE)
} else {
  library(evacpath)
}

target_crs <- "EPSG:32748"
readme_extent <- ext(669000, 675000, 9223000, 9225000)
output_dir <- file.path("man", "figures")

dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)

focus_area <- as.polygons(readme_extent, crs = target_crs)

dem_raw <- rast(system.file("extdata/dem.tif", package = "evacpath"))
roads_raw <- vect(system.file("extdata/rds.gpkg", package = "evacpath"))
inundation_raw <- rast(system.file("extdata/tsunami_inundation_depth.tif", package = "evacpath"))

# Crop the raw source data first, before preparing zones or running the model.
focus_area_source <- project(focus_area, crs(dem_raw))
dem <- crop(dem_raw, focus_area_source)
inundation <- crop(inundation_raw, focus_area_source)
roads_raw <- crop(roads_raw, focus_area_source)

zones <- prepare_tsunami_zones(
  inundation = inundation,
  dem = dem,
  target_crs = target_crs,
  inundation_threshold = 0,
  dem_sign_multiplier = 1,
  as_polygon = TRUE,
  dissolve = TRUE
)

roads <- clean_roads(
  roads_raw,
  exclude = list(field = "man_made", values = "pier"),
  target_crs = target_crs
)

hazard_zone <- crop(zones$hazard_zone, readme_extent)
escape_zone <- crop(zones$escape_zone, readme_extent)
dem <- crop(zones$dem, readme_extent)
roads <- crop(roads, readme_extent)

roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads,
  zone = escape_zone,
  inset_x_m = 0,
  inset_y_m = 300
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 2,
  crop_buffer_m = 3
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

escape_points_window <- crop(escape_points, readme_extent)

local({
  png(
    filename = file.path(output_dir, "readme-example-inputs.png"),
    width = 1400,
    height = 700,
    res = 180
  )
  oldpar <- par(no.readonly = TRUE)
  on.exit({
    par(oldpar)
    dev.off()
  })

  par(mar = c(3, 3, 3, 1), mgp = c(1.8, 0.6, 0), las = 1, bg = "white")
  plot(
    escape_zone,
    ext = readme_extent,
    col = "grey94",
    border = "grey70",
    main = "Example Hazard Zone, Roads, and Escape Points",
    cex.main = 1.1,
    axes = FALSE
  )
  plot(hazard_zone, add = TRUE, col = "#f28e2b99", border = NA)
  plot(roads, add = TRUE, col = "#333333", lwd = 0.5)
  plot(escape_points_window, add = TRUE, pch = 22, bg = "#e15759", col = "black", cex = 0.7)
})

result <- run_evacpath(
  hazard_zone = hazard_zone,
  escape_zone = escape_zone,
  roads = roads_for_escape,
  dem = dem,
  target_crs = target_crs,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,
  escape_roads_inset_x_m = 0,
  escape_roads_inset_y_m = 0,
  road_aware_escape_zone = TRUE,
  max_origins = 300,
  max_destinations = 100,
  seed = 23401,
  walking_speed_mps = 1.22,
  clip_mode = "hazard",
  progress = FALSE
)

time_window <- crop(result$time_grid, readme_extent)
result_roads_window <- crop(result$roads, readme_extent)
result_escape_window <- crop(result$escape_points, readme_extent)

local({
  png(
    filename = file.path(output_dir, "readme-example-time.png"),
    width = 1400,
    height = 700,
    res = 180
  )
  oldpar <- par(no.readonly = TRUE)
  on.exit({
    par(oldpar)
    dev.off()
  })

  par(mar = c(3, 3, 3, 5), mgp = c(1.8, 0.6, 0), las = 1, bg = "white")
  plot(
    time_window,
    "EvacTimeAvg",
    axes = FALSE,
    ext = readme_extent,
    col = hcl.colors(5, "YlOrRd", rev = TRUE),
    border = NA,
    main = "Example Modeled Evacuation Time",
    cex.main = 1.1,
    plg = list(title = "Minutes", cex = 0.8, title.cex = 0.85)
  )
  plot(result_roads_window, add = TRUE, col = "#33333380", lwd = 0.35)
  plot(escape_points_window, add = TRUE, pch = 22, bg = "#e15759", col = "black", cex = 0.65)
})

funcs <- c(
  "_scripts/00_evacuation_grid.R",
  "_scripts/01_region_area.R",
  "_scripts/02_escape_points.R",
  "_scripts/03_calc_lc_path.R",
  "_scripts/03_calc_min_distance.R",
  "_scripts/03_minimum_distance.R",
  "_scripts/04_distance_grid.R",
  "_scripts/05_pea.R"
)
for (i in funcs){
  source(i)
}

library(terra)
library(leastcostpath)

tez <- vect("_data/tez.gpkg")
inundation <- rast("_data/tsunami_inundation_zone.tif")
inundation <- as.polygons(inundation)
dem <- rast("_data/dem.tif")
land <- rast("_data/land.tif")
roads <- vect("_data/rds.gpkg")
road_locs <- vect("_data/rds_locs.gpkg")

crs_jak <- "EPSG:32748"
tez <- project(tez,crs_jak)
inundation <- project(inundation,crs_jak)
dem <- project(dem,crs_jak)
land <- project(land,crs_jak)
roads <- project(roads,crs_jak)
road_locs <- project(road_locs, crs_jak)

working_res <- res(dem)*5

evac_grid <- evacuation_grid(inundation,working_res)
writeVector(evac_grid,"_data/evac_grid.gpkg", overwrite=TRUE)

escape_locs <- escape_points(inundation,roads)

plot(evac_grid,col="gray")
plot(tez,col="blue",add=TRUE)
plot(roads,add=TRUE)
plot(escape_locs,col="red",add=TRUE)
writeVector(escape_locs,"_data/evac_points.gpkg", overwrite=TRUE)


# create escape areas within 5 meters of the escape points
escape_buffer <- buffer(escape_locs,5)
escape_buffer <- aggregate(escape_buffer)

roads_buffer <- buffer(roads, 2)
roads_buffer <- aggregate(roads_buffer)

roads_mask <- vect(c(roads_buffer,escape_buffer))
roads_mask <- aggregate(roads_mask)
writeVector(roads_mask,"_data/dem_roads_mask_lyr.gpkg",overwrite=TRUE)

# merge the escape area to the roads
dem_roads <- mask(dem,roads_mask)

# conductance matrix for least cost path analysis
cs <- create_slope_cs(dem_roads)

# create a grid evac zone
# convert the road network into points
road_points <- mask(evac_grid,roads_buffer)
road_points <- intersect(road_points, roads_buffer)
road_points <- as.points(as.lines(road_points))
writeVector(road_points,"_data/rds_locs_analysis.gpkg", overwrite=TRUE)

extract a random sample from the road points
fraction <- 2000
if(fraction < length(road_points)){
  set.seed(23401)
  road_points_ltd <- sample(road_points,fraction)
  print("Fraction used.")
}


# least cost path analysis
min_dist_points <- min_dist(cs, road_points_ltd, escape_locs)

# crop by the study area
# min_dist_points <- crop(min_dist_points,buffer(area.study,10))

municipality <- "Jakarta"
# voronai polygons
v <- voronoi(min_dist_points)
v$time <- as.numeric(v$distance) * (1/1.22) * (1/60)
v$muni <- municipality
v <- v[,c('muni','distance','time')]
names(v) <- c("Municipio","DistToSafety","EvacTimeAvg")

roads_inundation <- crop(buffer(roads_buffer,3),inundation)
roads_inundation <- vect(c(roads_inundation,tez))
roads_inundation <- aggregate(roads_inundation)
writeVector(roads_inundation, "_data/tsunami_inundation_zone_final.gpkg", overwrite=TRUE)

v <- crop(v,roads_inundation)

plot(v,"DistToSafety")
plot(v,"EvacTimeAvg")

writeVector(min_dist_points,"_data/jakartaPEAT_points.gpkg", overwrite=TRUE)
writeVector(v, "_data/jakartaPEAT_voronai.gpkg", overwrite=TRUE)

### Function: Minimum Distance
### This function calculates the minimum 'distance to safety'

# requires:
# library: terra, leastcostpath

# cs <- leastcostpath conductance matrix
# origins <- road points
# destinations <- escape points/locations


min_dist <- function(cs, origins, destinations){
  
  # first, make sure that the origin and destination are location within the same extent
  # as the conductance surface
  cs_rast <- rasterise(cs) # cs raster
  
  # the cs projection is the same, but changes format from the original dem projection
  # to avoid issues, reproject to the newly formatted cs crs
  origins <- project(origins,crs(cs_rast)) 
  destinations <- project(destinations,crs(cs_rast)) 
  
  # crop to cs extent
  destinations <- crop(destinations, cs_rast, ext=TRUE) 
  
  # loop through origins to get minimum distance from any destinations
  minDist <- data.frame() # empty df
  for (i in 1:length(origins)){
    # run the lcp analysis
      lc.paths <- create_lcp(cs,origin=origins[i],destinations)
      lc.paths <- perim(lc.paths) # return vector of distance
      lc.paths <- lc.paths[lc.paths != 0] # remove zero values
      lc.paths <- min(lc.paths) # calculate the minimum distance
      # add the results to minDist
      if (lc.paths != Inf){
        minDist[i,1:3] <- c(geom(origins[i])[,c('x','y')],lc.paths)
      }
  }

  # data cleaning
  minDist <- na.omit(minDist) # remove NA rows
  names(minDist) <- c('x','y','distance')
  minDist$type <- 'road'
  
  # add destinations to minDist (distance = 0)
  escapeDist <- data.frame(geom(destinations)[,c('x','y')],
                            distance = 0, type = 'escape')
  minDist <- rbind(minDist, escapeDist)
  
  # return a spatvector object
  minDist <- vect(minDist, geom=c("x", "y"), crs=crs(origins))
  
  return(minDist)
}

### Function: Distance Grid
### This function calculates a 'distance to safety' grid

# requires:
# library: terra, fields

# rds <- rds
# escape.pnts <- escape points/locations
# area.study <- study area
# area.region <- greater area
# res.1 <- low resolution (default = 100)
# res.2 <- high resolution (default = 1)
# crs <- coordinate reference system

distance_grid <- function(minDist.pnts,area.region,area.study,res.1,res.2){
  # crop roads to region area
  # rasterize the road points, using the distance attribute
  r.1 <- rast(resolution=res.1,crs=crs(minDist.pnts),extent=ext(area.region))
  dist.grid <- rasterize(minDist.pnts,r.1,'distance')

  # thin plate spline regression for kriging/interpolation
  xy <- data.frame(xyFromCell(dist.grid, 1:ncell(dist.grid)))
  v <- values(dist.grid)
  i <- !is.na(v)
  xy <- xy[i,]
  v <- v[i]
  tps <- Tps(xy, v)

  # needs to convert to default filetype (again)
  # this is required
  dist.grid <- rast(dist.grid)
  dist.grid <- terra::project(dist.grid,crs(minDist.pnts))

  # use model to predict values at all locations
  dist.grid <- interpolate(dist.grid, tps)
  
  # remove the values lower than 0, which are outside the study area
  dist.grid[dist.grid < 0] <- 0## Not run:
  
  # resample the dataset to project resolution
  r.2 <- rast(resolution=res.2,crs=crs(minDist.pnts),extent=ext(area.region))
  dist.grid <- resample(dist.grid,r.2)
  
  dist.grid <- crop(dist.grid,area.study)
  dist.grid <- mask(dist.grid,area.study)

  return(dist.grid)
}

#' Prepare a hazard zone from an inundation raster
#'
#' Converts an inundation raster to a binary hazard-zone raster or polygon using
#' a threshold. This is intentionally general so users can adapt it to tsunami,
#' flood, storm-surge, or other hazard layers.
#'
#' @param inundation A `SpatRaster` or path to a raster.
#' @param threshold Numeric threshold. Cells greater than `threshold` are treated
#'   as inside the hazard zone.
#' @param land_mask Optional `SpatRaster` or `SpatVector` used to mask the hazard
#'   zone to land.
#' @param target_crs Optional output CRS. Use a projected CRS in meters for later
#'   distance calculations.
#' @param as_polygon Logical. If `TRUE`, return a polygon hazard zone.
#' @param dissolve Logical. If `TRUE`, dissolve polygon pieces.
#' @return A binary `SpatRaster` or polygon `SpatVector`.
#' @examples
#' r <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5)
#' terra::values(r) <- c(rep(0, 12), rep(1, 13))
#' zone <- prepare_hazard_zone(r, threshold = 0, as_polygon = TRUE)
#' zone
#' @export
prepare_hazard_zone <- function(
  inundation,
  threshold = 0,
  land_mask = NULL,
  target_crs = NULL,
  as_polygon = TRUE,
  dissolve = TRUE
) {
  inundation <- read_spatial(inundation)

  if (!inherits(inundation, "SpatRaster")) {
    stop("`inundation` must be a raster or raster file path.", call. = FALSE)
  }

  if (!is.null(target_crs)) {
    inundation <- .project_to(inundation, target_crs)
  }

  hazard <- terra::ifel(inundation > threshold, 1, NA)

  if (!is.null(land_mask)) {
    land_mask <- read_spatial(land_mask)
    if (!is.null(target_crs)) {
      land_mask <- .project_to(land_mask, target_crs)
    }
    hazard <- terra::mask(hazard, land_mask)
  }

  if (isTRUE(as_polygon)) {
    hazard <- terra::as.polygons(hazard, dissolve = dissolve, na.rm = TRUE)
    if (dissolve) {
      hazard <- .safe_aggregate(hazard)
    }
  }

  hazard
}

#' Clean a road/pathway network
#'
#' Removes user-specified features from a road/pathway layer. This is useful for
#' excluding piers, tunnels, private ways, or other features that should not be
#' used in pedestrian evacuation modeling.
#'
#' @param roads A `SpatVector` or path to a vector layer.
#' @param exclude Optional named list with `field` and `values`, for example
#'   `list(field = "man_made", values = "pier")`.
#' @param target_crs Optional output CRS.
#' @return A cleaned `SpatVector`.
#' @examples
#' roads <- terra::vect(
#'   list(
#'     matrix(c(0, 0, 0, 1), ncol = 2, byrow = TRUE),
#'     matrix(c(1, 0, 1, 1), ncol = 2, byrow = TRUE)
#'   ),
#'   type = "lines",
#'   crs = "EPSG:3857"
#' )
#' roads$kind <- c("road", "pier")
#' clean_roads(roads, exclude = list(field = "kind", values = "pier"))
#' @export
clean_roads <- function(roads, exclude = NULL, target_crs = NULL) {
  roads <- read_spatial(roads)

  if (!inherits(roads, "SpatVector")) {
    stop("`roads` must be a vector layer or vector file path.", call. = FALSE)
  }

  if (!is.null(target_crs)) {
    roads <- .project_to(roads, target_crs)
  }

  if (!is.null(exclude)) {
    if (!all(c("field", "values") %in% names(exclude))) {
      stop("`exclude` must be a list with elements `field` and `values`.", call. = FALSE)
    }
    if (!.has_column(roads, exclude$field)) {
      warning("Road exclusion field not found: ", exclude$field, call. = FALSE)
    } else {
      values <- terra::values(roads)[[exclude$field]]
      keep <- !(values %in% exclude$values)
      keep[is.na(keep)] <- TRUE
      roads <- roads[keep, ]
    }
  }

  .stop_empty(roads, "Road network")
  roads
}

#' Read and project the core evacuation inputs
#'
#' @param hazard_zone Hazard/inundation zone as a `SpatRaster`, `SpatVector`, or file path.
#' @param roads Road/pathway network as a `SpatVector` or file path.
#' @param dem Elevation raster as a `SpatRaster` or file path.
#' @param target_crs Optional projected CRS in meters.
#' @param hazard_as_polygon Logical. Convert raster hazard zones to polygons.
#' @param dissolve_hazard Logical. Dissolve hazard polygon pieces.
#' @param road_exclude Optional road exclusion list passed to `clean_roads()`.
#' @return A named list with `hazard_zone`, `roads`, and `dem`.
#' @examples
#' dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
#'   vals = 1, crs = "EPSG:3857")
#' hazard <- terra::as.polygons(dem, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(0, 2.5, 5, 2.5), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' inputs <- prepare_evac_inputs(hazard, roads, dem)
#' names(inputs)
#' @export
prepare_evac_inputs <- function(
  hazard_zone,
  roads,
  dem,
  target_crs = NULL,
  hazard_as_polygon = TRUE,
  dissolve_hazard = TRUE,
  road_exclude = NULL
) {
  hazard_zone <- read_spatial(hazard_zone)
  roads <- clean_roads(roads, exclude = road_exclude)
  dem <- read_spatial(dem)

  if (!inherits(dem, "SpatRaster")) {
    stop("`dem` must be a raster or raster file path.", call. = FALSE)
  }

  if (!is.null(target_crs)) {
    hazard_zone <- .project_to(hazard_zone, target_crs)
    roads <- .project_to(roads, target_crs)
    dem <- .project_to(dem, target_crs)
  }

  if (isTRUE(hazard_as_polygon)) {
    hazard_zone <- .as_hazard_polygon(hazard_zone, dissolve = dissolve_hazard)
  }

  list(
    hazard_zone = hazard_zone,
    roads = roads,
    dem = dem,
    target_crs = target_crs
  )
}

test_that("clean_roads removes excluded road features", {
  roads <- terra::vect(
    list(
      matrix(c(0, 0, 0, 1), ncol = 2, byrow = TRUE),
      matrix(c(1, 0, 1, 1), ncol = 2, byrow = TRUE)
    ),
    type = "lines",
    crs = "EPSG:3857"
  )
  roads$kind <- c("road", "pier")

  cleaned <- clean_roads(roads, exclude = list(field = "kind", values = "pier"))

  expect_equal(nrow(cleaned), 1)
  expect_equal(terra::values(cleaned)$kind, "road")
})

#' evacpath: Least-cost pedestrian evacuation modeling in R
#'
#' `evacpath` provides tools for modeling pedestrian evacuation distance and
#' travel time from hazard zones to safety areas using road-constrained least-cost
#' path analysis.
#'
#' The package is organized as a set of modular functions. Typical workflows use
#' `prepare_tsunami_zones()` to create tsunami-specific hazard and escape zones,
#' `crop_roads_to_inner_extent()` and `make_road_aware_escape_zone()` to prevent
#' false escape points along artificial study-area boundaries while retaining
#' bridge and walkway corridors, `find_escape_points()` to identify candidate
#' exits, `make_conductance_surface()` to build a slope-based conductance surface,
#' and `run_evacpath()` to run the full workflow.
#'
#' @keywords package
"_PACKAGE"

#' Prepare separate tsunami zones for escape analysis and visualization
#'
#' Tsunami evacuation workflows often need two different zone objects. The
#' land-only inundation zone is the area where road origins and output time
#' surfaces should be mapped. The escape-boundary zone should combine the
#' land-only inundation zone with water so that the coastline is not treated as
#' an artificial escape boundary. This prevents false escape/safety points along
#' the water-land edge when roads touch or approach the shoreline.
#'
#' @param inundation Inundation-depth raster or path to a raster. Cells greater
#'   than `inundation_threshold` are treated as inundated.
#' @param dem Elevation/topobathymetry raster or path to a raster.
#' @param target_crs Optional projected CRS in meters for returned objects.
#' @param inundation_threshold Numeric threshold used to define inundated cells.
#' @param land_threshold Numeric DEM threshold used to define land. Default is
#'   `0`, so land is `dem > 0` after applying `dem_sign_multiplier`.
#' @param water_threshold Numeric DEM threshold used to define water. Default is
#'   `0`, so water is `dem < 0` after applying `dem_sign_multiplier`.
#' @param dem_sign_multiplier Multiplier applied to the DEM before land/water
#'   classification. Use `-1` when the DEM sign convention is reversed.
#' @param resample_method Method passed to `terra::resample()` when aligning the
#'   inundation raster to the DEM. Use `"bilinear"` for depth rasters and
#'   `"near"` for categorical rasters.
#' @param as_polygon Logical. If `TRUE`, return polygon zones in addition to
#'   rasters.
#' @param dissolve Logical. Dissolve polygon pieces.
#'
#' @return A named list with land-only `hazard_zone`, water-combined
#'   `escape_zone`, and supporting rasters. `hazard_zone` should usually be used
#'   for origin generation, mapping, and output clipping. `escape_zone` should
#'   usually be passed to `run_evacpath(escape_zone = ...)` or
#'   `find_escape_points()`.
#' @examples
#' dem <- terra::rast(nrows = 6, ncols = 6, xmin = 0, xmax = 6, ymin = 0, ymax = 6,
#'   crs = "EPSG:3857")
#' xy <- terra::crds(dem, df = TRUE)
#' terra::values(dem) <- -1.5 + 0.7 * xy$x + 0.2 * sin(xy$y)
#' inundation <- dem
#' terra::values(inundation) <- ifelse(terra::values(dem) > 0 & terra::values(dem) < 2.5, 1, 0)
#' zones <- prepare_tsunami_zones(inundation, dem, as_polygon = TRUE)
#' names(zones)
#' @export
prepare_tsunami_zones <- function(
  inundation,
  dem,
  target_crs = NULL,
  inundation_threshold = 0,
  land_threshold = 0,
  water_threshold = 0,
  dem_sign_multiplier = 1,
  resample_method = "bilinear",
  as_polygon = TRUE,
  dissolve = TRUE
) {
  inundation <- read_spatial(inundation)
  dem <- read_spatial(dem)

  if (!inherits(inundation, "SpatRaster")) {
    stop("`inundation` must be a SpatRaster or raster file path.", call. = FALSE)
  }
  if (!inherits(dem, "SpatRaster")) {
    stop("`dem` must be a SpatRaster or raster file path.", call. = FALSE)
  }

  dem_work <- dem * dem_sign_multiplier

  # Align the inundation raster to the DEM grid before masking by land/water.
  if (!terra::same.crs(inundation, dem_work)) {
    inundation <- terra::project(inundation, dem_work)
  }
  inundation <- terra::resample(inundation, dem_work, method = resample_method)

  land_mask <- terra::ifel(dem_work > land_threshold, 1, NA)
  water_mask <- terra::ifel(dem_work < water_threshold, 1, NA)

  inundation_land <- terra::mask(inundation, land_mask)
  hazard_land <- terra::ifel(inundation_land > inundation_threshold, 1, NA)

  # This is the important tsunami-specific step: combine the land inundation
  # footprint with water so the coast is not interpreted as a safety boundary.
  escape_raster <- terra::mosaic(hazard_land, water_mask)

  out <- list(
    hazard_raster = hazard_land,
    escape_raster = escape_raster,
    land_mask = land_mask,
    water_mask = water_mask,
    inundation_aligned = inundation,
    dem = dem_work
  )

  if (isTRUE(as_polygon)) {
    hazard_zone <- terra::as.polygons(hazard_land, dissolve = dissolve, na.rm = TRUE)
    escape_zone <- terra::as.polygons(escape_raster, dissolve = dissolve, na.rm = TRUE)

    if (dissolve) {
      hazard_zone <- .safe_aggregate(hazard_zone)
      escape_zone <- .safe_aggregate(escape_zone)
    }

    if (!is.null(target_crs)) {
      hazard_zone <- .project_to(hazard_zone, target_crs)
      escape_zone <- .project_to(escape_zone, target_crs)
    }

    out$hazard_zone <- hazard_zone
    out$escape_zone <- escape_zone
  }

  if (!is.null(target_crs)) {
    out$hazard_raster <- .project_to(out$hazard_raster, target_crs)
    out$escape_raster <- .project_to(out$escape_raster, target_crs)
    out$land_mask <- .project_to(out$land_mask, target_crs)
    out$water_mask <- .project_to(out$water_mask, target_crs)
    out$inundation_aligned <- .project_to(out$inundation_aligned, target_crs)
    out$dem <- .project_to(out$dem, target_crs)
  }

  class(out) <- c("evacpath_tsunami_zones", class(out))
  out
}

#' Create a buffered road area inside an inundation or analysis zone
#'
#' Mirrors the original script logic where roads were buffered, optionally
#' buffered again for tolerance, cropped to the inundation zone, and then
#' combined with the zone. This is useful for quality assurance and quality
#' control and for reproducing the earlier road-plus-inundation analysis area.
#'
#' @param roads Road/pathway network.
#' @param zone Polygon/raster zone used to crop buffered roads.
#' @param road_buffer_m First road buffer distance.
#' @param crop_buffer_m Optional second buffer applied before cropping.
#' @param include_zone Logical. If `TRUE`, combine the cropped road buffer with
#'   `zone` and dissolve the result.
#' @return A `SpatVector`.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' zone <- terra::as.polygons(r, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(0, 2, 4, 2), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' make_roads_in_zone(roads, zone, road_buffer_m = 0.1)
#' @export
make_roads_in_zone <- function(
  roads,
  zone,
  road_buffer_m = 2,
  crop_buffer_m = 3,
  include_zone = TRUE
) {
  roads <- read_spatial(roads)
  zone <- .as_hazard_polygon(read_spatial(zone), dissolve = TRUE)

  roads_buffer <- terra::buffer(roads, road_buffer_m)
  roads_buffer <- .safe_aggregate(roads_buffer)

  crop_buffer <- terra::buffer(roads_buffer, crop_buffer_m)
  roads_in_zone <- terra::crop(crop_buffer, zone)
  roads_in_zone <- .safe_aggregate(roads_in_zone)

  if (isTRUE(include_zone)) {
    roads_in_zone <- .combine_vectors(roads_in_zone, zone)
    roads_in_zone <- .safe_aggregate(roads_in_zone)
  }

  roads_in_zone
}

#' @export
print.evacpath_tsunami_zones <- function(x, ...) {
  cat("<evacpath_tsunami_zones>\n")
  cat("Use `hazard_zone` for land-only origins/output mapping.\n")
  cat("Use `escape_zone` for escape-point detection so coastline boundaries are not treated as safety exits.\n")
  invisible(x)
}

#' Run the full evacuation-path modeling workflow
#'
#' This high-level wrapper runs the core `evacpath` pipeline: read/project inputs,
#' create an evacuation grid, identify escape/safety points, build a road mask,
#' create a slope-based conductance surface, calculate minimum least-cost distance
#' to safety, and create evacuation-time polygons.
#'
#' For tsunami applications, `hazard_zone` and `escape_zone` should often be
#' different. Use a land-only `hazard_zone` for origins and output mapping, but
#' use a water-combined `escape_zone` for escape-point detection so the coastline
#' is not treated as an artificial safety boundary. The helper
#' `prepare_tsunami_zones()` creates both objects.
#'
#' @param hazard_zone Hazard/inundation zone as `SpatRaster`, `SpatVector`, or path.
#'   This should usually be the land-only area where evacuation origins and final
#'   output surfaces are mapped.
#' @param roads Road/pathway network as `SpatVector` or path.
#' @param dem Elevation raster as `SpatRaster` or path.
#' @param target_crs Optional projected CRS in meters, for example `"EPSG:32748"`.
#' @param region_name Optional region name stored in output polygons.
#' @param escape_zone Optional boundary zone used only to identify escape/safety
#'   points. For tsunami workflows, pass the land-inundation-plus-water zone from
#'   `prepare_tsunami_zones()` here. If `NULL`, `hazard_zone` is used.
#' @param roads_for_escape Optional road/pathway layer used only for escape-point
#'   detection. If `NULL`, `roads` is used. This is useful when the full road
#'   dataset extends beyond the reliable hazard-zone study extent.
#' @param escape_roads_inset_x_m Optional x-direction inset applied to
#'   `roads_for_escape` before escape-point detection. This prevents roads from
#'   intersecting artificial study-area extent boundaries.
#' @param escape_roads_inset_y_m Optional y-direction inset applied to
#'   `roads_for_escape` before escape-point detection.
#' @param road_aware_escape_zone Logical. If `TRUE`, buffered `roads_for_escape`
#'   are combined with `escape_zone` before escape points are generated. This
#'   preserves bridge, causeway, and walkway corridors over water that can be lost
#'   when the tsunami layer is split into land and water masks.
#' @param escape_zone_road_buffer_m Road buffer used when `road_aware_escape_zone = TRUE`.
#' @param escape_zone_crop_buffer_m Additional buffer used when `road_aware_escape_zone = TRUE`.
#' @param study_area Optional local study area for limiting escape-point search.
#' @param road_exclude Optional list passed to `clean_roads()`.
#' @param grid_resolution Evacuation-grid resolution. If `NULL`, calculated from
#'   `terra::res(dem) * grid_resolution_factor`.
#' @param grid_resolution_factor Multiplier applied to DEM resolution when
#'   `grid_resolution = NULL`.
#' @param road_buffer_m Road buffer distance.
#' @param escape_buffer_m Escape-point buffer distance.
#' @param final_road_buffer_m Output clipping buffer around roads.
#' @param region_buffer_m Buffer around `study_area` used when finding escape points.
#' @param dem_resolution Optional DEM resolution used before conductance creation.
#' @param max_origins Optional maximum number of road origin points.
#' @param max_destinations Optional maximum number of escape/safety destination points.
#'   This is useful for quick tests in regions where roads intersect the hazard
#'   boundary many times.
#' @param seed Random seed used when `max_origins` or `max_destinations` is supplied.
#' @param walking_speed_mps Walking speed in meters per second.
#' @param clip_mode Output clipping mode. The default `"hazard"` creates a
#'   continuous time-grid polygon surface clipped to the land-only hazard zone.
#'   Use `"road_hazard"` for the older road-buffer-limited output,
#'   `"hazard_plus_roads"` to retain both, or `"none"` for unclipped Voronoi polygons.
#' @param progress Logical. Print progress while running least-cost paths.
#' @param progress_every Integer. Print progress every `n` origins when `progress = TRUE`.
#' @param lcp_check_locations Logical passed to `leastcostpath::create_lcp()`.
#'   Default is `FALSE` for speed after projection/cropping checks.
#' @return An `evacpath_result` list containing spatial outputs and parameters.
#' @examples
#' dem <- terra::rast(nrows = 7, ncols = 7, xmin = 0, xmax = 7, ymin = 0, ymax = 7,
#'   vals = 1, crs = "EPSG:3857")
#' hazard_raster <- terra::crop(dem, terra::ext(1, 6, 1, 6))
#' hazard <- terra::as.polygons(hazard_raster, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(0, 3.5, 7, 3.5), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' result <- run_evacpath(
#'   hazard_zone = hazard,
#'   roads = roads,
#'   dem = dem,
#'   grid_resolution = 1,
#'   road_buffer_m = 0.2,
#'   escape_buffer_m = 0.3,
#'   final_road_buffer_m = 0.2,
#'   max_origins = 2,
#'   max_destinations = 2,
#'   seed = 1
#' )
#' result
#' @export
run_evacpath <- function(
  hazard_zone,
  roads,
  dem,
  target_crs = NULL,
  region_name = NULL,
  escape_zone = NULL,
  roads_for_escape = NULL,
  escape_roads_inset_x_m = 0,
  escape_roads_inset_y_m = 0,
  road_aware_escape_zone = FALSE,
  escape_zone_road_buffer_m = NULL,
  escape_zone_crop_buffer_m = NULL,
  study_area = NULL,
  road_exclude = NULL,
  grid_resolution = NULL,
  grid_resolution_factor = 5,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,
  region_buffer_m = 5000,
  dem_resolution = NULL,
  max_origins = NULL,
  max_destinations = NULL,
  seed = 23401,
  walking_speed_mps = 1.22,
  clip_mode = c("hazard", "road_hazard", "hazard_plus_roads", "none"),
  progress = FALSE,
  progress_every = 1L,
  lcp_check_locations = FALSE
) {
  clip_mode <- match.arg(clip_mode)

  inputs <- prepare_evac_inputs(
    hazard_zone = hazard_zone,
    roads = roads,
    dem = dem,
    target_crs = target_crs,
    hazard_as_polygon = TRUE,
    dissolve_hazard = TRUE,
    road_exclude = road_exclude
  )

  hazard_zone <- inputs$hazard_zone
  roads <- inputs$roads
  dem <- inputs$dem

  if (is.null(escape_zone)) {
    escape_zone <- hazard_zone
  } else {
    escape_zone <- read_spatial(escape_zone)
    if (!is.null(target_crs)) {
      escape_zone <- .project_to(escape_zone, target_crs)
    }
    escape_zone <- .as_hazard_polygon(escape_zone, dissolve = TRUE)
  }

  if (is.null(roads_for_escape)) {
    roads_for_escape <- roads
  } else {
    roads_for_escape <- read_spatial(roads_for_escape)
    if (!inherits(roads_for_escape, "SpatVector")) {
      stop("`roads_for_escape` must be a SpatVector or vector file path.", call. = FALSE)
    }
    if (!is.null(target_crs)) {
      roads_for_escape <- .project_to(roads_for_escape, target_crs)
    }
  }

  if (isTRUE(escape_roads_inset_x_m > 0 || escape_roads_inset_y_m > 0)) {
    roads_for_escape <- crop_roads_to_inner_extent(
      roads = roads_for_escape,
      zone = escape_zone,
      inset_x_m = escape_roads_inset_x_m,
      inset_y_m = escape_roads_inset_y_m
    )
  }

  escape_boundary_zone <- escape_zone
  if (isTRUE(road_aware_escape_zone)) {
    if (is.null(escape_zone_road_buffer_m)) {
      escape_zone_road_buffer_m <- road_buffer_m
    }
    if (is.null(escape_zone_crop_buffer_m)) {
      escape_zone_crop_buffer_m <- final_road_buffer_m
    }

    escape_boundary_zone <- make_road_aware_escape_zone(
      escape_zone = escape_zone,
      roads = roads_for_escape,
      road_buffer_m = escape_zone_road_buffer_m,
      crop_buffer_m = escape_zone_crop_buffer_m,
      include_base_zone = TRUE
    )
  }

  if (!is.null(study_area)) {
    study_area <- read_spatial(study_area)
    if (!is.null(target_crs)) {
      study_area <- .project_to(study_area, target_crs)
    }
  }

  if (is.null(grid_resolution)) {
    grid_resolution <- terra::res(dem) * grid_resolution_factor
  }

  evac_grid <- make_evac_grid(hazard_zone, resolution = grid_resolution)

  escape_points <- find_escape_points(
    hazard_zone = escape_boundary_zone,
    roads = roads_for_escape,
    study_area = study_area,
    region_buffer_m = region_buffer_m
  )

  if (!is.null(max_destinations)) {
    nd <- nrow(escape_points)
    if (!is.na(nd) && max_destinations < nd) {
      set.seed(seed)
      escape_points <- escape_points[sort(sample.int(nd, max_destinations)), ]
      if (isTRUE(progress)) {
        message("Using ", nrow(escape_points), " sampled escape/safety destinations.")
      }
    }
  }

  if (isTRUE(progress)) {
    message("Escape/safety destinations: ", nrow(escape_points))
  }

  road_mask_parts <- make_road_mask(
    roads = roads,
    escape_points = escape_points,
    road_buffer_m = road_buffer_m,
    escape_buffer_m = escape_buffer_m,
    return_components = TRUE
  )

  conductance <- make_conductance_surface(
    dem = dem,
    road_mask = road_mask_parts$mask,
    resolution = dem_resolution
  )

  road_points <- make_road_origins(
    evac_grid = evac_grid,
    roads_buffer = road_mask_parts$roads_buffer,
    hazard_zone = hazard_zone,
    max_origins = max_origins,
    seed = seed
  )

  if (isTRUE(progress)) {
    message("Road origins: ", nrow(road_points))
  }

  distance_points <- calc_min_distance_to_safety(
    cs = conductance,
    origins = road_points,
    destinations = escape_points,
    progress = progress,
    progress_every = progress_every,
    check_locations = lcp_check_locations
  )

  clip_area <- make_output_clip_area(
    hazard_zone = hazard_zone,
    roads_buffer = road_mask_parts$roads_buffer,
    final_road_buffer_m = final_road_buffer_m,
    clip_mode = clip_mode
  )

  evac_polygons <- make_evac_polygons(
    distance_points = distance_points,
    clip_area = clip_area,
    walking_speed_mps = walking_speed_mps,
    region_name = region_name
  )

  result <- list(
    region_name = region_name,
    target_crs = target_crs,
    parameters = list(
      grid_resolution = grid_resolution,
      grid_resolution_factor = grid_resolution_factor,
      road_buffer_m = road_buffer_m,
      escape_buffer_m = escape_buffer_m,
      escape_roads_inset_x_m = escape_roads_inset_x_m,
      escape_roads_inset_y_m = escape_roads_inset_y_m,
      road_aware_escape_zone = road_aware_escape_zone,
      escape_zone_road_buffer_m = escape_zone_road_buffer_m,
      escape_zone_crop_buffer_m = escape_zone_crop_buffer_m,
      final_road_buffer_m = final_road_buffer_m,
      region_buffer_m = region_buffer_m,
      dem_resolution = dem_resolution,
      max_origins = max_origins,
      max_destinations = max_destinations,
      seed = seed,
      walking_speed_mps = walking_speed_mps,
      clip_mode = clip_mode,
      progress_every = progress_every,
      lcp_check_locations = lcp_check_locations
    ),
    hazard_zone = hazard_zone,
    escape_zone = escape_zone,
    escape_boundary_zone = escape_boundary_zone,
    roads = roads,
    roads_for_escape = roads_for_escape,
    dem = dem,
    study_area = study_area,
    evac_grid = evac_grid,
    escape_points = escape_points,
    road_mask = road_mask_parts$mask,
    roads_buffer = road_mask_parts$roads_buffer,
    escape_buffer = road_mask_parts$escape_buffer,
    road_points = road_points,
    conductance = conductance,
    distance_points = distance_points,
    clip_area = clip_area,
    evac_polygons = evac_polygons,
    time_grid = evac_polygons
  )

  class(result) <- c("evacpath_result", class(result))
  result
}

#' Make an output clip area for evacuation polygons
#'
#' @param hazard_zone Hazard zone polygon.
#' @param roads_buffer Buffered roads.
#' @param final_road_buffer_m Additional road buffer for output clipping.
#' @param clip_mode One of `"hazard"`, `"road_hazard"`, `"hazard_plus_roads"`, or `"none"`.
#' @return A `SpatVector` clip area or `NULL`.
make_output_clip_area <- function(
  hazard_zone,
  roads_buffer,
  final_road_buffer_m = 3,
  clip_mode = c("hazard", "road_hazard", "hazard_plus_roads", "none")
) {
  clip_mode <- match.arg(clip_mode)

  if (clip_mode == "none") {
    return(NULL)
  }

  if (clip_mode == "hazard") {
    return(hazard_zone)
  }

  road_clip <- terra::buffer(roads_buffer, final_road_buffer_m)
  road_clip <- terra::crop(road_clip, hazard_zone)
  road_clip <- .safe_aggregate(road_clip)

  if (clip_mode == "road_hazard") {
    return(road_clip)
  }

  out <- .combine_vectors(road_clip, hazard_zone)
  .safe_aggregate(out)
}

#' @export
print.evacpath_result <- function(x, ...) {
  cat("<evacpath_result>\n")
  if (!is.null(x$region_name)) {
    cat("Region: ", x$region_name, "\n", sep = "")
  }
  if (!is.null(x$target_crs)) {
    cat("Target CRS: ", x$target_crs, "\n", sep = "")
  }
  cat("Outputs:\n")
  cat("  - hazard_zone (land-only output/origin zone)\n")
  cat("  - escape_zone (base escape zone)\n")
  cat("  - escape_boundary_zone (road-aware/inset-adjusted boundary used for escape points)\n")
  cat("  - roads_for_escape\n")
  cat("  - evac_grid\n")
  cat("  - escape_points\n")
  cat("  - road_points\n")
  cat("  - distance_points\n")
  cat("  - evac_polygons / time_grid\n")
  invisible(x)
}

#' Create a road-constrained analysis mask
#'
#' Buffers the road network and candidate escape points, combines those buffered
#' areas, and dissolves the result. The mask is used to constrain least-cost
#' movement to the road/pathway network while allowing access around escape
#' locations.
#'
#' @param roads Road/pathway network.
#' @param escape_points Candidate escape/safety points.
#' @param road_buffer_m Road buffer distance in map units, typically meters.
#' @param escape_buffer_m Escape-point buffer distance in map units, typically meters.
#' @param return_components Logical. If `TRUE`, return a list with the mask and
#'   component buffers.
#' @return A dissolved road mask `SpatVector`, or a list when `return_components = TRUE`.
#' @examples
#' roads <- terra::vect(matrix(c(0, 1, 4, 1), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' escape <- terra::vect(data.frame(x = 4, y = 1), geom = c("x", "y"), crs = "EPSG:3857")
#' make_road_mask(roads, escape, road_buffer_m = 0.1, escape_buffer_m = 0.2)
#' @export
make_road_mask <- function(
  roads,
  escape_points,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  return_components = FALSE
) {
  roads <- read_spatial(roads)
  escape_points <- read_spatial(escape_points)

  roads_buffer <- terra::buffer(roads, road_buffer_m)
  roads_buffer <- .safe_aggregate(roads_buffer)

  escape_buffer <- terra::buffer(escape_points, escape_buffer_m)
  escape_buffer <- .safe_aggregate(escape_buffer)

  mask <- .combine_vectors(roads_buffer, escape_buffer)
  mask <- .safe_aggregate(mask)

  if (isTRUE(return_components)) {
    return(list(
      mask = mask,
      roads_buffer = roads_buffer,
      escape_buffer = escape_buffer
    ))
  }

  mask
}

#' Create road-based origin points inside the evacuation zone
#'
#' Intersects an evacuation grid with a buffered road network and converts the
#' resulting road-crossing cell geometry to points.
#'
#' @param evac_grid Polygon grid from `make_evac_grid()`.
#' @param roads_buffer Buffered road/pathway network.
#' @param hazard_zone Optional hazard-zone polygon used to crop candidate
#'   origins after intersecting the grid with the road buffer.
#' @param max_origins Optional maximum number of origin points to retain. Useful
#'   for large regions or exploratory runs.
#' @param seed Random seed used when `max_origins` is supplied.
#' @return A point `SpatVector` of road-based evacuation origins.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' grid <- make_evac_grid(terra::as.polygons(r, dissolve = TRUE), resolution = 1)
#' roads <- terra::vect(matrix(c(0, 2, 4, 2), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' roads_buffer <- terra::buffer(roads, 0.2)
#' make_road_origins(grid, roads_buffer, hazard_zone = terra::as.polygons(r, dissolve = TRUE),
#'   max_origins = 3, seed = 1)
#' @export
make_road_origins <- function(
    evac_grid,
    roads_buffer,
    hazard_zone = NULL,
    max_origins = NULL,
    seed = NULL
) {
  if (!inherits(evac_grid, "SpatVector")) {
    stop("`evac_grid` must be a terra SpatVector.", call. = FALSE)
  }
  
  if (!inherits(roads_buffer, "SpatVector")) {
    stop("`roads_buffer` must be a terra SpatVector.", call. = FALSE)
  }

  if (!is.null(hazard_zone) && !inherits(hazard_zone, "SpatVector")) {
    stop("`hazard_zone` must be a terra SpatVector.", call. = FALSE)
  }

  candidate_area <- terra::intersect(evac_grid, roads_buffer)
  
  if (!is.null(hazard_zone)) {
    candidate_area <- terra::intersect(candidate_area, hazard_zone)
  }
  
  origins <- terra::centroids(candidate_area, inside = TRUE)
  
  if (!is.null(hazard_zone)) {
    origins <- terra::intersect(origins, hazard_zone)
  }
  
  if (!is.null(max_origins) && max_origins < nrow(origins)) {
    if (!is.null(seed)) set.seed(seed)
    origins <- origins[sort(sample.int(nrow(origins), max_origins)), ]
  }
  
  origins
}

#' Create a slope-based conductance surface
#'
#' Masks a DEM to an optional road/pathway mask and creates a slope conductance
#' surface using `leastcostpath`.
#'
#' @param dem Elevation raster.
#' @param road_mask Optional road/pathway mask.
#' @param resolution Optional target DEM resolution before conductance creation.
#' @param method Conductance method. Currently only `"slope"` is implemented.
#' @return A `leastcostpath` conductance surface object.
#' @examples
#' dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
#'   vals = 1, crs = "EPSG:3857")
#' make_conductance_surface(dem)
#' @export
make_conductance_surface <- function(
  dem,
  road_mask = NULL,
  resolution = NULL,
  method = "slope"
) {
  dem <- read_spatial(dem)

  if (!inherits(dem, "SpatRaster")) {
    stop("`dem` must be a SpatRaster or raster file path.", call. = FALSE)
  }

  if (!is.null(resolution)) {
    template <- terra::rast(ext = terra::ext(dem), crs = terra::crs(dem), resolution = resolution)
    dem <- terra::resample(dem, template)
  }

  if (!is.null(road_mask)) {
    road_mask <- read_spatial(road_mask)
    dem <- terra::mask(dem, road_mask)
  }

  method <- match.arg(method, choices = c("slope"))
  leastcostpath::create_slope_cs(dem)
}

test_that("road origins are kept inside the hazard zone", {
  r <- terra::rast(
    nrows = 4,
    ncols = 4,
    xmin = 0,
    xmax = 4,
    ymin = 0,
    ymax = 4,
    vals = 1,
    crs = "EPSG:3857"
  )
  hazard <- terra::as.polygons(terra::crop(r, terra::ext(0, 4, 0, 2)), dissolve = TRUE)
  grid <- make_evac_grid(hazard, resolution = 1)
  roads <- terra::vect(
    matrix(c(-1, 1.95, 5, 1.95), ncol = 2, byrow = TRUE),
    type = "lines",
    crs = "EPSG:3857"
  )
  roads_buffer <- terra::buffer(roads, 0.2)

  origins <- make_road_origins(grid, roads_buffer, hazard_zone = hazard)

  expect_gt(nrow(origins), 0)
  expect_true(all(terra::relate(origins, hazard, "intersects")))
})

# Internal utilities ---------------------------------------------------------

.raster_ext <- c("tif", "tiff", "img", "grd", "nc", "asc")
.vector_ext <- c("gpkg", "shp", "geojson", "json", "kml", "gml")

#' Read a spatial input
#'
#' Accepts an existing `terra` object or a file path and returns a `SpatRaster`
#' or `SpatVector`. Raster-like extensions are read with [terra::rast()] and
#' vector-like extensions are read with [terra::vect()].
#'
#' @param x A `SpatRaster`, `SpatVector`, or file path.
#' @return A `SpatRaster` or `SpatVector`.
#' @examples
#' r <- terra::rast(nrows = 2, ncols = 2, vals = 1)
#' read_spatial(r)
#' @export
read_spatial <- function(x) {
  if (inherits(x, c("SpatRaster", "SpatVector"))) {
    return(x)
  }

  if (!is.character(x) || length(x) != 1L) {
    stop("Input must be a SpatRaster, SpatVector, or single file path.", call. = FALSE)
  }

  if (!file.exists(x)) {
    stop("File does not exist: ", x, call. = FALSE)
  }

  ext <- tolower(utils::tail(strsplit(x, "\\.")[[1]], 1L))

  if (ext %in% .raster_ext) {
    return(terra::rast(x))
  }

  if (ext %in% .vector_ext) {
    return(terra::vect(x))
  }

  stop(
    "Unknown spatial file extension: .", ext,
    ". Use a terra SpatRaster/SpatVector or a known raster/vector file.",
    call. = FALSE
  )
}

.project_to <- function(x, target_crs = NULL) {
  if (is.null(target_crs)) {
    return(x)
  }

  if (is.na(terra::crs(x))) {
    stop("Cannot project an object with a missing CRS.", call. = FALSE)
  }

  terra::project(x, target_crs)
}

.as_hazard_polygon <- function(x, dissolve = TRUE) {
  if (inherits(x, "SpatRaster")) {
    # Assumes non-NA cells represent the hazard area. Users can prepare the
    # raster with prepare_hazard_zone() when a threshold is needed.
    x <- terra::as.polygons(x, dissolve = dissolve, na.rm = TRUE)
  }

  if (!inherits(x, "SpatVector")) {
    stop("Hazard zone must be a SpatRaster or SpatVector.", call. = FALSE)
  }

  if (dissolve) {
    x <- .safe_aggregate(x)
  }

  x
}

.safe_aggregate <- function(x) {
  tryCatch(
    terra::aggregate(x, dissolve = TRUE),
    error = function(e) terra::aggregate(x)
  )
}

.safe_as_lines <- function(x) {
  tryCatch(
    terra::as.lines(x),
    error = function(e) {
      stop("Could not convert object to lines: ", conditionMessage(e), call. = FALSE)
    }
  )
}

.combine_vectors <- function(...) {
  xs <- list(...)
  xs <- xs[!vapply(xs, is.null, logical(1))]

  if (length(xs) == 0L) {
    return(NULL)
  }
  if (length(xs) == 1L) {
    return(xs[[1]])
  }

  out <- xs[[1]]
  for (i in seq.int(2L, length(xs))) {
    out <- tryCatch(
      rbind(out, xs[[i]]),
      error = function(e) terra::vect(c(out, xs[[i]]))
    )
  }
  out
}

.has_column <- function(x, name) {
  name %in% names(x)
}

.stop_empty <- function(x, what) {
  n <- tryCatch(nrow(x), error = function(e) NA_integer_)
  if (is.na(n)) {
    n <- tryCatch(length(x), error = function(e) NA_integer_)
  }
  if (!is.na(n) && n == 0L) {
    stop(what, " is empty.", call. = FALSE)
  }
  invisible(x)
}

#' Create an evacuation grid
#'
#' Creates polygon grid cells over a hazard/evacuation zone and masks the grid to
#' the zone. The resulting cells can be intersected with buffered roads to create
#' road-based origin points.
#'
#' @param hazard_zone Hazard/evacuation zone as `SpatVector`, `SpatRaster`, or file path.
#' @param resolution Grid cell resolution in map units. Use meters when data are
#'   in a projected CRS. Can be length 1 or 2.
#' @return A polygon `SpatVector` grid clipped/masked to the hazard zone.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' zone <- terra::as.polygons(r, dissolve = TRUE)
#' make_evac_grid(zone, resolution = 1)
#' @export
make_evac_grid <- function(hazard_zone, resolution) {
  hazard_zone <- read_spatial(hazard_zone)
  hazard_poly <- .as_hazard_polygon(hazard_zone, dissolve = TRUE)

  grid_r <- terra::rast(
    ext = terra::ext(hazard_poly),
    crs = terra::crs(hazard_poly),
    resolution = resolution
  )
  grid_p <- terra::as.polygons(grid_r)
  grid <- terra::mask(grid_p, hazard_poly)
  .stop_empty(grid, "Evacuation grid")
  grid
}

#' Create a broader analysis region around a study area
#'
#' Dissolves the full hazard zone, buffers a smaller study area, and crops the
#' full zone to that buffer. This is useful when escape/safety points outside a
#' municipality or local study area should still be considered.
#'
#' @param hazard_zone Full hazard/evacuation zone.
#' @param study_area Local study area.
#' @param buffer_m Buffer distance in map units, typically meters.
#' @return A polygon `SpatVector` for the broader analysis region.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' zone <- terra::as.polygons(r, dissolve = TRUE)
#' study <- terra::as.polygons(terra::crop(r, terra::ext(1, 3, 1, 3)), dissolve = TRUE)
#' make_region_area(zone, study, buffer_m = 1)
#' @export
make_region_area <- function(hazard_zone, study_area, buffer_m = 5000) {
  hazard_zone <- .as_hazard_polygon(read_spatial(hazard_zone), dissolve = TRUE)
  study_area <- .as_hazard_polygon(read_spatial(study_area), dissolve = TRUE)

  study_buffer <- terra::buffer(study_area, buffer_m)
  study_buffer <- .safe_aggregate(study_buffer)

  region <- terra::crop(hazard_zone, study_buffer)
  region <- tryCatch(terra::erase(region), error = function(e) region)
  .stop_empty(region, "Region area")
  region
}

#' Identify escape/safety points where roads cross the hazard-zone boundary
#'
#' Intersects a road/pathway network with the boundary of the hazard zone and
#' converts the intersection geometry to points. These points represent candidate
#' exits from the hazard zone.
#'
#' @param hazard_zone Hazard/evacuation zone.
#' @param roads Road/pathway network.
#' @param study_area Optional local study area used to crop candidate escape
#'   points to a broader region around the study area.
#' @param region_buffer_m Buffer distance passed to `make_region_area()` when
#'   `study_area` is supplied.
#' @return A point `SpatVector` of candidate escape/safety points.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' zone <- terra::as.polygons(r, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(-1, 2, 5, 2), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' find_escape_points(zone, roads)
#' @export
find_escape_points <- function(
  hazard_zone,
  roads,
  study_area = NULL,
  region_buffer_m = 5000
) {
  hazard_zone <- .as_hazard_polygon(read_spatial(hazard_zone), dissolve = TRUE)
  roads <- read_spatial(roads)

  if (!inherits(roads, "SpatVector")) {
    stop("`roads` must be a SpatVector or vector file path.", call. = FALSE)
  }

  hazard_boundary <- .safe_as_lines(hazard_zone)
  roads_lines <- .safe_as_lines(roads)

  escape <- terra::intersect(roads_lines, hazard_boundary)
  .stop_empty(escape, "Road/hazard-boundary intersection")

  # terra can return mixed line/point geometry. Coordinate extraction gives a
  # stable point representation for least-cost destinations.
  coords <- terra::crds(escape, df = TRUE)
  if (!all(c("x", "y") %in% names(coords))) {
    names(coords)[1:2] <- c("x", "y")
  }
  escape <- terra::vect(coords[, c("x", "y"), drop = FALSE], geom = c("x", "y"), crs = terra::crs(hazard_zone))

  if (!is.null(study_area)) {
    region <- make_region_area(hazard_zone, study_area, buffer_m = region_buffer_m)
    escape <- terra::crop(escape, region, ext = TRUE)
  }

  .stop_empty(escape, "Escape points")
  escape
}

#' Calculate a least-cost path between one origin and one destination
#'
#' Compatibility wrapper around [leastcostpath::create_lcp()] that returns `NULL`
#' when a path cannot be created.
#'
#' @param cs A `leastcostpath` conductance surface.
#' @param origin Origin point.
#' @param destination Destination point.
#' @return A least-cost path object or `NULL`.
#' @examples
#' dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
#'   vals = 1, crs = "EPSG:3857")
#' cs <- make_conductance_surface(dem)
#' origin <- terra::vect(data.frame(x = 0.5, y = 0.5), geom = c("x", "y"), crs = "EPSG:3857")
#' destination <- terra::vect(data.frame(x = 4.5, y = 4.5), geom = c("x", "y"), crs = "EPSG:3857")
#' calculate_lc_path(cs, origin, destination)
#' @export
calculate_lc_path <- function(cs, origin, destination) {
  tryCatch(
    leastcostpath::create_lcp(
      x = cs,
      origin = origin,
      destination = destination,
      check_locations = TRUE
    ),
    error = function(e) NULL
  )
}

#' Calculate the minimum path distance from a list of least-cost paths
#'
#' @param lc_paths_list A list of least-cost path line vectors.
#' @return Minimum non-zero, finite path distance.
#' @examples
#' line <- terra::vect(matrix(c(0, 0, 1, 1), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' calculate_min_dist(list(line))
#' @export
calculate_min_dist <- function(lc_paths_list) {
  lc_paths <- terra::vect(lc_paths_list)
  d <- terra::perim(lc_paths)
  d <- d[is.finite(d) & d != 0]

  if (length(d) == 0L) {
    return(Inf)
  }

  min(d)
}

#' Calculate minimum least-cost distance from origins to safety
#'
#' For each origin point, calculates least-cost paths to all candidate safety
#' points and stores the shortest finite path distance. Destination points are
#' appended to the output with distance equal to zero.
#'
#' @param cs A `leastcostpath` conductance surface.
#' @param origins Origin points.
#' @param destinations Escape/safety destination points.
#' @param include_destinations Logical. Add destination points with distance = 0.
#' @param progress Logical. Print simple progress messages.
#' @param progress_every Integer. Print progress every `n` origins when `progress = TRUE`.
#' @param check_locations Logical passed to `leastcostpath::create_lcp()`. The default
#'   is `FALSE` for speed after inputs have already been projected and cropped.
#' @return A point `SpatVector` with columns `distance` and `type`.
#' @examples
#' dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
#'   vals = 1, crs = "EPSG:3857")
#' cs <- make_conductance_surface(dem)
#' origins <- terra::vect(data.frame(x = 0.5, y = 0.5), geom = c("x", "y"), crs = "EPSG:3857")
#' destinations <- terra::vect(data.frame(x = 4.5, y = 4.5), geom = c("x", "y"), crs = "EPSG:3857")
#' calc_min_distance_to_safety(cs, origins, destinations)
#' @export
calc_min_distance_to_safety <- function(
  cs,
  origins,
  destinations,
  include_destinations = TRUE,
  progress = FALSE,
  progress_every = 1L,
  check_locations = FALSE
) {
  origins <- read_spatial(origins)
  destinations <- read_spatial(destinations)

  cs_rast <- leastcostpath::rasterise(cs)
  origins <- terra::project(origins, terra::crs(cs_rast))
  destinations <- terra::project(destinations, terra::crs(cs_rast))
  destinations <- terra::crop(destinations, cs_rast, ext = TRUE)

  .stop_empty(origins, "Origin points")
  .stop_empty(destinations, "Destination points")

  progress_every <- as.integer(progress_every)
  if (is.na(progress_every) || progress_every < 1L) {
    progress_every <- 1L
  }

  if (isTRUE(progress)) {
    message("Calculating least-cost distance for ", nrow(origins),
            " origins and ", nrow(destinations), " destinations.")
    message("Conductance raster: ", paste(terra::res(cs_rast), collapse = " x "),
            " map units; ", terra::ncell(cs_rast), " cells.")
  }

  out <- vector("list", nrow(origins))

  for (i in seq_len(nrow(origins))) {
    if (isTRUE(progress) && (i %% progress_every == 0L || i == 1L || i == nrow(origins))) {
      message("Processing origin ", i, " of ", nrow(origins))
    }

    paths <- tryCatch(
      leastcostpath::create_lcp(
        x = cs,
        origin = origins[i, ],
        destination = destinations,
        check_locations = check_locations
      ),
      error = function(e) NULL
    )

    if (is.null(paths)) {
      next
    }

    d <- terra::perim(paths)
    d <- d[is.finite(d) & d != 0]

    if (length(d) == 0L) {
      next
    }

    xy <- terra::crds(origins[i, ], df = TRUE)[1, c("x", "y")]
    out[[i]] <- data.frame(
      x = xy$x,
      y = xy$y,
      distance = min(d),
      type = "road"
    )
  }

  out <- do.call(rbind, out)

  if (is.null(out) || nrow(out) == 0L) {
    warning("No finite least-cost paths were found from origins to destinations.", call. = FALSE)
    out <- data.frame(x = numeric(), y = numeric(), distance = numeric(), type = character())
  }

  if (isTRUE(include_destinations)) {
    dest_xy <- terra::crds(destinations, df = TRUE)
    dest_xy <- dest_xy[, c("x", "y"), drop = FALSE]
    dest_out <- data.frame(
      x = dest_xy$x,
      y = dest_xy$y,
      distance = 0,
      type = "escape"
    )
    out <- rbind(out, dest_out)
  }

  terra::vect(out, geom = c("x", "y"), crs = terra::crs(origins))
}

#' Convert distance to evacuation time
#'
#' @param distance_m Distance in meters.
#' @param walking_speed_mps Walking speed in meters per second. The paper-style
#'   default is 1.22 m/s, but this should be changed for local planning scenarios.
#' @param units Output units: `"minutes"`, `"seconds"`, or `"hours"`.
#' @return Numeric vector of evacuation times.
#' @examples
#' calc_evac_time(c(0, 120, 240), walking_speed_mps = 1.2)
#' @export
calc_evac_time <- function(distance_m, walking_speed_mps = 1.22, units = "minutes") {
  units <- match.arg(units, choices = c("minutes", "seconds", "hours"))

  if (walking_speed_mps <= 0) {
    stop("`walking_speed_mps` must be greater than zero.", call. = FALSE)
  }

  seconds <- distance_m / walking_speed_mps

  switch(
    units,
    seconds = seconds,
    minutes = seconds / 60,
    hours = seconds / 3600
  )
}

#' Create evacuation-distance and evacuation-time polygons
#'
#' Creates Voronoi polygons from least-cost distance points, converts distance to
#' travel time, and optionally clips the output to an inundated road/study mask.
#'
#' @param distance_points Point output from `calc_min_distance_to_safety()`.
#' @param clip_area Optional polygon used to crop the Voronoi output.
#' @param walking_speed_mps Walking speed in meters per second.
#' @param region_name Optional region/municipality name stored in the output.
#' @param distance_col Name of the output distance column.
#' @param time_col Name of the output time column.
#' @return A polygon `SpatVector`.
#' @examples
#' pts <- terra::vect(
#'   data.frame(x = c(0, 1, 0, 1), y = c(0, 0, 1, 1), distance = c(0, 5, 10, 15)),
#'   geom = c("x", "y"),
#'   crs = "EPSG:3857"
#' )
#' clip <- terra::as.polygons(terra::rast(nrows = 2, ncols = 2, xmin = -1, xmax = 2,
#'   ymin = -1, ymax = 2, vals = 1, crs = "EPSG:3857"), dissolve = TRUE)
#' make_evac_polygons(pts, clip_area = clip)
#' @export
make_evac_polygons <- function(
  distance_points,
  clip_area = NULL,
  walking_speed_mps = 1.22,
  region_name = NULL,
  distance_col = "DistToSafety",
  time_col = "EvacTimeAvg"
) {
  distance_points <- read_spatial(distance_points)

  if (!"distance" %in% names(distance_points)) {
    stop("`distance_points` must contain a `distance` column.", call. = FALSE)
  }

  v <- terra::voronoi(distance_points)
  v[[time_col]] <- calc_evac_time(as.numeric(v$distance), walking_speed_mps, units = "minutes")
  v[[distance_col]] <- as.numeric(v$distance)

  if (!is.null(region_name)) {
    v$Region <- region_name
  }

  keep <- c(if (!is.null(region_name)) "Region", distance_col, time_col)
  v <- v[, keep]

  if (!is.null(clip_area)) {
    clip_area <- read_spatial(clip_area)
    v <- terra::crop(v, clip_area)
  }

  v
}

test_that("calc_evac_time converts meters to minutes", {
  expect_equal(calc_evac_time(122, walking_speed_mps = 1.22), 100 / 60)
})

test_that("calc_evac_time supports seconds and hours", {
  expect_equal(calc_evac_time(122, walking_speed_mps = 1.22, units = "seconds"), 100)
  expect_equal(calc_evac_time(3600, walking_speed_mps = 1, units = "hours"), 1)
})

#' Interpolate a distance surface from least-cost distance points
#'
#' Uses thin-plate spline interpolation to create a continuous distance surface.
#' This preserves the original paper-script approach while keeping interpolation
#' optional. For most package workflows, `make_evac_polygons()` is the simpler
#' output.
#'
#' @param distance_points Point layer with a distance column.
#' @param region_area Broader analysis region used for interpolation extent.
#' @param study_area Final study area used to crop and mask the output.
#' @param resolution_coarse Coarse interpolation resolution.
#' @param resolution_fine Fine output resolution.
#' @param distance_col Name of distance column in `distance_points`.
#' @return A `SpatRaster` distance surface.
#' @examples
#' if (requireNamespace("fields", quietly = TRUE)) {
#'   pts <- terra::vect(
#'     data.frame(
#'       x = c(0, 1, 0, 1, 0.5, 1.5),
#'       y = c(0, 0, 1, 1, 0.5, 1.5),
#'       distance = c(0, 5, 10, 15, 7, 20)
#'     ),
#'     geom = c("x", "y"),
#'     crs = "EPSG:3857"
#'   )
#'   area <- terra::as.polygons(terra::rast(nrows = 2, ncols = 2, xmin = -1, xmax = 2,
#'     ymin = -1, ymax = 2, vals = 1, crs = "EPSG:3857"), dissolve = TRUE)
#'   interpolate_distance_surface(pts, area, area, resolution_coarse = 0.5, resolution_fine = 1)
#' }
#' @export
interpolate_distance_surface <- function(
  distance_points,
  region_area,
  study_area,
  resolution_coarse = 100,
  resolution_fine = 1,
  distance_col = "distance"
) {
  if (!requireNamespace("fields", quietly = TRUE)) {
    stop("Package `fields` is required for thin-plate spline interpolation.", call. = FALSE)
  }

  distance_points <- read_spatial(distance_points)
  region_area <- read_spatial(region_area)
  study_area <- read_spatial(study_area)

  if (!distance_col %in% names(distance_points)) {
    stop("`distance_points` must contain column `", distance_col, "`.", call. = FALSE)
  }

  r1 <- terra::rast(
    resolution = resolution_coarse,
    crs = terra::crs(distance_points),
    extent = terra::ext(region_area)
  )

  dist_grid <- terra::rasterize(distance_points, r1, distance_col)
  xy <- data.frame(terra::xyFromCell(dist_grid, seq_len(terra::ncell(dist_grid))))
  v <- terra::values(dist_grid)
  if (is.matrix(v) || is.data.frame(v)) {
    v <- v[, 1]
  }
  keep <- !is.na(v)

  if (sum(keep) < 3L) {
    stop("At least three non-NA points are required for thin-plate spline interpolation.", call. = FALSE)
  }

  tps <- fields::Tps(xy[keep, ], v[keep])
  predict_tps <- function(model, x, ...) {
    fields::predict.Krig(model, as.matrix(x[, c("x", "y")]))
  }
  dist_grid <- terra::interpolate(dist_grid, tps, fun = predict_tps)
  dist_grid[dist_grid < 0] <- 0

  r2 <- terra::rast(
    resolution = resolution_fine,
    crs = terra::crs(distance_points),
    extent = terra::ext(region_area)
  )

  dist_grid <- terra::resample(dist_grid, r2)
  dist_grid <- terra::crop(dist_grid, study_area)
  terra::mask(dist_grid, study_area)
}

#' Write evacpath outputs to disk
#'
#' Writes the main spatial outputs in an `evacpath_result` list to GeoPackage and
#' GeoTIFF files. Non-spatial objects, including the conductance surface, are not
#' written.
#'
#' @param result An object returned by `run_evacpath()`.
#' @param output_dir Output directory.
#' @param prefix Filename prefix.
#' @param overwrite Logical. Overwrite existing files.
#' @param include_inputs Logical. Also write projected input layers.
#' @return A named character vector of written file paths.
#' @examples
#' dem <- terra::rast(nrows = 7, ncols = 7, xmin = 0, xmax = 7, ymin = 0, ymax = 7,
#'   vals = 1, crs = "EPSG:3857")
#' hazard_raster <- terra::crop(dem, terra::ext(1, 6, 1, 6))
#' hazard <- terra::as.polygons(hazard_raster, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(0, 3.5, 7, 3.5), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' result <- run_evacpath(
#'   hazard_zone = hazard,
#'   roads = roads,
#'   dem = dem,
#'   grid_resolution = 1,
#'   road_buffer_m = 0.2,
#'   escape_buffer_m = 0.3,
#'   final_road_buffer_m = 0.2,
#'   max_origins = 2,
#'   max_destinations = 2,
#'   seed = 1
#' )
#' out_dir <- file.path(tempdir(), "evacpath-example")
#' paths <- write_evac_outputs(result, output_dir = out_dir, prefix = "example")
#' unlink(out_dir, recursive = TRUE)
#' @export
write_evac_outputs <- function(
  result,
  output_dir,
  prefix = "evacpath",
  overwrite = TRUE,
  include_inputs = FALSE
) {
  if (!dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE)
  }

  vector_names <- c(
    "evac_grid",
    "escape_points",
    "road_mask",
    "roads_buffer",
    "escape_buffer",
    "road_points",
    "distance_points",
    "clip_area",
    "evac_polygons",
    "time_grid"
  )

  raster_names <- character(0)

  if (isTRUE(include_inputs)) {
    vector_names <- c("hazard_zone", "escape_zone", "roads", "study_area", vector_names)
    raster_names <- c("dem", raster_names)
  }

  written <- character(0)

  for (nm in unique(vector_names)) {
    obj <- result[[nm]]
    if (is.null(obj) || !inherits(obj, "SpatVector")) {
      next
    }
    path <- file.path(output_dir, paste0(prefix, "_", nm, ".gpkg"))
    terra::writeVector(obj, path, overwrite = overwrite)
    written[nm] <- path
  }

  for (nm in unique(raster_names)) {
    obj <- result[[nm]]
    if (is.null(obj) || !inherits(obj, "SpatRaster")) {
      next
    }
    path <- file.path(output_dir, paste0(prefix, "_", nm, ".tif"))
    terra::writeRaster(obj, path, overwrite = overwrite)
    written[nm] <- path
  }

  written
}

#' Crop roads to an inset extent before escape-point detection
#'
#' Crops a road/pathway layer to a slightly reduced bounding box around a zone.
#' This is useful before escape-point detection because roads extending beyond the
#' study-area coverage can intersect artificial raster or polygon extent edges and
#' create false escape/safety points.
#'
#' @param roads Road/pathway network as a `SpatVector` or file path.
#' @param zone Zone used to define the outer extent. Can be a `SpatVector`,
#'   `SpatRaster`, or file path.
#' @param inset_x_m Numeric. Distance to inset the minimum and maximum x
#'   boundaries, in map units. Use meters when the data are projected.
#' @param inset_y_m Numeric. Distance to inset the minimum and maximum y
#'   boundaries, in map units. Use meters when the data are projected.
#'
#' @return A cropped road/pathway `SpatVector`.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' zone <- terra::as.polygons(r, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(-1, 2, 5, 2), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' crop_roads_to_inner_extent(roads, zone, inset_x_m = 0.5, inset_y_m = 0)
#' @export
crop_roads_to_inner_extent <- function(
  roads,
  zone,
  inset_x_m = 250,
  inset_y_m = 250
) {
  roads <- read_spatial(roads)
  zone <- read_spatial(zone)

  if (!inherits(roads, "SpatVector")) {
    stop("`roads` must be a terra SpatVector or vector file path.", call. = FALSE)
  }

  if (!inherits(zone, c("SpatVector", "SpatRaster"))) {
    stop("`zone` must be a terra SpatVector, SpatRaster, or spatial file path.", call. = FALSE)
  }

  inset_x_m <- as.numeric(inset_x_m)
  inset_y_m <- as.numeric(inset_y_m)

  if (length(inset_x_m) != 1L || is.na(inset_x_m) || inset_x_m < 0) {
    stop("`inset_x_m` must be a single non-negative number.", call. = FALSE)
  }

  if (length(inset_y_m) != 1L || is.na(inset_y_m) || inset_y_m < 0) {
    stop("`inset_y_m` must be a single non-negative number.", call. = FALSE)
  }

  e <- terra::ext(zone)
  width_x <- terra::xmax(e) - terra::xmin(e)
  width_y <- terra::ymax(e) - terra::ymin(e)

  if (width_x <= 2 * inset_x_m) {
    stop("`inset_x_m` is too large for the zone extent.", call. = FALSE)
  }

  if (width_y <= 2 * inset_y_m) {
    stop("`inset_y_m` is too large for the zone extent.", call. = FALSE)
  }

  inner_extent <- terra::ext(
    terra::xmin(e) + inset_x_m,
    terra::xmax(e) - inset_x_m,
    terra::ymin(e) + inset_y_m,
    terra::ymax(e) - inset_y_m
  )

  out <- terra::crop(roads, inner_extent)
  .stop_empty(out, "Roads cropped to inner extent")
  out
}

#' Add buffered road corridors to an escape-boundary zone
#'
#' Creates a road-aware escape zone by combining a base escape zone with buffered
#' road/pathway corridors. This is useful for tsunami workflows in coastal cities
#' where bridges, causeways, or other walkways over water can be lost when the
#' inundation layer is split into land and water masks. The resulting object can
#' be passed to `find_escape_points()` so escape/safety points are generated from
#' a boundary that includes relevant road corridors as well as the tsunami zone.
#'
#' @param escape_zone Base escape-boundary zone, usually the land-inundation-plus-
#'   water zone from `prepare_tsunami_zones()`.
#' @param roads Road/pathway network used to create the road-aware corridor.
#' @param road_buffer_m First road buffer distance in map units.
#' @param crop_buffer_m Optional second buffer applied before cropping/combining.
#' @param include_base_zone Logical. If `TRUE`, combine the buffered roads with
#'   the original `escape_zone`. If `FALSE`, only the buffered road corridor is
#'   returned.
#'
#' @return A dissolved `SpatVector` escape-boundary zone.
#' @examples
#' r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
#'   vals = 1, crs = "EPSG:3857")
#' zone <- terra::as.polygons(r, dissolve = TRUE)
#' roads <- terra::vect(matrix(c(0, 2, 4, 2), ncol = 2, byrow = TRUE),
#'   type = "lines", crs = "EPSG:3857")
#' make_road_aware_escape_zone(zone, roads, road_buffer_m = 0.1)
#' @export
make_road_aware_escape_zone <- function(
  escape_zone,
  roads,
  road_buffer_m = 2,
  crop_buffer_m = 3,
  include_base_zone = TRUE
) {
  make_roads_in_zone(
    roads = roads,
    zone = escape_zone,
    road_buffer_m = road_buffer_m,
    crop_buffer_m = crop_buffer_m,
    include_zone = include_base_zone
  )
}

---
title: "Using evacpath in a new region"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using evacpath in a new region}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE, purl = FALSE)
```

For a new region, `evacpath` expects three core spatial inputs:

1. a hazard or inundation zone,
2. a road/pathway network, and
3. a DEM or elevation raster.

The most important requirement is that distance calculations should use a projected coordinate reference system in meters.

The workflow builds on open-source least-cost path methods for evacuation planning (Cordero et al. 2025) and uses `leastcostpath` for least-cost path and movement-potential modeling (Lewis 2023; Lewis 2021).

```{r setup}
library(evacpath)
```

## Minimal workflow

```{r minimal}
result <- run_evacpath(
  hazard_zone = "path/to/hazard_zone.gpkg",
  roads = "path/to/roads.gpkg",
  dem = "path/to/dem.tif",
  target_crs = "EPSG:XXXX",
  region_name = "New region",
  walking_speed_mps = 1.22
)
```

## Common customizations

```{r custom}
result <- run_evacpath(
  hazard_zone = "path/to/hazard_zone.gpkg",
  roads = "path/to/roads.gpkg",
  dem = "path/to/dem.tif",
  target_crs = "EPSG:XXXX",
  region_name = "New region",

  # Road and escape-point geometry assumptions
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,

  # Origin-point density
  grid_resolution = 50,
  max_origins = NULL,

  # Travel-time assumption
  walking_speed_mps = 1.22,

  # Output clipping
  clip_mode = "road_hazard"
)
```

## Case-specific preprocessing

Keep region-specific choices outside the package. For example, if your DEM uses negative elevation for land, correct it before calling `run_evacpath()`. If your road layer contains piers or other unsuitable features, remove them with `clean_roads()`.

```{r preprocessing}
roads <- clean_roads(
  roads = "path/to/roads.gpkg",
  exclude = list(field = "road_type", values = c("pier", "private")),
  target_crs = "EPSG:XXXX"
)
```

## References

Cordero, E., Ruiz Vélez, R., Huérfano Moreno, V., Sherman, C., 2025. Enhancing tsunami evacuation strategies in Puerto Rico using open-source least-cost path analysis. J. Disaster Sci. Manag. 1, 18. https://doi.org/10.1007/s44367-025-00018-y

Lewis, J., 2023. leastcostpath: Modelling Pathways and Movement Potential Within a Landscape.

Lewis, J., 2021. Probabilistic Modelling for Incorporating Uncertainty in Least Cost Path Results: a Postdictive Roman Road Case Study. Journal of Archaeological Method and Theory 28, 911-924. https://doi.org/10.1007/s10816-021-09522-w

---
title: "Evacuation workflow: function-by-function diagnostics"
output: rmarkdown::html_vignette
params:
  run_diagnostics: false
  run_lcp: false
  data_dir: "data-raw"
  target_crs: "EPSG:32748"
  region_name: "Example region"
  dem_sign_multiplier: 1
  inundation_threshold: 0
  land_threshold: 0
  water_threshold: 0
  resample_method: "bilinear"
  road_exclude_field: "man_made"
  road_exclude_values: !r c("pier")
  grid_multiplier: 50
  dem_multiplier: 50
  road_buffer_m: 2
  escape_buffer_m: 5
  final_road_buffer_m: 3
  escape_roads_inset_x_m: 50
  escape_roads_inset_y_m: 50
  road_aware_escape_zone: true
  max_origins: null
  max_destinations: null
  walking_speed_mps: 1.22
  seed: 23401
  output_dir: "_outputs/example"
vignette: >
  %\VignetteIndexEntry{Evacuation workflow: function-by-function diagnostics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 6,
  dpi = 120
)
```

This vignette is a diagnostic walkthrough for the example tsunami-evacuation workflow. It is intentionally more detailed than a normal user vignette because its purpose is to test every major `evacpath` function independently before trusting the high-level `run_evacpath()` wrapper.

The workflow builds on open-source least-cost path methods for evacuation planning (Cordero et al. 2025) and uses `leastcostpath` for least-cost path and movement-potential modeling (Lewis 2023; Lewis 2021).

This example workflow has three important tsunami-specific complications:

1. The **land-only inundation zone** is the correct area for evacuation origins and final output mapping, but it is not necessarily the correct boundary for escape-point detection.
2. The **water-combined escape zone** prevents the coastline from becoming a fake safety boundary.
3. The **road-aware escape zone** combines buffered roads with the escape zone before escape points are generated. This preserves bridges, causeways, and walkways over water that can be lost when the tsunami layer is split into land and water masks.

The diagnostic workflow therefore uses these separate objects:

```text
hazard_zone          = land-only tsunami evacuation zone for origins/output
escape_zone          = land-only TEZ + water, used as base escape boundary
roads_for_escape     = roads cropped to an inset study extent
escape_boundary_zone = escape_zone + buffered roads_for_escape
```

To run the full diagnostic workflow locally:

```{r render-command, eval = FALSE}
rmarkdown::render(
  "vignettes/diagnostic-example.Rmd",
  params = list(
    run_diagnostics = TRUE,
    run_lcp = TRUE,
    data_dir = "data-raw",
    target_crs = "EPSG:32748",
    dem_sign_multiplier = 1,
    grid_multiplier = 50,
    dem_multiplier = 50,
    escape_roads_inset_x_m = 50,
    escape_roads_inset_y_m = 50,
    road_aware_escape_zone = TRUE
  )
)
```

For a faster diagnostic that stops before least-cost paths:

```{r render-fast, eval = FALSE}
rmarkdown::render(
  "vignettes/diagnostic-example.Rmd",
  params = list(
    run_diagnostics = TRUE,
    run_lcp = FALSE,
    data_dir = "data-raw"
  )
)
```

# 1. Setup

```{r setup}
# This loader works both when the package is installed and during local package development.
if (requireNamespace("evacpath", quietly = TRUE)) {
  library(evacpath)
} else if (requireNamespace("devtools", quietly = TRUE) && file.exists("DESCRIPTION")) {
  devtools::load_all(".")
} else if (requireNamespace("devtools", quietly = TRUE) && file.exists(file.path("evacpath", "DESCRIPTION"))) {
  devtools::load_all("evacpath")
} else {
  stop("Install evacpath or run this from the package root/parent directory.", call. = FALSE)
}

library(terra)

run_diagnostics <- isTRUE(params$run_diagnostics)
run_lcp <- isTRUE(params$run_lcp)

assert_true <- function(x, message) {
  if (!isTRUE(x)) {
    stop(message, call. = FALSE)
  }
  invisible(TRUE)
}

is_spatvector <- function(x) inherits(x, "SpatVector")
is_spatraster <- function(x) inherits(x, "SpatRaster")

count_features <- function(x) {
  if (inherits(x, "SpatRaster")) return(terra::ncell(x))
  if (inherits(x, "SpatVector")) return(nrow(x))
  NA_integer_
}
```

The diagnostic switches are:

```{r switches}
run_diagnostics
run_lcp
```

# 2. Locate example input files

The diagnostic expects these files:

```text
data-raw/dem.tif
data-raw/rds.gpkg
data-raw/tsunami_inundation_depth.tif
```

The function below searches for them from the package root, the parent folder, or installed package `extdata`.

```{r locate-files}
find_input_file <- function(filename, data_dir = params$data_dir) {
  candidate_dirs <- unique(c(
    data_dir,
    file.path("evacpath", data_dir),
    file.path(getwd(), data_dir),
    file.path(getwd(), "evacpath", data_dir),
    system.file("extdata", package = "evacpath")
  ))

  candidate_dirs <- candidate_dirs[nzchar(candidate_dirs)]
  candidates <- file.path(candidate_dirs, filename)
  hits <- candidates[file.exists(candidates)]

  if (length(hits) == 0L) {
    return(NA_character_)
  }

  normalizePath(hits[1])
}

dem_file <- find_input_file("dem.tif")
roads_file <- find_input_file("rds.gpkg")
inundation_file <- find_input_file("tsunami_inundation_depth.tif")

input_files <- data.frame(
  layer = c("DEM", "roads", "inundation depth"),
  file = c(dem_file, roads_file, inundation_file),
  exists = file.exists(c(dem_file, roads_file, inundation_file))
)

input_files

example_data_available <- all(input_files$exists)
example_data_available
```

If `example_data_available` is `FALSE`, check `params` or use the packaged files in `inst/extdata/`.

# 3. Read the spatial inputs with `read_spatial()`

`read_spatial()` accepts a file path or an existing `terra` object. Raster formats are read with `terra::rast()` and vector formats are read with `terra::vect()`.

```{r read-inputs, eval = run_diagnostics && example_data_available}
dem_raw <- read_spatial(dem_file)
roads_raw <- read_spatial(roads_file)
inundation_raw <- read_spatial(inundation_file)

assert_true(is_spatraster(dem_raw), "DEM was not read as a SpatRaster.")
assert_true(is_spatvector(roads_raw), "Roads were not read as a SpatVector.")
assert_true(is_spatraster(inundation_raw), "Inundation input was not read as a SpatRaster.")

list(
  dem_class = class(dem_raw),
  roads_class = class(roads_raw),
  inundation_class = class(inundation_raw)
)
```

## 3.1 Raw input diagnostics

Before modeling, inspect CRS, extent, resolution, raster value ranges, and road attributes.

```{r raw-diagnostics, eval = run_diagnostics && example_data_available}
cat("DEM CRS:\n", terra::crs(dem_raw), "\n\n")
cat("Roads CRS:\n", terra::crs(roads_raw), "\n\n")
cat("Inundation CRS:\n", terra::crs(inundation_raw), "\n\n")

cat("DEM resolution:\n")
print(terra::res(dem_raw))

cat("Inundation resolution:\n")
print(terra::res(inundation_raw))

cat("DEM cell count:\n")
print(terra::ncell(dem_raw))

cat("Road feature count:\n")
print(nrow(roads_raw))

cat("Road attribute names:\n")
print(names(roads_raw))

cat("DEM range:\n")
print(terra::global(dem_raw, fun = range, na.rm = TRUE))

cat("Inundation range:\n")
print(terra::global(inundation_raw, fun = range, na.rm = TRUE))
```

```{r plot-raw-inputs, eval = run_diagnostics && example_data_available, fig.height = 7}
oldpar <- par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
plot(dem_raw, main = "Raw DEM")
plot(inundation_raw, main = "Raw inundation depth")
par(oldpar)
```

# 4. Clean roads with `clean_roads()`

This is where region-specific road filtering belongs. In the packaged example, piers are excluded because they can create misleading road-water intersections.

```{r clean-roads, eval = run_diagnostics && example_data_available}
road_exclude <- NULL
if (!is.null(params$road_exclude_field) && nzchar(params$road_exclude_field)) {
  road_exclude <- list(
    field = params$road_exclude_field,
    values = params$road_exclude_values
  )
}

roads_clean <- clean_roads(
  roads = roads_raw,
  exclude = road_exclude,
  target_crs = params$target_crs
)

assert_true(is_spatvector(roads_clean), "clean_roads() did not return a SpatVector.")
assert_true(nrow(roads_clean) > 0, "clean_roads() returned no roads.")

nrow(roads_raw)
nrow(roads_clean)
```

# 5. Prepare tsunami zones with `prepare_tsunami_zones()`

This is the critical tsunami-specific step. The old script created a land mask, water mask, land-only TEZ, and then a TEZ-plus-water layer. The package makes that separation explicit.

```{r prepare-tsunami-zones, eval = run_diagnostics && example_data_available}
zones <- prepare_tsunami_zones(
  inundation = inundation_raw,
  dem = dem_raw,
  target_crs = params$target_crs,
  inundation_threshold = params$inundation_threshold,
  land_threshold = params$land_threshold,
  water_threshold = params$water_threshold,
  dem_sign_multiplier = params$dem_sign_multiplier,
  resample_method = params$resample_method,
  as_polygon = TRUE,
  dissolve = TRUE
)

zones

assert_true(is_spatvector(zones$hazard_zone), "zones$hazard_zone is not a SpatVector.")
assert_true(is_spatvector(zones$escape_zone), "zones$escape_zone is not a SpatVector.")
assert_true(is_spatraster(zones$hazard_raster), "zones$hazard_raster is not a SpatRaster.")
assert_true(is_spatraster(zones$escape_raster), "zones$escape_raster is not a SpatRaster.")

cat("Land-only hazard-zone polygons:", nrow(zones$hazard_zone), "\n")
cat("Water-combined escape-zone polygons:", nrow(zones$escape_zone), "\n")
```

## 5.1 Check the land/water classification

If land and water are reversed, change `dem_sign_multiplier` from `1` to `-1` or adjust the thresholds.

```{r land-water-check, eval = run_diagnostics && example_data_available, fig.height = 7}
oldpar <- par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
plot(zones$land_mask, col = "grey85", legend = FALSE, main = "DEM-derived land mask")
plot(zones$water_mask, col = "lightskyblue1", legend = FALSE, main = "DEM-derived water mask")
par(oldpar)
```

## 5.2 Check the two tsunami zones

The land-only zone is for origins and final mapped evacuation time. The water-combined zone is the base zone for escape-point detection.

```{r plot-tsunami-zones, eval = run_diagnostics && example_data_available, fig.height = 7}
oldpar <- par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
plot(zones$hazard_zone, col = "tomato", border = NA, main = "hazard_zone: land-only TEZ")
plot(zones$escape_zone, col = "grey90", border = NA, main = "escape_zone: TEZ + water")
plot(zones$hazard_zone, add = TRUE, col = "tomato", border = NA)
par(oldpar)
```

# 6. Crop roads to an inner extent before escape-point detection

If the road dataset extends beyond the reliable hazard-zone coverage, roads can intersect the artificial rectangular study-area boundary. Those intersections are not real escape points. The fix is to crop the roads used for escape-point detection to a slightly inset extent. The x and y insets are controlled separately.

```{r crop-roads-inner-extent, eval = run_diagnostics && example_data_available}
roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads_clean,
  zone = zones$escape_zone,
  inset_x_m = params$escape_roads_inset_x_m,
  inset_y_m = params$escape_roads_inset_y_m
)

assert_true(is_spatvector(roads_for_escape), "roads_for_escape is not a SpatVector.")
assert_true(nrow(roads_for_escape) > 0, "roads_for_escape has no features.")

cat("Clean roads:", nrow(roads_clean), "\n")
cat("Roads used for escape-point detection:", nrow(roads_for_escape), "\n")
```

```{r plot-roads-inner-extent, eval = run_diagnostics && example_data_available, fig.height = 7}
plot(zones$escape_zone, col = "grey92", border = "grey60", main = "Roads cropped to inset extent for escape detection")
plot(roads_clean, add = TRUE, col = adjustcolor("red", alpha.f = 0.25), lwd = 0.35)
plot(roads_for_escape, add = TRUE, col = "black", lwd = 0.45)
legend(
  "topright",
  legend = c("Original cleaned roads", "Roads used for escape detection"),
  col = c(adjustcolor("red", alpha.f = 0.5), "black"),
  lty = 1,
  lwd = 1,
  bty = "n",
  cex = 0.8
)
```

# 7. Combine buffered roads with the escape zone

This is the second major correction. The buffered roads should be combined with the tsunami escape zone **before** escape points are generated. Otherwise, bridge, causeway, or walkway segments over water can be lost because the land and water masks do not always represent the accessible transportation surface.

The package helper is `make_road_aware_escape_zone()`.

```{r road-aware-escape-zone, eval = run_diagnostics && example_data_available}
escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = params$road_buffer_m,
  crop_buffer_m = params$final_road_buffer_m,
  include_base_zone = TRUE
)

assert_true(is_spatvector(escape_boundary_zone), "escape_boundary_zone is not a SpatVector.")
assert_true(nrow(escape_boundary_zone) > 0, "escape_boundary_zone has no features.")

nrow(escape_boundary_zone)
```

```{r plot-road-aware-zone, eval = run_diagnostics && example_data_available, fig.height = 7}
plot(escape_boundary_zone, col = "grey90", border = NA, main = "Road-aware escape-boundary zone")
plot(zones$hazard_zone, add = TRUE, col = adjustcolor("tomato", alpha.f = 0.45), border = NA)
plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.45), lwd = 0.35)
```

# 8. Compare escape-point methods

This section intentionally compares three methods:

1. Wrong: land-only `hazard_zone`.
2. Better: TEZ + water `escape_zone`.
3. Best for this workflow: road-aware escape boundary plus inset-cropped roads.

```{r compare-escape-points, eval = run_diagnostics && example_data_available}
escape_wrong <- try(
  find_escape_points(
    hazard_zone = zones$hazard_zone,
    roads = roads_clean
  ),
  silent = TRUE
)

escape_water <- find_escape_points(
  hazard_zone = zones$escape_zone,
  roads = roads_for_escape
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

comparison <- data.frame(
  method = c(
    "wrong: land-only hazard zone",
    "better: TEZ + water, inset roads",
    "preferred: road-aware escape zone, inset roads"
  ),
  escape_point_count = c(
    if (inherits(escape_wrong, "try-error")) NA_integer_ else nrow(escape_wrong),
    nrow(escape_water),
    nrow(escape_points)
  )
)
comparison

assert_true(nrow(escape_points) > 0, "Preferred escape-point calculation returned no points.")
```

```{r plot-escape-comparison, eval = run_diagnostics && example_data_available, fig.height = 8}
oldpar <- par(mfrow = c(1, 3))

plot(zones$hazard_zone, col = "grey90", border = "grey40", main = "Wrong\nland-only boundary")
plot(roads_clean, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
if (!inherits(escape_wrong, "try-error")) {
  plot(escape_wrong, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
}

plot(zones$escape_zone, col = "grey90", border = "grey40", main = "Better\nTEZ + water")
plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
plot(escape_water, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

plot(escape_boundary_zone, col = "grey90", border = "grey40", main = "Preferred\nroad-aware boundary")
plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

par(oldpar)
```

From this point forward, the diagnostic uses:

```text
hazard_zone          = zones$hazard_zone
escape_zone          = zones$escape_zone
roads_for_escape     = inset-cropped roads
escape_boundary_zone = road-aware TEZ + water boundary
escape_points        = preferred escape points
```

```{r assign-core-objects, eval = run_diagnostics && example_data_available}
hazard_zone <- zones$hazard_zone
escape_zone <- zones$escape_zone
dem <- zones$dem
roads <- roads_clean
```

# 9. Prepare core inputs with `prepare_evac_inputs()`

`prepare_evac_inputs()` reads, cleans, projects, and converts the land-only `hazard_zone` into a polygon if needed. It does not create the road-aware escape boundary; that is handled separately above.

```{r prepare-core-inputs, eval = run_diagnostics && example_data_available}
inputs <- prepare_evac_inputs(
  hazard_zone = hazard_zone,
  roads = roads,
  dem = dem,
  target_crs = params$target_crs,
  hazard_as_polygon = TRUE,
  dissolve_hazard = TRUE,
  road_exclude = NULL
)

hazard_zone <- inputs$hazard_zone
roads <- inputs$roads
dem <- inputs$dem

assert_true(is_spatvector(hazard_zone), "Prepared hazard_zone is not a SpatVector.")
assert_true(is_spatvector(roads), "Prepared roads are not a SpatVector.")
assert_true(is_spatraster(dem), "Prepared DEM is not a SpatRaster.")
```

# 10. Create the evacuation grid with `make_evac_grid()`

The evacuation grid should be created from the land-only `hazard_zone`, not the water-combined or road-aware escape zone.

```{r make-grid, eval = run_diagnostics && example_data_available}
grid_resolution <- terra::res(dem) * params$grid_multiplier

evac_grid <- make_evac_grid(
  hazard_zone = hazard_zone,
  resolution = grid_resolution
)

assert_true(is_spatvector(evac_grid), "make_evac_grid() did not return a SpatVector.")
assert_true(nrow(evac_grid) > 0, "make_evac_grid() returned no grid cells.")

cat("Grid resolution:", paste(grid_resolution, collapse = " x "), "\n")
cat("Grid cells:", nrow(evac_grid), "\n")
```

```{r plot-grid, eval = run_diagnostics && example_data_available}
plot(hazard_zone, col = "grey95", border = "grey50", main = "Evacuation grid over land-only TEZ")
plot(evac_grid, add = TRUE, border = adjustcolor("black", alpha.f = 0.25), col = NA)
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.45), lwd = 0.35)
```

# 11. Create the road mask with `make_road_mask()`

`make_road_mask()` buffers the roads and preferred escape points, then combines them. The mask constrains least-cost movement to the road/pathway network while still allowing access around escape points.

```{r make-road-mask, eval = run_diagnostics && example_data_available}
road_mask_parts <- make_road_mask(
  roads = roads,
  escape_points = escape_points,
  road_buffer_m = params$road_buffer_m,
  escape_buffer_m = params$escape_buffer_m,
  return_components = TRUE
)

road_mask <- road_mask_parts$mask
roads_buffer <- road_mask_parts$roads_buffer
escape_buffer <- road_mask_parts$escape_buffer

assert_true(is_spatvector(road_mask), "Road mask is not a SpatVector.")
assert_true(is_spatvector(roads_buffer), "roads_buffer is not a SpatVector.")
assert_true(is_spatvector(escape_buffer), "escape_buffer is not a SpatVector.")
```

```{r plot-road-mask, eval = run_diagnostics && example_data_available}
plot(hazard_zone, col = "grey95", border = "grey60", main = "Road-constrained movement mask")
plot(road_mask, add = TRUE, col = adjustcolor("grey30", alpha.f = 0.35), border = NA)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 12. Create road origin points with `make_road_origins()`

Origins should be road-based points inside the land-only inundation zone.

```{r make-road-origins, eval = run_diagnostics && example_data_available}
road_points <- make_road_origins(
  evac_grid = evac_grid,
  roads_buffer = roads_buffer,
  hazard_zone = hazard_zone,
  max_origins = params$max_origins,
  seed = params$seed
)

assert_true(is_spatvector(road_points), "make_road_origins() did not return a SpatVector.")
assert_true(nrow(road_points) > 0, "make_road_origins() returned no origins.")

cat("Road origin points retained:", nrow(road_points), "\n")
```

```{r plot-road-origins, eval = run_diagnostics && example_data_available}
plot(hazard_zone, col = "grey95", border = "grey60", main = "Road origin points inside land-only TEZ")
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
plot(road_points, add = TRUE, pch = 21, bg = "dodgerblue", col = "black", cex = 0.45)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 13. Create the conductance surface with `make_conductance_surface()`

The conductance surface is built from the DEM but masked to the road-constrained movement area. This is the expensive part. `dem_resolution` can be coarsened for testing.

```{r make-conductance, eval = run_diagnostics && example_data_available}
dem_resolution <- terra::res(dem) * params$dem_multiplier

conductance <- make_conductance_surface(
  dem = dem,
  road_mask = road_mask,
  resolution = dem_resolution
)

cs_raster <- leastcostpath::rasterise(conductance)

assert_true(is_spatraster(cs_raster), "Conductance rasterization failed.")

cat("Conductance raster resolution:", paste(terra::res(cs_raster), collapse = " x "), "\n")
cat("Conductance raster cells:", terra::ncell(cs_raster), "\n")
```

```{r plot-conductance, eval = run_diagnostics && example_data_available}
plot(cs_raster, main = "Road-constrained conductance surface")
plot(hazard_zone, add = TRUE, border = "black", col = NA)
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 1), lwd = 0.35)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 14. Calculate least-cost distance with `calc_min_distance_to_safety()`

This section is optional because it can be slow for large study areas. The packaged example area is small, so the default parameters use all available origins and destinations.

```{r sample-destinations, eval = run_diagnostics && example_data_available}
escape_points_ltd <- escape_points
if (!is.null(params$max_destinations) && params$max_destinations < nrow(escape_points_ltd)) {
  set.seed(params$seed)
  escape_points_ltd <- escape_points_ltd[sort(sample.int(nrow(escape_points_ltd), params$max_destinations)), ]
}

cat("Escape points available:", nrow(escape_points), "\n")
cat("Escape points used in diagnostic LCP:", nrow(escape_points_ltd), "\n")
```

```{r calc-distance, eval = run_diagnostics && example_data_available && run_lcp}
distance_points <- calc_min_distance_to_safety(
  cs = conductance,
  origins = road_points,
  destinations = escape_points_ltd,
  include_destinations = TRUE,
  progress = TRUE,
  progress_every = 500,
  check_locations = FALSE
)

assert_true(is_spatvector(distance_points), "calc_min_distance_to_safety() did not return a SpatVector.")
assert_true("distance" %in% names(distance_points), "distance_points does not contain a distance field.")

summary(distance_points$distance)
table(distance_points$type)
```

```{r plot-distance-points, eval = run_diagnostics && example_data_available && run_lcp}
plot(hazard_zone, col = "grey95", border = "grey60", main = "Minimum least-cost distance points")
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.30), lwd = 0.35)
plot(distance_points, "distance", add = TRUE)
```

# 15. Convert distance to time with `calc_evac_time()`

`calc_evac_time()` converts distance to time using a walking speed. The default paper-style value is 1.22 m/s, but local planning scenarios should modify it.

```{r calc-time, eval = run_diagnostics && example_data_available && run_lcp}
example_distances <- c(0, 100, 500, 1000, 2000)
data.frame(
  distance_m = example_distances,
  time_min = calc_evac_time(
    distance_m = example_distances,
    walking_speed_mps = params$walking_speed_mps,
    units = "minutes"
  )
)
```

# 16. Create evacuation polygons with `make_evac_polygons()`

`make_evac_polygons()` converts the distance points to Voronoi polygons, calculates evacuation time, and clips the result to the selected output area. For the broad time grid, the clip area should be the land-only `hazard_zone`.

```{r make-polygons, eval = run_diagnostics && example_data_available && run_lcp}
evac_polygons <- make_evac_polygons(
  distance_points = distance_points,
  clip_area = hazard_zone,
  walking_speed_mps = params$walking_speed_mps,
  region_name = params$region_name
)

assert_true(is_spatvector(evac_polygons), "make_evac_polygons() did not return a SpatVector.")
assert_true("EvacTimeAvg" %in% names(evac_polygons), "EvacTimeAvg field is missing.")

summary(evac_polygons$EvacTimeAvg)
```

```{r plot-evac-polygons, eval = run_diagnostics && example_data_available && run_lcp, fig.height = 7}
plot(evac_polygons, "EvacTimeAvg", border = NA, main = "Evacuation-time polygons")
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.30), lwd = 0.35)
plot(escape_points_ltd, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 17. Run the high-level wrapper with `run_evacpath()`

After the individual steps work, the same logic should run through `run_evacpath()`. The key tsunami-specific arguments are:

```r
hazard_zone = zones$hazard_zone
escape_zone = zones$escape_zone
escape_roads_inset_x_m = 50
escape_roads_inset_y_m = 50
road_aware_escape_zone = TRUE
```

```{r run-wrapper, eval = run_diagnostics && example_data_available && run_lcp, message=FALSE,echo=FALSE, eval = FALSE}
result <- run_evacpath(
  hazard_zone = zones$hazard_zone,
  escape_zone = zones$escape_zone,
  roads = roads_clean,
  dem = zones$dem,
  target_crs = params$target_crs,
  region_name = params$region_name,
  grid_resolution = terra::res(zones$dem) * params$grid_multiplier,
  dem_resolution = terra::res(zones$dem) * params$dem_multiplier,
  road_buffer_m = params$road_buffer_m,
  escape_buffer_m = params$escape_buffer_m,
  final_road_buffer_m = params$final_road_buffer_m,
  escape_roads_inset_x_m = params$escape_roads_inset_x_m,
  escape_roads_inset_y_m = params$escape_roads_inset_y_m,
  road_aware_escape_zone = params$road_aware_escape_zone,
  escape_zone_road_buffer_m = params$road_buffer_m,
  escape_zone_crop_buffer_m = params$final_road_buffer_m,
  max_origins = params$max_origins,
  max_destinations = params$max_destinations,
  seed = params$seed,
  walking_speed_mps = params$walking_speed_mps,
  clip_mode = "hazard",
  progress = TRUE,
  progress_every = 500,
  lcp_check_locations = FALSE
)

result

assert_true("escape_boundary_zone" %in% names(result), "Result does not contain escape_boundary_zone.")
assert_true("roads_for_escape" %in% names(result), "Result does not contain roads_for_escape.")
assert_true("time_grid" %in% names(result), "Result does not contain time_grid.")
assert_true("EvacTimeAvg" %in% names(result$time_grid), "Result time_grid lacks EvacTimeAvg.")
```

# 18. Final map check

This map uses land/water as background, the land-only evacuation-time polygons as the main layer, and the preferred escape points as squares.

```{r final-map, eval = run_diagnostics && example_data_available && run_lcp, fig.width = 9, fig.height = 7, eval = FALSE}
evac_map <- result$time_grid
roads_plot <- crop(result$roads, evac_map)
escape_plot <- crop(result$escape_points, evac_map)

dem_bg <- result$dem
if (!same.crs(dem_bg, evac_map)) {
  dem_bg <- project(dem_bg, crs(evac_map))
}

e <- ext(evac_map)
xpad <- (xmax(e) - xmin(e)) * 0.06
ypad <- (ymax(e) - ymin(e)) * 0.06
e2 <- ext(xmin(e) - xpad, xmax(e) + xpad, ymin(e) - ypad, ymax(e) + ypad)

dem_bg <- crop(dem_bg, e2)
land_water <- ifel(dem_bg > params$land_threshold, 2, 1)
names(land_water) <- "land_water"

bg_cols <- c(
  adjustcolor("lightskyblue1", alpha.f = 0.65),
  adjustcolor("grey92", alpha.f = 1.00)
)
time_cols <- hcl.colors(100, "YlOrRd", rev = TRUE)

oldpar <- par(mar = c(4, 4, 3, 5), mgp = c(2.2, 0.7, 0), las = 1, bg = "white", xpd = FALSE)

plot(
  land_water,
  ext = e2,
  col = bg_cols,
  legend = FALSE,
  axes = TRUE,
  box = FALSE,
  main = "Modeled Pedestrian Evacuation Time",
  cex.main = 1.1
)

plot(
  evac_map,
  "EvacTimeAvg",
  add = TRUE,
  col = time_cols,
  border = NA,
  plg = list(title = "Minutes", cex = 0.75, title.cex = 0.85, x = "right")
)

plot(roads_plot, add = TRUE, col = adjustcolor("grey25", alpha.f = 0.30), lwd = 0.30)
plot(escape_plot, add = TRUE, pch = 22, bg = adjustcolor("red", alpha.f = 0.75), col = "black", cex = 0.38, lwd = 0.35)

legend(
  "topright",
  legend = c("Escape / safety points", "Roads", "Land", "Water"),
  pch = c(22, NA, 15, 15),
  pt.bg = c("red", NA, "grey92", "lightskyblue1"),
  col = c("black", adjustcolor("grey25", alpha.f = 0.5), "grey92", "lightskyblue1"),
  lty = c(NA, 1, NA, NA),
  lwd = c(NA, 1, NA, NA),
  pt.cex = c(0.8, NA, 1.1, 1.1),
  bty = "n",
  cex = 0.7,
  inset = 0.02
)
par(oldpar)
```

# 19. Write outputs with `write_evac_outputs()`

`write_evac_outputs()` writes the major spatial outputs from a completed result object. This should only run after the diagnostic workflow succeeds.

```{r write-outputs, eval = run_diagnostics && example_data_available && run_lcp, eval = FALSE}
write_evac_outputs(
  result,
  output_dir = params$output_dir,
  prefix = "example"
)

list.files(params$output_dir, full.names = TRUE)
```

# 20. Troubleshooting checklist

If escape points look wrong, check these first:

1. Did you create both `hazard_zone` and `escape_zone` with `prepare_tsunami_zones()`?
2. Did you crop the roads used for escape detection with `crop_roads_to_inner_extent()`?
3. Did you combine buffered roads with the escape zone using `make_road_aware_escape_zone()`?
4. Did you use the road-aware boundary for `find_escape_points()`?
5. Does the DEM-derived land/water mask look correct?
6. If land/water are reversed, did you set `dem_sign_multiplier = -1`?
7. Are piers, ferry routes, or non-road features still present in the road layer?
8. Is the target CRS projected in meters?
9. Are you using `dem_resolution` to coarsen the conductance raster for testing, not only `grid_resolution`?

The expected tsunami pattern is:

```r
zones <- prepare_tsunami_zones(inundation, dem, target_crs = target_crs)

roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads,
  zone = zones$escape_zone,
  inset_x_m = 50,
  inset_y_m = 50
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 2,
  crop_buffer_m = 3
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

result <- run_evacpath(
  hazard_zone = zones$hazard_zone,
  escape_zone = zones$escape_zone,
  roads = roads,
  dem = zones$dem,
  target_crs = target_crs,
  escape_roads_inset_x_m = 50,
  escape_roads_inset_y_m = 50,
  road_aware_escape_zone = TRUE
)
```

# References

Cordero, E., Ruiz Vélez, R., Huérfano Moreno, V., Sherman, C., 2025. Enhancing tsunami evacuation strategies in Puerto Rico using open-source least-cost path analysis. J. Disaster Sci. Manag. 1, 18. https://doi.org/10.1007/s44367-025-00018-y

Lewis, J., 2023. leastcostpath: Modelling Pathways and Movement Potential Within a Landscape.

Lewis, J., 2021. Probabilistic Modelling for Incorporating Uncertainty in Least Cost Path Results: a Postdictive Roman Road Case Study. Journal of Archaeological Method and Theory 28, 911-924. https://doi.org/10.1007/s10816-021-09522-w

---
title: "Evacuation workflow: function-by-function diagnostics"
output: rmarkdown::html_vignette
params:
  run_diagnostics: false
  run_lcp: false
  data_dir: "data-raw"
  target_crs: "EPSG:32748"
  region_name: "Example region"
  dem_sign_multiplier: 1
  inundation_threshold: 0
  land_threshold: 0
  water_threshold: 0
  resample_method: "bilinear"
  road_exclude_field: "man_made"
  road_exclude_values: !r c("pier")
  grid_multiplier: 50
  dem_multiplier: 50
  road_buffer_m: 2
  escape_buffer_m: 5
  final_road_buffer_m: 3
  escape_roads_inset_x_m: 250
  escape_roads_inset_y_m: 250
  road_aware_escape_zone: true
  max_origins: null
  max_destinations: null
  walking_speed_mps: 1.22
  seed: 23401
  output_dir: "_outputs/example"
vignette: >
  %\VignetteIndexEntry{Evacuation workflow: function-by-function diagnostics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 6,
  dpi = 120
)
```

This vignette is a diagnostic walkthrough for the example tsunami-evacuation workflow. It is intentionally more detailed than a normal user vignette because its purpose is to test every major `evacpath` function independently before trusting the high-level `run_evacpath()` wrapper.

This example workflow has three important tsunami-specific complications:

1. The **land-only inundation zone** is the correct area for evacuation origins and final output mapping, but it is not necessarily the correct boundary for escape-point detection.
2. The **water-combined escape zone** prevents the coastline from becoming a fake safety boundary.
3. The **road-aware escape zone** combines buffered roads with the escape zone before escape points are generated. This preserves bridges, causeways, and walkways over water that can be lost when the tsunami layer is split into land and water masks.

The diagnostic workflow therefore uses these separate objects:

```text
hazard_zone          = land-only tsunami evacuation zone for origins/output
escape_zone          = land-only TEZ + water, used as base escape boundary
roads_for_escape     = roads cropped to an inset study extent
escape_boundary_zone = escape_zone + buffered roads_for_escape
```

To run the full diagnostic workflow locally:

```{r render-command, eval = FALSE}
rmarkdown::render(
  "vignettes/diagnostic-example.Rmd",
  params = list(
    run_diagnostics = TRUE,
    run_lcp = TRUE,
    data_dir = "data-raw",
    target_crs = "EPSG:32748",
    dem_sign_multiplier = 1,
    grid_multiplier = 50,
    dem_multiplier = 50,
    escape_roads_inset_x_m = 250,
    escape_roads_inset_y_m = 250,
    road_aware_escape_zone = TRUE
  )
)
```

For a faster diagnostic that stops before least-cost paths:

```{r render-fast, eval = FALSE}
rmarkdown::render(
  "vignettes/diagnostic-example.Rmd",
  params = list(
    run_diagnostics = TRUE,
    run_lcp = FALSE,
    data_dir = "data-raw"
  )
)
```

# 1. Setup

```{r setup}
# This loader works both when the package is installed and during local package development.
if (requireNamespace("evacpath", quietly = TRUE)) {
  library(evacpath)
} else if (requireNamespace("devtools", quietly = TRUE) && file.exists("DESCRIPTION")) {
  devtools::load_all(".")
} else if (requireNamespace("devtools", quietly = TRUE) && file.exists(file.path("evacpath", "DESCRIPTION"))) {
  devtools::load_all("evacpath")
} else {
  stop("Install evacpath or run this from the package root/parent directory.", call. = FALSE)
}

library(terra)

run_diagnostics <- isTRUE(params$run_diagnostics)
run_lcp <- isTRUE(params$run_lcp)

assert_true <- function(x, message) {
  if (!isTRUE(x)) {
    stop(message, call. = FALSE)
  }
  invisible(TRUE)
}

is_spatvector <- function(x) inherits(x, "SpatVector")
is_spatraster <- function(x) inherits(x, "SpatRaster")

count_features <- function(x) {
  if (inherits(x, "SpatRaster")) return(terra::ncell(x))
  if (inherits(x, "SpatVector")) return(nrow(x))
  NA_integer_
}
```

The diagnostic switches are:

```{r switches}
run_diagnostics
run_lcp
```

# 2. Locate example input files

The diagnostic expects these files:

```text
data-raw/dem.tif
data-raw/rds.gpkg
data-raw/tsunami_inundation_depth.tif
```

The function below searches for them from the package root, the parent folder, or installed package `extdata`.

```{r locate-files}
find_input_file <- function(filename, data_dir = params$data_dir) {
  candidate_dirs <- unique(c(
    data_dir,
    file.path("evacpath", data_dir),
    file.path(getwd(), data_dir),
    file.path(getwd(), "evacpath", data_dir),
    system.file("extdata", package = "evacpath")
  ))

  candidate_dirs <- candidate_dirs[nzchar(candidate_dirs)]
  candidates <- file.path(candidate_dirs, filename)
  hits <- candidates[file.exists(candidates)]

  if (length(hits) == 0L) {
    return(NA_character_)
  }

  normalizePath(hits[1])
}

dem_file <- find_input_file("dem.tif")
roads_file <- find_input_file("rds.gpkg")
inundation_file <- find_input_file("tsunami_inundation_depth.tif")

input_files <- data.frame(
  layer = c("DEM", "roads", "inundation depth"),
  file = c(dem_file, roads_file, inundation_file),
  exists = file.exists(c(dem_file, roads_file, inundation_file))
)

input_files

example_data_available <- all(input_files$exists)
example_data_available
```

If `example_data_available` is `FALSE`, check `params` or use the packaged files in `inst/extdata/`.

# 3. Read the spatial inputs with `read_spatial()`

`read_spatial()` accepts a file path or an existing `terra` object. Raster formats are read with `terra::rast()` and vector formats are read with `terra::vect()`.

```{r read-inputs, eval = run_diagnostics && example_data_available}
dem_raw <- read_spatial(dem_file)
roads_raw <- read_spatial(roads_file)
inundation_raw <- read_spatial(inundation_file)

assert_true(is_spatraster(dem_raw), "DEM was not read as a SpatRaster.")
assert_true(is_spatvector(roads_raw), "Roads were not read as a SpatVector.")
assert_true(is_spatraster(inundation_raw), "Inundation input was not read as a SpatRaster.")

list(
  dem_class = class(dem_raw),
  roads_class = class(roads_raw),
  inundation_class = class(inundation_raw)
)
```

## 3.1 Raw input diagnostics

Before modeling, inspect CRS, extent, resolution, raster value ranges, and road attributes.

```{r raw-diagnostics, eval = run_diagnostics && example_data_available}
cat("DEM CRS:\n", terra::crs(dem_raw), "\n\n")
cat("Roads CRS:\n", terra::crs(roads_raw), "\n\n")
cat("Inundation CRS:\n", terra::crs(inundation_raw), "\n\n")

cat("DEM resolution:\n")
print(terra::res(dem_raw))

cat("Inundation resolution:\n")
print(terra::res(inundation_raw))

cat("DEM cell count:\n")
print(terra::ncell(dem_raw))

cat("Road feature count:\n")
print(nrow(roads_raw))

cat("Road attribute names:\n")
print(names(roads_raw))

cat("DEM range:\n")
print(terra::global(dem_raw, fun = range, na.rm = TRUE))

cat("Inundation range:\n")
print(terra::global(inundation_raw, fun = range, na.rm = TRUE))
```

```{r plot-raw-inputs, eval = run_diagnostics && example_data_available, fig.height = 7}
par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
plot(dem_raw, main = "Raw DEM")
plot(inundation_raw, main = "Raw inundation depth")
par(mfrow = c(1, 1))
```

# 4. Clean roads with `clean_roads()`

This is where region-specific road filtering belongs. In the packaged example, piers are excluded because they can create misleading road-water intersections.

```{r clean-roads, eval = run_diagnostics && example_data_available}
road_exclude <- NULL
if (!is.null(params$road_exclude_field) && nzchar(params$road_exclude_field)) {
  road_exclude <- list(
    field = params$road_exclude_field,
    values = params$road_exclude_values
  )
}

roads_clean <- clean_roads(
  roads = roads_raw,
  exclude = road_exclude,
  target_crs = params$target_crs
)

assert_true(is_spatvector(roads_clean), "clean_roads() did not return a SpatVector.")
assert_true(nrow(roads_clean) > 0, "clean_roads() returned no roads.")

nrow(roads_raw)
nrow(roads_clean)
```

# 5. Prepare tsunami zones with `prepare_tsunami_zones()`

This is the critical tsunami-specific step. The old script created a land mask, water mask, land-only TEZ, and then a TEZ-plus-water layer. The package makes that separation explicit.

```{r prepare-tsunami-zones, eval = run_diagnostics && example_data_available}
zones <- prepare_tsunami_zones(
  inundation = inundation_raw,
  dem = dem_raw,
  target_crs = params$target_crs,
  inundation_threshold = params$inundation_threshold,
  land_threshold = params$land_threshold,
  water_threshold = params$water_threshold,
  dem_sign_multiplier = params$dem_sign_multiplier,
  resample_method = params$resample_method,
  as_polygon = TRUE,
  dissolve = TRUE
)

zones

assert_true(is_spatvector(zones$hazard_zone), "zones$hazard_zone is not a SpatVector.")
assert_true(is_spatvector(zones$escape_zone), "zones$escape_zone is not a SpatVector.")
assert_true(is_spatraster(zones$hazard_raster), "zones$hazard_raster is not a SpatRaster.")
assert_true(is_spatraster(zones$escape_raster), "zones$escape_raster is not a SpatRaster.")

cat("Land-only hazard-zone polygons:", nrow(zones$hazard_zone), "\n")
cat("Water-combined escape-zone polygons:", nrow(zones$escape_zone), "\n")
```

## 5.1 Check the land/water classification

If land and water are reversed, change `dem_sign_multiplier` from `1` to `-1` or adjust the thresholds.

```{r land-water-check, eval = run_diagnostics && example_data_available, fig.height = 7}
par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
plot(zones$land_mask, col = "grey85", legend = FALSE, main = "DEM-derived land mask")
plot(zones$water_mask, col = "lightskyblue1", legend = FALSE, main = "DEM-derived water mask")
par(mfrow = c(1, 1))
```

## 5.2 Check the two tsunami zones

The land-only zone is for origins and final mapped evacuation time. The water-combined zone is the base zone for escape-point detection.

```{r plot-tsunami-zones, eval = run_diagnostics && example_data_available, fig.height = 7}
par(mfrow = c(1, 2), mar = c(3, 3, 3, 5), las = 1)
plot(zones$hazard_zone, col = "tomato", border = NA, main = "hazard_zone: land-only TEZ")
plot(zones$escape_zone, col = "grey90", border = NA, main = "escape_zone: TEZ + water")
plot(zones$hazard_zone, add = TRUE, col = "tomato", border = NA)
par(mfrow = c(1, 1))
```

# 6. Crop roads to an inner extent before escape-point detection

If the road dataset extends beyond the reliable hazard-zone coverage, roads can intersect the artificial rectangular study-area boundary. Those intersections are not real escape points. The fix is to crop the roads used for escape-point detection to a slightly inset extent. The x and y insets are controlled separately.

```{r crop-roads-inner-extent, eval = run_diagnostics && example_data_available}
roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads_clean,
  zone = zones$escape_zone,
  inset_x_m = params$escape_roads_inset_x_m,
  inset_y_m = params$escape_roads_inset_y_m
)

assert_true(is_spatvector(roads_for_escape), "roads_for_escape is not a SpatVector.")
assert_true(nrow(roads_for_escape) > 0, "roads_for_escape has no features.")

cat("Clean roads:", nrow(roads_clean), "\n")
cat("Roads used for escape-point detection:", nrow(roads_for_escape), "\n")
```

```{r plot-roads-inner-extent, eval = run_diagnostics && example_data_available, fig.height = 7}
plot(zones$escape_zone, col = "grey92", border = "grey60", main = "Roads cropped to inset extent for escape detection")
plot(roads_clean, add = TRUE, col = adjustcolor("red", alpha.f = 0.25), lwd = 0.35)
plot(roads_for_escape, add = TRUE, col = "black", lwd = 0.45)
legend(
  "topright",
  legend = c("Original cleaned roads", "Roads used for escape detection"),
  col = c(adjustcolor("red", alpha.f = 0.5), "black"),
  lty = 1,
  lwd = 1,
  bty = "n",
  cex = 0.8
)
```

# 7. Combine buffered roads with the escape zone

This is the second major correction. The buffered roads should be combined with the tsunami escape zone **before** escape points are generated. Otherwise, bridge, causeway, or walkway segments over water can be lost because the land and water masks do not always represent the accessible transportation surface.

The package helper is `make_road_aware_escape_zone()`.

```{r road-aware-escape-zone, eval = run_diagnostics && example_data_available}
escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = params$road_buffer_m,
  crop_buffer_m = params$final_road_buffer_m,
  include_base_zone = TRUE
)

assert_true(is_spatvector(escape_boundary_zone), "escape_boundary_zone is not a SpatVector.")
assert_true(nrow(escape_boundary_zone) > 0, "escape_boundary_zone has no features.")

nrow(escape_boundary_zone)
```

```{r plot-road-aware-zone, eval = run_diagnostics && example_data_available, fig.height = 7}
plot(escape_boundary_zone, col = "grey90", border = NA, main = "Road-aware escape-boundary zone")
plot(zones$hazard_zone, add = TRUE, col = adjustcolor("tomato", alpha.f = 0.45), border = NA)
plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.45), lwd = 0.35)
```

# 8. Compare escape-point methods

This section intentionally compares three methods:

1. Wrong: land-only `hazard_zone`.
2. Better: TEZ + water `escape_zone`.
3. Best for this workflow: road-aware escape boundary plus inset-cropped roads.

```{r compare-escape-points, eval = run_diagnostics && example_data_available}
escape_wrong <- try(
  find_escape_points(
    hazard_zone = zones$hazard_zone,
    roads = roads_clean
  ),
  silent = TRUE
)

escape_water <- find_escape_points(
  hazard_zone = zones$escape_zone,
  roads = roads_for_escape
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

comparison <- data.frame(
  method = c(
    "wrong: land-only hazard zone",
    "better: TEZ + water, inset roads",
    "preferred: road-aware escape zone, inset roads"
  ),
  escape_point_count = c(
    if (inherits(escape_wrong, "try-error")) NA_integer_ else nrow(escape_wrong),
    nrow(escape_water),
    nrow(escape_points)
  )
)
comparison

assert_true(nrow(escape_points) > 0, "Preferred escape-point calculation returned no points.")
```

```{r plot-escape-comparison, eval = run_diagnostics && example_data_available, fig.height = 8}
par(mfrow = c(1, 3), mar = c(3, 3, 3, 1), las = 1)

plot(zones$hazard_zone, col = "grey90", border = "grey40", main = "Wrong\nland-only boundary")
plot(roads_clean, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
if (!inherits(escape_wrong, "try-error")) {
  plot(escape_wrong, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
}

plot(zones$escape_zone, col = "grey90", border = "grey40", main = "Better\nTEZ + water")
plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
plot(escape_water, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

plot(escape_boundary_zone, col = "grey90", border = "grey40", main = "Preferred\nroad-aware boundary")
plot(roads_for_escape, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)

par(mfrow = c(1, 1))
```

From this point forward, the diagnostic uses:

```text
hazard_zone          = zones$hazard_zone
escape_zone          = zones$escape_zone
roads_for_escape     = inset-cropped roads
escape_boundary_zone = road-aware TEZ + water boundary
escape_points        = preferred escape points
```

```{r assign-core-objects, eval = run_diagnostics && example_data_available}
hazard_zone <- zones$hazard_zone
escape_zone <- zones$escape_zone
dem <- zones$dem
roads <- roads_clean
```

# 9. Prepare core inputs with `prepare_evac_inputs()`

`prepare_evac_inputs()` reads, cleans, projects, and converts the land-only `hazard_zone` into a polygon if needed. It does not create the road-aware escape boundary; that is handled separately above.

```{r prepare-core-inputs, eval = run_diagnostics && example_data_available}
inputs <- prepare_evac_inputs(
  hazard_zone = hazard_zone,
  roads = roads,
  dem = dem,
  target_crs = params$target_crs,
  hazard_as_polygon = TRUE,
  dissolve_hazard = TRUE,
  road_exclude = NULL
)

hazard_zone <- inputs$hazard_zone
roads <- inputs$roads
dem <- inputs$dem

assert_true(is_spatvector(hazard_zone), "Prepared hazard_zone is not a SpatVector.")
assert_true(is_spatvector(roads), "Prepared roads are not a SpatVector.")
assert_true(is_spatraster(dem), "Prepared DEM is not a SpatRaster.")
```

# 10. Create the evacuation grid with `make_evac_grid()`

The evacuation grid should be created from the land-only `hazard_zone`, not the water-combined or road-aware escape zone.

```{r make-grid, eval = run_diagnostics && example_data_available}
grid_resolution <- terra::res(dem) * params$grid_multiplier

evac_grid <- make_evac_grid(
  hazard_zone = hazard_zone,
  resolution = grid_resolution
)

assert_true(is_spatvector(evac_grid), "make_evac_grid() did not return a SpatVector.")
assert_true(nrow(evac_grid) > 0, "make_evac_grid() returned no grid cells.")

cat("Grid resolution:", paste(grid_resolution, collapse = " x "), "\n")
cat("Grid cells:", nrow(evac_grid), "\n")
```

```{r plot-grid, eval = run_diagnostics && example_data_available}
plot(hazard_zone, col = "grey95", border = "grey50", main = "Evacuation grid over land-only TEZ")
plot(evac_grid, add = TRUE, border = adjustcolor("black", alpha.f = 0.25), col = NA)
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.45), lwd = 0.35)
```

# 11. Create the road mask with `make_road_mask()`

`make_road_mask()` buffers the roads and preferred escape points, then combines them. The mask constrains least-cost movement to the road/pathway network while still allowing access around escape points.

```{r make-road-mask, eval = run_diagnostics && example_data_available}
road_mask_parts <- make_road_mask(
  roads = roads,
  escape_points = escape_points,
  road_buffer_m = params$road_buffer_m,
  escape_buffer_m = params$escape_buffer_m,
  return_components = TRUE
)

road_mask <- road_mask_parts$mask
roads_buffer <- road_mask_parts$roads_buffer
escape_buffer <- road_mask_parts$escape_buffer

assert_true(is_spatvector(road_mask), "Road mask is not a SpatVector.")
assert_true(is_spatvector(roads_buffer), "roads_buffer is not a SpatVector.")
assert_true(is_spatvector(escape_buffer), "escape_buffer is not a SpatVector.")
```

```{r plot-road-mask, eval = run_diagnostics && example_data_available}
plot(hazard_zone, col = "grey95", border = "grey60", main = "Road-constrained movement mask")
plot(road_mask, add = TRUE, col = adjustcolor("grey30", alpha.f = 0.35), border = NA)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 12. Create road origin points with `make_road_origins()`

Origins should be road-based points inside the land-only inundation zone.

```{r make-road-origins, eval = run_diagnostics && example_data_available}
road_points <- make_road_origins(
  evac_grid = evac_grid,
  roads_buffer = roads_buffer,
  max_origins = params$max_origins,
  seed = params$seed
)

assert_true(is_spatvector(road_points), "make_road_origins() did not return a SpatVector.")
assert_true(nrow(road_points) > 0, "make_road_origins() returned no origins.")

cat("Road origin points retained:", nrow(road_points), "\n")
```

```{r plot-road-origins, eval = run_diagnostics && example_data_available}
plot(hazard_zone, col = "grey95", border = "grey60", main = "Road origin points inside land-only TEZ")
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.35), lwd = 0.35)
plot(road_points, add = TRUE, pch = 21, bg = "dodgerblue", col = "black", cex = 0.45)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 13. Create the conductance surface with `make_conductance_surface()`

The conductance surface is built from the DEM but masked to the road-constrained movement area. This is the expensive part. `dem_resolution` can be coarsened for testing.

```{r make-conductance, eval = run_diagnostics && example_data_available}
dem_resolution <- terra::res(dem) * params$dem_multiplier

conductance <- make_conductance_surface(
  dem = dem,
  road_mask = road_mask,
  resolution = dem_resolution
)

cs_raster <- leastcostpath::rasterise(conductance)

assert_true(is_spatraster(cs_raster), "Conductance rasterization failed.")

cat("Conductance raster resolution:", paste(terra::res(cs_raster), collapse = " x "), "\n")
cat("Conductance raster cells:", terra::ncell(cs_raster), "\n")
```

```{r plot-conductance, eval = run_diagnostics && example_data_available}
plot(cs_raster, main = "Road-constrained conductance surface")
plot(hazard_zone, add = TRUE, border = "black", col = NA)
plot(escape_points, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 14. Calculate least-cost distance with `calc_min_distance_to_safety()`

This section is optional because it can be slow for large study areas. The packaged example area is small, so the default parameters use all available origins and destinations.

```{r sample-destinations, eval = run_diagnostics && example_data_available}
escape_points_ltd <- escape_points
if (!is.null(params$max_destinations) && params$max_destinations < nrow(escape_points_ltd)) {
  set.seed(params$seed)
  escape_points_ltd <- escape_points_ltd[sort(sample.int(nrow(escape_points_ltd), params$max_destinations)), ]
}

cat("Escape points available:", nrow(escape_points), "\n")
cat("Escape points used in diagnostic LCP:", nrow(escape_points_ltd), "\n")
```

```{r calc-distance, eval = run_diagnostics && example_data_available && run_lcp}
distance_points <- calc_min_distance_to_safety(
  cs = conductance,
  origins = road_points,
  destinations = escape_points_ltd,
  include_destinations = TRUE,
  progress = TRUE,
  progress_every = 1,
  check_locations = FALSE
)

assert_true(is_spatvector(distance_points), "calc_min_distance_to_safety() did not return a SpatVector.")
assert_true("distance" %in% names(distance_points), "distance_points does not contain a distance field.")

summary(distance_points$distance)
table(distance_points$type)
```

```{r plot-distance-points, eval = run_diagnostics && example_data_available && run_lcp}
plot(hazard_zone, col = "grey95", border = "grey60", main = "Minimum least-cost distance points")
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.30), lwd = 0.35)
plot(distance_points, "distance", add = TRUE, pch = 21, cex = 0.55)
```

# 15. Convert distance to time with `calc_evac_time()`

`calc_evac_time()` converts distance to time using a walking speed. The default paper-style value is 1.22 m/s, but local planning scenarios should modify it.

```{r calc-time, eval = run_diagnostics && example_data_available && run_lcp}
example_distances <- c(0, 100, 500, 1000, 2000)
data.frame(
  distance_m = example_distances,
  time_min = calc_evac_time(
    distance_m = example_distances,
    walking_speed_mps = params$walking_speed_mps,
    units = "minutes"
  )
)
```

# 16. Create evacuation polygons with `make_evac_polygons()`

`make_evac_polygons()` converts the distance points to Voronoi polygons, calculates evacuation time, and clips the result to the selected output area. For the broad time grid, the clip area should be the land-only `hazard_zone`.

```{r make-polygons, eval = run_diagnostics && example_data_available && run_lcp}
evac_polygons <- make_evac_polygons(
  distance_points = distance_points,
  clip_area = hazard_zone,
  walking_speed_mps = params$walking_speed_mps,
  region_name = params$region_name
)

assert_true(is_spatvector(evac_polygons), "make_evac_polygons() did not return a SpatVector.")
assert_true("EvacTimeAvg" %in% names(evac_polygons), "EvacTimeAvg field is missing.")

summary(evac_polygons$EvacTimeAvg)
```

```{r plot-evac-polygons, eval = run_diagnostics && example_data_available && run_lcp, fig.height = 7}
plot(evac_polygons, "EvacTimeAvg", border = NA, main = "Evacuation-time polygons")
plot(roads, add = TRUE, col = adjustcolor("grey20", alpha.f = 0.30), lwd = 0.35)
plot(escape_points_ltd, add = TRUE, pch = 22, bg = "red", col = "black", cex = 0.45)
```

# 17. Run the high-level wrapper with `run_evacpath()`

After the individual steps work, the same logic should run through `run_evacpath()`. The key tsunami-specific arguments are:

```r
hazard_zone = zones$hazard_zone
escape_zone = zones$escape_zone
escape_roads_inset_x_m = 250
escape_roads_inset_y_m = 250
road_aware_escape_zone = TRUE
```

```{r run-wrapper, eval = run_diagnostics && example_data_available && run_lcp}
result <- run_evacpath(
  hazard_zone = zones$hazard_zone,
  escape_zone = zones$escape_zone,
  roads = roads_clean,
  dem = zones$dem,
  target_crs = params$target_crs,
  region_name = params$region_name,
  grid_resolution = terra::res(zones$dem) * params$grid_multiplier,
  dem_resolution = terra::res(zones$dem) * params$dem_multiplier,
  road_buffer_m = params$road_buffer_m,
  escape_buffer_m = params$escape_buffer_m,
  final_road_buffer_m = params$final_road_buffer_m,
  escape_roads_inset_x_m = params$escape_roads_inset_x_m,
  escape_roads_inset_y_m = params$escape_roads_inset_y_m,
  road_aware_escape_zone = params$road_aware_escape_zone,
  escape_zone_road_buffer_m = params$road_buffer_m,
  escape_zone_crop_buffer_m = params$final_road_buffer_m,
  max_origins = params$max_origins,
  max_destinations = params$max_destinations,
  seed = params$seed,
  walking_speed_mps = params$walking_speed_mps,
  clip_mode = "hazard",
  progress = TRUE,
  progress_every = 1,
  lcp_check_locations = FALSE
)

result

assert_true("escape_boundary_zone" %in% names(result), "Result does not contain escape_boundary_zone.")
assert_true("roads_for_escape" %in% names(result), "Result does not contain roads_for_escape.")
assert_true("time_grid" %in% names(result), "Result does not contain time_grid.")
assert_true("EvacTimeAvg" %in% names(result$time_grid), "Result time_grid lacks EvacTimeAvg.")
```

# 18. Terra-only final map

This map uses land/water as background, the land-only evacuation-time polygons as the main layer, and the preferred escape points as squares.

```{r final-map, eval = run_diagnostics && example_data_available && run_lcp, fig.width = 9, fig.height = 7}
evac_map <- result$time_grid
roads_plot <- crop(result$roads, evac_map)
escape_plot <- crop(result$escape_points, evac_map)

dem_bg <- result$dem
if (!same.crs(dem_bg, evac_map)) {
  dem_bg <- project(dem_bg, crs(evac_map))
}

e <- ext(evac_map)
xpad <- (xmax(e) - xmin(e)) * 0.06
ypad <- (ymax(e) - ymin(e)) * 0.06
e2 <- ext(xmin(e) - xpad, xmax(e) + xpad, ymin(e) - ypad, ymax(e) + ypad)

dem_bg <- crop(dem_bg, e2)
land_water <- ifel(dem_bg > params$land_threshold, 2, 1)
names(land_water) <- "land_water"

bg_cols <- c(
  adjustcolor("lightskyblue1", alpha.f = 0.65),
  adjustcolor("grey92", alpha.f = 1.00)
)
time_cols <- hcl.colors(100, "YlOrRd", rev = TRUE)

par(mar = c(4, 4, 3, 5), mgp = c(2.2, 0.7, 0), las = 1, bg = "white", xpd = FALSE)

plot(
  land_water,
  ext = e2,
  col = bg_cols,
  legend = FALSE,
  axes = TRUE,
  box = FALSE,
  main = "Modeled Pedestrian Evacuation Time",
  cex.main = 1.1
)

plot(
  evac_map,
  "EvacTimeAvg",
  add = TRUE,
  col = time_cols,
  border = NA,
  plg = list(title = "Minutes", cex = 0.75, title.cex = 0.85, x = "right")
)

plot(roads_plot, add = TRUE, col = adjustcolor("grey25", alpha.f = 0.30), lwd = 0.30)
plot(escape_plot, add = TRUE, pch = 22, bg = adjustcolor("red", alpha.f = 0.75), col = "black", cex = 0.38, lwd = 0.35)

legend(
  "topright",
  legend = c("Escape / safety points", "Roads", "Land", "Water"),
  pch = c(22, NA, 15, 15),
  pt.bg = c("red", NA, "grey92", "lightskyblue1"),
  col = c("black", adjustcolor("grey25", alpha.f = 0.5), "grey92", "lightskyblue1"),
  lty = c(NA, 1, NA, NA),
  lwd = c(NA, 1, NA, NA),
  pt.cex = c(0.8, NA, 1.1, 1.1),
  bty = "n",
  cex = 0.7,
  inset = 0.02
)
```

# 19. Write outputs with `write_evac_outputs()`

`write_evac_outputs()` writes the major spatial outputs from a completed result object. This should only run after the diagnostic workflow succeeds.

```{r write-outputs, eval = run_diagnostics && example_data_available && run_lcp}
write_evac_outputs(
  result,
  output_dir = params$output_dir,
  prefix = "example"
)

list.files(params$output_dir, full.names = TRUE)
```

# 20. Troubleshooting checklist

If escape points look wrong, check these first:

1. Did you create both `hazard_zone` and `escape_zone` with `prepare_tsunami_zones()`?
2. Did you crop the roads used for escape detection with `crop_roads_to_inner_extent()`?
3. Did you combine buffered roads with the escape zone using `make_road_aware_escape_zone()`?
4. Did you use the road-aware boundary for `find_escape_points()`?
5. Does the DEM-derived land/water mask look correct?
6. If land/water are reversed, did you set `dem_sign_multiplier = -1`?
7. Are piers, ferry routes, or non-road features still present in the road layer?
8. Is the target CRS projected in meters?
9. Are you using `dem_resolution` to coarsen the conductance raster for testing, not only `grid_resolution`?

The expected tsunami pattern is:

```r
zones <- prepare_tsunami_zones(inundation, dem, target_crs = crs_jak)

roads_for_escape <- crop_roads_to_inner_extent(
  roads = roads,
  zone = zones$escape_zone,
  inset_x_m = 250,
  inset_y_m = 250
)

escape_boundary_zone <- make_road_aware_escape_zone(
  escape_zone = zones$escape_zone,
  roads = roads_for_escape,
  road_buffer_m = 2,
  crop_buffer_m = 3
)

escape_points <- find_escape_points(
  hazard_zone = escape_boundary_zone,
  roads = roads_for_escape
)

result <- run_evacpath(
  hazard_zone = zones$hazard_zone,
  escape_zone = zones$escape_zone,
  roads = roads,
  dem = zones$dem,
  target_crs = crs_jak,
  escape_roads_inset_x_m = 250,
  escape_roads_inset_y_m = 250,
  road_aware_escape_zone = TRUE
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evacpath-package.R
\docType{package}
\name{evacpath-package}
\alias{evacpath}
\alias{evacpath-package}
\title{evacpath: Least-cost pedestrian evacuation modeling in R}
\description{
\code{evacpath} provides tools for modeling pedestrian evacuation distance and
travel time from hazard zones to safety areas using road-constrained least-cost
path analysis.
}
\details{
The package is organized as a set of modular functions. Typical workflows use
\code{prepare_tsunami_zones()} to create tsunami-specific hazard and escape zones,
\code{crop_roads_to_inner_extent()} and \code{make_road_aware_escape_zone()} to prevent
false escape points along artificial study-area boundaries while retaining
bridge and walkway corridors, \code{find_escape_points()} to identify candidate
exits, \code{make_conductance_surface()} to build a slope-based conductance surface,
and \code{run_evacpath()} to run the full workflow.
}
\author{
\strong{Maintainer}: Elvin Cordero \email{elvin.cordero1@upr.edu}

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance_time_polygons.R
\name{calc_min_distance_to_safety}
\alias{calc_min_distance_to_safety}
\title{Calculate minimum least-cost distance from origins to safety}
\usage{
calc_min_distance_to_safety(
  cs,
  origins,
  destinations,
  include_destinations = TRUE,
  progress = FALSE,
  progress_every = 1L,
  check_locations = FALSE
)
}
\arguments{
\item{cs}{A \code{leastcostpath} conductance surface.}

\item{origins}{Origin points.}

\item{destinations}{Escape/safety destination points.}

\item{include_destinations}{Logical. Add destination points with distance = 0.}

\item{progress}{Logical. Print simple progress messages.}

\item{progress_every}{Integer. Print progress every \code{n} origins when \code{progress = TRUE}.}

\item{check_locations}{Logical passed to \code{leastcostpath::create_lcp()}. The default
is \code{FALSE} for speed after inputs have already been projected and cropped.}
}
\value{
A point \code{SpatVector} with columns \code{distance} and \code{type}.
}
\description{
For each origin point, calculates least-cost paths to all candidate safety
points and stores the shortest finite path distance. Destination points are
appended to the output with distance equal to zero.
}
\examples{
dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
  vals = 1, crs = "EPSG:3857")
cs <- make_conductance_surface(dem)
origins <- terra::vect(data.frame(x = 0.5, y = 0.5), geom = c("x", "y"), crs = "EPSG:3857")
destinations <- terra::vect(data.frame(x = 4.5, y = 4.5), geom = c("x", "y"), crs = "EPSG:3857")
calc_min_distance_to_safety(cs, origins, destinations)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/escape_boundary.R
\name{crop_roads_to_inner_extent}
\alias{crop_roads_to_inner_extent}
\title{Crop roads to an inset extent before escape-point detection}
\usage{
crop_roads_to_inner_extent(roads, zone, inset_x_m = 250, inset_y_m = 250)
}
\arguments{
\item{roads}{Road/pathway network as a \code{SpatVector} or file path.}

\item{zone}{Zone used to define the outer extent. Can be a \code{SpatVector},
\code{SpatRaster}, or file path.}

\item{inset_x_m}{Numeric. Distance to inset the minimum and maximum x
boundaries, in map units. Use meters when the data are projected.}

\item{inset_y_m}{Numeric. Distance to inset the minimum and maximum y
boundaries, in map units. Use meters when the data are projected.}
}
\value{
A cropped road/pathway \code{SpatVector}.
}
\description{
Crops a road/pathway layer to a slightly reduced bounding box around a zone.
This is useful before escape-point detection because roads extending beyond the
study-area coverage can intersect artificial raster or polygon extent edges and
create false escape/safety points.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
zone <- terra::as.polygons(r, dissolve = TRUE)
roads <- terra::vect(matrix(c(-1, 2, 5, 2), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
crop_roads_to_inner_extent(roads, zone, inset_x_m = 0.5, inset_y_m = 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid_region_escape.R
\name{make_region_area}
\alias{make_region_area}
\title{Create a broader analysis region around a study area}
\usage{
make_region_area(hazard_zone, study_area, buffer_m = 5000)
}
\arguments{
\item{hazard_zone}{Full hazard/evacuation zone.}

\item{study_area}{Local study area.}

\item{buffer_m}{Buffer distance in map units, typically meters.}
}
\value{
A polygon \code{SpatVector} for the broader analysis region.
}
\description{
Dissolves the full hazard zone, buffers a smaller study area, and crops the
full zone to that buffer. This is useful when escape/safety points outside a
municipality or local study area should still be considered.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
zone <- terra::as.polygons(r, dissolve = TRUE)
study <- terra::as.polygons(terra::crop(r, terra::ext(1, 3, 1, 3)), dissolve = TRUE)
make_region_area(zone, study, buffer_m = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpolation_legacy.R
\name{interpolate_distance_surface}
\alias{interpolate_distance_surface}
\title{Interpolate a distance surface from least-cost distance points}
\usage{
interpolate_distance_surface(
  distance_points,
  region_area,
  study_area,
  resolution_coarse = 100,
  resolution_fine = 1,
  distance_col = "distance"
)
}
\arguments{
\item{distance_points}{Point layer with a distance column.}

\item{region_area}{Broader analysis region used for interpolation extent.}

\item{study_area}{Final study area used to crop and mask the output.}

\item{resolution_coarse}{Coarse interpolation resolution.}

\item{resolution_fine}{Fine output resolution.}

\item{distance_col}{Name of distance column in \code{distance_points}.}
}
\value{
A \code{SpatRaster} distance surface.
}
\description{
Uses thin-plate spline interpolation to create a continuous distance surface.
This preserves the original paper-script approach while keeping interpolation
optional. For most package workflows, \code{make_evac_polygons()} is the simpler
output.
}
\examples{
if (requireNamespace("fields", quietly = TRUE)) {
  pts <- terra::vect(
    data.frame(
      x = c(0, 1, 0, 1, 0.5, 1.5),
      y = c(0, 0, 1, 1, 0.5, 1.5),
      distance = c(0, 5, 10, 15, 7, 20)
    ),
    geom = c("x", "y"),
    crs = "EPSG:3857"
  )
  area <- terra::as.polygons(terra::rast(nrows = 2, ncols = 2, xmin = -1, xmax = 2,
    ymin = -1, ymax = 2, vals = 1, crs = "EPSG:3857"), dissolve = TRUE)
  interpolate_distance_surface(pts, area, area, resolution_coarse = 0.5, resolution_fine = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_evacpath.R
\name{run_evacpath}
\alias{run_evacpath}
\title{Run the full evacuation-path modeling workflow}
\usage{
run_evacpath(
  hazard_zone,
  roads,
  dem,
  target_crs = NULL,
  region_name = NULL,
  escape_zone = NULL,
  roads_for_escape = NULL,
  escape_roads_inset_x_m = 0,
  escape_roads_inset_y_m = 0,
  road_aware_escape_zone = FALSE,
  escape_zone_road_buffer_m = NULL,
  escape_zone_crop_buffer_m = NULL,
  study_area = NULL,
  road_exclude = NULL,
  grid_resolution = NULL,
  grid_resolution_factor = 5,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  final_road_buffer_m = 3,
  region_buffer_m = 5000,
  dem_resolution = NULL,
  max_origins = NULL,
  max_destinations = NULL,
  seed = 23401,
  walking_speed_mps = 1.22,
  clip_mode = c("hazard", "road_hazard", "hazard_plus_roads", "none"),
  progress = FALSE,
  progress_every = 1L,
  lcp_check_locations = FALSE
)
}
\arguments{
\item{hazard_zone}{Hazard/inundation zone as \code{SpatRaster}, \code{SpatVector}, or path.
This should usually be the land-only area where evacuation origins and final
output surfaces are mapped.}

\item{roads}{Road/pathway network as \code{SpatVector} or path.}

\item{dem}{Elevation raster as \code{SpatRaster} or path.}

\item{target_crs}{Optional projected CRS in meters, for example \code{"EPSG:32748"}.}

\item{region_name}{Optional region name stored in output polygons.}

\item{escape_zone}{Optional boundary zone used only to identify escape/safety
points. For tsunami workflows, pass the land-inundation-plus-water zone from
\code{prepare_tsunami_zones()} here. If \code{NULL}, \code{hazard_zone} is used.}

\item{roads_for_escape}{Optional road/pathway layer used only for escape-point
detection. If \code{NULL}, \code{roads} is used. This is useful when the full road
dataset extends beyond the reliable hazard-zone study extent.}

\item{escape_roads_inset_x_m}{Optional x-direction inset applied to
\code{roads_for_escape} before escape-point detection. This prevents roads from
intersecting artificial study-area extent boundaries.}

\item{escape_roads_inset_y_m}{Optional y-direction inset applied to
\code{roads_for_escape} before escape-point detection.}

\item{road_aware_escape_zone}{Logical. If \code{TRUE}, buffered \code{roads_for_escape}
are combined with \code{escape_zone} before escape points are generated. This
preserves bridge, causeway, and walkway corridors over water that can be lost
when the tsunami layer is split into land and water masks.}

\item{escape_zone_road_buffer_m}{Road buffer used when \code{road_aware_escape_zone = TRUE}.}

\item{escape_zone_crop_buffer_m}{Additional buffer used when \code{road_aware_escape_zone = TRUE}.}

\item{study_area}{Optional local study area for limiting escape-point search.}

\item{road_exclude}{Optional list passed to \code{clean_roads()}.}

\item{grid_resolution}{Evacuation-grid resolution. If \code{NULL}, calculated from
\code{terra::res(dem) * grid_resolution_factor}.}

\item{grid_resolution_factor}{Multiplier applied to DEM resolution when
\code{grid_resolution = NULL}.}

\item{road_buffer_m}{Road buffer distance.}

\item{escape_buffer_m}{Escape-point buffer distance.}

\item{final_road_buffer_m}{Output clipping buffer around roads.}

\item{region_buffer_m}{Buffer around \code{study_area} used when finding escape points.}

\item{dem_resolution}{Optional DEM resolution used before conductance creation.}

\item{max_origins}{Optional maximum number of road origin points.}

\item{max_destinations}{Optional maximum number of escape/safety destination points.
This is useful for quick tests in regions where roads intersect the hazard
boundary many times.}

\item{seed}{Random seed used when \code{max_origins} or \code{max_destinations} is supplied.}

\item{walking_speed_mps}{Walking speed in meters per second.}

\item{clip_mode}{Output clipping mode. The default \code{"hazard"} creates a
continuous time-grid polygon surface clipped to the land-only hazard zone.
Use \code{"road_hazard"} for the older road-buffer-limited output,
\code{"hazard_plus_roads"} to retain both, or \code{"none"} for unclipped Voronoi polygons.}

\item{progress}{Logical. Print progress while running least-cost paths.}

\item{progress_every}{Integer. Print progress every \code{n} origins when \code{progress = TRUE}.}

\item{lcp_check_locations}{Logical passed to \code{leastcostpath::create_lcp()}.
Default is \code{FALSE} for speed after projection/cropping checks.}
}
\value{
An \code{evacpath_result} list containing spatial outputs and parameters.
}
\description{
This high-level wrapper runs the core \code{evacpath} pipeline: read/project inputs,
create an evacuation grid, identify escape/safety points, build a road mask,
create a slope-based conductance surface, calculate minimum least-cost distance
to safety, and create evacuation-time polygons.
}
\details{
For tsunami applications, \code{hazard_zone} and \code{escape_zone} should often be
different. Use a land-only \code{hazard_zone} for origins and output mapping, but
use a water-combined \code{escape_zone} for escape-point detection so the coastline
is not treated as an artificial safety boundary. The helper
\code{prepare_tsunami_zones()} creates both objects.
}
\examples{
dem <- terra::rast(nrows = 7, ncols = 7, xmin = 0, xmax = 7, ymin = 0, ymax = 7,
  vals = 1, crs = "EPSG:3857")
hazard_raster <- terra::crop(dem, terra::ext(1, 6, 1, 6))
hazard <- terra::as.polygons(hazard_raster, dissolve = TRUE)
roads <- terra::vect(matrix(c(0, 3.5, 7, 3.5), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
result <- run_evacpath(
  hazard_zone = hazard,
  roads = roads,
  dem = dem,
  grid_resolution = 1,
  road_buffer_m = 0.2,
  escape_buffer_m = 0.3,
  final_road_buffer_m = 0.2,
  max_origins = 2,
  max_destinations = 2,
  seed = 1
)
result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsunami_zones.R
\name{prepare_tsunami_zones}
\alias{prepare_tsunami_zones}
\title{Prepare separate tsunami zones for escape analysis and visualization}
\usage{
prepare_tsunami_zones(
  inundation,
  dem,
  target_crs = NULL,
  inundation_threshold = 0,
  land_threshold = 0,
  water_threshold = 0,
  dem_sign_multiplier = 1,
  resample_method = "bilinear",
  as_polygon = TRUE,
  dissolve = TRUE
)
}
\arguments{
\item{inundation}{Inundation-depth raster or path to a raster. Cells greater
than \code{inundation_threshold} are treated as inundated.}

\item{dem}{Elevation/topobathymetry raster or path to a raster.}

\item{target_crs}{Optional projected CRS in meters for returned objects.}

\item{inundation_threshold}{Numeric threshold used to define inundated cells.}

\item{land_threshold}{Numeric DEM threshold used to define land. Default is
\code{0}, so land is \code{dem > 0} after applying \code{dem_sign_multiplier}.}

\item{water_threshold}{Numeric DEM threshold used to define water. Default is
\code{0}, so water is \code{dem < 0} after applying \code{dem_sign_multiplier}.}

\item{dem_sign_multiplier}{Multiplier applied to the DEM before land/water
classification. Use \code{-1} when the DEM sign convention is reversed.}

\item{resample_method}{Method passed to \code{terra::resample()} when aligning the
inundation raster to the DEM. Use \code{"bilinear"} for depth rasters and
\code{"near"} for categorical rasters.}

\item{as_polygon}{Logical. If \code{TRUE}, return polygon zones in addition to
rasters.}

\item{dissolve}{Logical. Dissolve polygon pieces.}
}
\value{
A named list with land-only \code{hazard_zone}, water-combined
\code{escape_zone}, and supporting rasters. \code{hazard_zone} should usually be used
for origin generation, mapping, and output clipping. \code{escape_zone} should
usually be passed to \code{run_evacpath(escape_zone = ...)} or
\code{find_escape_points()}.
}
\description{
Tsunami evacuation workflows often need two different zone objects. The
land-only inundation zone is the area where road origins and output time
surfaces should be mapped. The escape-boundary zone should combine the
land-only inundation zone with water so that the coastline is not treated as
an artificial escape boundary. This prevents false escape/safety points along
the water-land edge when roads touch or approach the shoreline.
}
\examples{
dem <- terra::rast(nrows = 6, ncols = 6, xmin = 0, xmax = 6, ymin = 0, ymax = 6,
  crs = "EPSG:3857")
xy <- terra::crds(dem, df = TRUE)
terra::values(dem) <- -1.5 + 0.7 * xy$x + 0.2 * sin(xy$y)
inundation <- dem
terra::values(inundation) <- ifelse(terra::values(dem) > 0 & terra::values(dem) < 2.5, 1, 0)
zones <- prepare_tsunami_zones(inundation, dem, as_polygon = TRUE)
names(zones)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance_time_polygons.R
\name{calculate_lc_path}
\alias{calculate_lc_path}
\title{Calculate a least-cost path between one origin and one destination}
\usage{
calculate_lc_path(cs, origin, destination)
}
\arguments{
\item{cs}{A \code{leastcostpath} conductance surface.}

\item{origin}{Origin point.}

\item{destination}{Destination point.}
}
\value{
A least-cost path object or \code{NULL}.
}
\description{
Compatibility wrapper around \code{\link[leastcostpath:create_lcp]{leastcostpath::create_lcp()}} that returns \code{NULL}
when a path cannot be created.
}
\examples{
dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
  vals = 1, crs = "EPSG:3857")
cs <- make_conductance_surface(dem)
origin <- terra::vect(data.frame(x = 0.5, y = 0.5), geom = c("x", "y"), crs = "EPSG:3857")
destination <- terra::vect(data.frame(x = 4.5, y = 4.5), geom = c("x", "y"), crs = "EPSG:3857")
calculate_lc_path(cs, origin, destination)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance_time_polygons.R
\name{calculate_min_dist}
\alias{calculate_min_dist}
\title{Calculate the minimum path distance from a list of least-cost paths}
\usage{
calculate_min_dist(lc_paths_list)
}
\arguments{
\item{lc_paths_list}{A list of least-cost path line vectors.}
}
\value{
Minimum non-zero, finite path distance.
}
\description{
Calculate the minimum path distance from a list of least-cost paths
}
\examples{
line <- terra::vect(matrix(c(0, 0, 1, 1), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
calculate_min_dist(list(line))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare.R
\name{clean_roads}
\alias{clean_roads}
\title{Clean a road/pathway network}
\usage{
clean_roads(roads, exclude = NULL, target_crs = NULL)
}
\arguments{
\item{roads}{A \code{SpatVector} or path to a vector layer.}

\item{exclude}{Optional named list with \code{field} and \code{values}, for example
\code{list(field = "man_made", values = "pier")}.}

\item{target_crs}{Optional output CRS.}
}
\value{
A cleaned \code{SpatVector}.
}
\description{
Removes user-specified features from a road/pathway layer. This is useful for
excluding piers, tunnels, private ways, or other features that should not be
used in pedestrian evacuation modeling.
}
\examples{
roads <- terra::vect(
  list(
    matrix(c(0, 0, 0, 1), ncol = 2, byrow = TRUE),
    matrix(c(1, 0, 1, 1), ncol = 2, byrow = TRUE)
  ),
  type = "lines",
  crs = "EPSG:3857"
)
roads$kind <- c("road", "pier")
clean_roads(roads, exclude = list(field = "kind", values = "pier"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance_time_polygons.R
\name{make_evac_polygons}
\alias{make_evac_polygons}
\title{Create evacuation-distance and evacuation-time polygons}
\usage{
make_evac_polygons(
  distance_points,
  clip_area = NULL,
  walking_speed_mps = 1.22,
  region_name = NULL,
  distance_col = "DistToSafety",
  time_col = "EvacTimeAvg"
)
}
\arguments{
\item{distance_points}{Point output from \code{calc_min_distance_to_safety()}.}

\item{clip_area}{Optional polygon used to crop the Voronoi output.}

\item{walking_speed_mps}{Walking speed in meters per second.}

\item{region_name}{Optional region/municipality name stored in the output.}

\item{distance_col}{Name of the output distance column.}

\item{time_col}{Name of the output time column.}
}
\value{
A polygon \code{SpatVector}.
}
\description{
Creates Voronoi polygons from least-cost distance points, converts distance to
travel time, and optionally clips the output to an inundated road/study mask.
}
\examples{
pts <- terra::vect(
  data.frame(x = c(0, 1, 0, 1), y = c(0, 0, 1, 1), distance = c(0, 5, 10, 15)),
  geom = c("x", "y"),
  crs = "EPSG:3857"
)
clip <- terra::as.polygons(terra::rast(nrows = 2, ncols = 2, xmin = -1, xmax = 2,
  ymin = -1, ymax = 2, vals = 1, crs = "EPSG:3857"), dissolve = TRUE)
make_evac_polygons(pts, clip_area = clip)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roads_conductance.R
\name{make_road_mask}
\alias{make_road_mask}
\title{Create a road-constrained analysis mask}
\usage{
make_road_mask(
  roads,
  escape_points,
  road_buffer_m = 2,
  escape_buffer_m = 5,
  return_components = FALSE
)
}
\arguments{
\item{roads}{Road/pathway network.}

\item{escape_points}{Candidate escape/safety points.}

\item{road_buffer_m}{Road buffer distance in map units, typically meters.}

\item{escape_buffer_m}{Escape-point buffer distance in map units, typically meters.}

\item{return_components}{Logical. If \code{TRUE}, return a list with the mask and
component buffers.}
}
\value{
A dissolved road mask \code{SpatVector}, or a list when \code{return_components = TRUE}.
}
\description{
Buffers the road network and candidate escape points, combines those buffered
areas, and dissolves the result. The mask is used to constrain least-cost
movement to the road/pathway network while allowing access around escape
locations.
}
\examples{
roads <- terra::vect(matrix(c(0, 1, 4, 1), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
escape <- terra::vect(data.frame(x = 4, y = 1), geom = c("x", "y"), crs = "EPSG:3857")
make_road_mask(roads, escape, road_buffer_m = 0.1, escape_buffer_m = 0.2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare.R
\name{prepare_hazard_zone}
\alias{prepare_hazard_zone}
\title{Prepare a hazard zone from an inundation raster}
\usage{
prepare_hazard_zone(
  inundation,
  threshold = 0,
  land_mask = NULL,
  target_crs = NULL,
  as_polygon = TRUE,
  dissolve = TRUE
)
}
\arguments{
\item{inundation}{A \code{SpatRaster} or path to a raster.}

\item{threshold}{Numeric threshold. Cells greater than \code{threshold} are treated
as inside the hazard zone.}

\item{land_mask}{Optional \code{SpatRaster} or \code{SpatVector} used to mask the hazard
zone to land.}

\item{target_crs}{Optional output CRS. Use a projected CRS in meters for later
distance calculations.}

\item{as_polygon}{Logical. If \code{TRUE}, return a polygon hazard zone.}

\item{dissolve}{Logical. If \code{TRUE}, dissolve polygon pieces.}
}
\value{
A binary \code{SpatRaster} or polygon \code{SpatVector}.
}
\description{
Converts an inundation raster to a binary hazard-zone raster or polygon using
a threshold. This is intentionally general so users can adapt it to tsunami,
flood, storm-surge, or other hazard layers.
}
\examples{
r <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5)
terra::values(r) <- c(rep(0, 12), rep(1, 13))
zone <- prepare_hazard_zone(r, threshold = 0, as_polygon = TRUE)
zone
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance_time_polygons.R
\name{calc_evac_time}
\alias{calc_evac_time}
\title{Convert distance to evacuation time}
\usage{
calc_evac_time(distance_m, walking_speed_mps = 1.22, units = "minutes")
}
\arguments{
\item{distance_m}{Distance in meters.}

\item{walking_speed_mps}{Walking speed in meters per second. The paper-style
default is 1.22 m/s, but this should be changed for local planning scenarios.}

\item{units}{Output units: \code{"minutes"}, \code{"seconds"}, or \code{"hours"}.}
}
\value{
Numeric vector of evacuation times.
}
\description{
Convert distance to evacuation time
}
\examples{
calc_evac_time(c(0, 120, 240), walking_speed_mps = 1.2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/escape_boundary.R
\name{make_road_aware_escape_zone}
\alias{make_road_aware_escape_zone}
\title{Add buffered road corridors to an escape-boundary zone}
\usage{
make_road_aware_escape_zone(
  escape_zone,
  roads,
  road_buffer_m = 2,
  crop_buffer_m = 3,
  include_base_zone = TRUE
)
}
\arguments{
\item{escape_zone}{Base escape-boundary zone, usually the land-inundation-plus-
water zone from \code{prepare_tsunami_zones()}.}

\item{roads}{Road/pathway network used to create the road-aware corridor.}

\item{road_buffer_m}{First road buffer distance in map units.}

\item{crop_buffer_m}{Optional second buffer applied before cropping/combining.}

\item{include_base_zone}{Logical. If \code{TRUE}, combine the buffered roads with
the original \code{escape_zone}. If \code{FALSE}, only the buffered road corridor is
returned.}
}
\value{
A dissolved \code{SpatVector} escape-boundary zone.
}
\description{
Creates a road-aware escape zone by combining a base escape zone with buffered
road/pathway corridors. This is useful for tsunami workflows in coastal cities
where bridges, causeways, or other walkways over water can be lost when the
inundation layer is split into land and water masks. The resulting object can
be passed to \code{find_escape_points()} so escape/safety points are generated from
a boundary that includes relevant road corridors as well as the tsunami zone.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
zone <- terra::as.polygons(r, dissolve = TRUE)
roads <- terra::vect(matrix(c(0, 2, 4, 2), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
make_road_aware_escape_zone(zone, roads, road_buffer_m = 0.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_evacpath.R
\name{make_output_clip_area}
\alias{make_output_clip_area}
\title{Make an output clip area for evacuation polygons}
\usage{
make_output_clip_area(
  hazard_zone,
  roads_buffer,
  final_road_buffer_m = 3,
  clip_mode = c("hazard", "road_hazard", "hazard_plus_roads", "none")
)
}
\arguments{
\item{hazard_zone}{Hazard zone polygon.}

\item{roads_buffer}{Buffered roads.}

\item{final_road_buffer_m}{Additional road buffer for output clipping.}

\item{clip_mode}{One of \code{"hazard"}, \code{"road_hazard"}, \code{"hazard_plus_roads"}, or \code{"none"}.}
}
\value{
A \code{SpatVector} clip area or \code{NULL}.
}
\description{
Make an output clip area for evacuation polygons
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roads_conductance.R
\name{make_conductance_surface}
\alias{make_conductance_surface}
\title{Create a slope-based conductance surface}
\usage{
make_conductance_surface(
  dem,
  road_mask = NULL,
  resolution = NULL,
  method = "slope"
)
}
\arguments{
\item{dem}{Elevation raster.}

\item{road_mask}{Optional road/pathway mask.}

\item{resolution}{Optional target DEM resolution before conductance creation.}

\item{method}{Conductance method. Currently only \code{"slope"} is implemented.}
}
\value{
A \code{leastcostpath} conductance surface object.
}
\description{
Masks a DEM to an optional road/pathway mask and creates a slope conductance
surface using \code{leastcostpath}.
}
\examples{
dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
  vals = 1, crs = "EPSG:3857")
make_conductance_surface(dem)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsunami_zones.R
\name{make_roads_in_zone}
\alias{make_roads_in_zone}
\title{Create a buffered road area inside an inundation or analysis zone}
\usage{
make_roads_in_zone(
  roads,
  zone,
  road_buffer_m = 2,
  crop_buffer_m = 3,
  include_zone = TRUE
)
}
\arguments{
\item{roads}{Road/pathway network.}

\item{zone}{Polygon/raster zone used to crop buffered roads.}

\item{road_buffer_m}{First road buffer distance.}

\item{crop_buffer_m}{Optional second buffer applied before cropping.}

\item{include_zone}{Logical. If \code{TRUE}, combine the cropped road buffer with
\code{zone} and dissolve the result.}
}
\value{
A \code{SpatVector}.
}
\description{
Mirrors the original script logic where roads were buffered, optionally
buffered again for tolerance, cropped to the inundation zone, and then
combined with the zone. This is useful for quality assurance and quality
control and for reproducing the earlier road-plus-inundation analysis area.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
zone <- terra::as.polygons(r, dissolve = TRUE)
roads <- terra::vect(matrix(c(0, 2, 4, 2), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
make_roads_in_zone(roads, zone, road_buffer_m = 0.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid_region_escape.R
\name{make_evac_grid}
\alias{make_evac_grid}
\title{Create an evacuation grid}
\usage{
make_evac_grid(hazard_zone, resolution)
}
\arguments{
\item{hazard_zone}{Hazard/evacuation zone as \code{SpatVector}, \code{SpatRaster}, or file path.}

\item{resolution}{Grid cell resolution in map units. Use meters when data are
in a projected CRS. Can be length 1 or 2.}
}
\value{
A polygon \code{SpatVector} grid clipped/masked to the hazard zone.
}
\description{
Creates polygon grid cells over a hazard/evacuation zone and masks the grid to
the zone. The resulting cells can be intersected with buffered roads to create
road-based origin points.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
zone <- terra::as.polygons(r, dissolve = TRUE)
make_evac_grid(zone, resolution = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{read_spatial}
\alias{read_spatial}
\title{Read a spatial input}
\usage{
read_spatial(x)
}
\arguments{
\item{x}{A \code{SpatRaster}, \code{SpatVector}, or file path.}
}
\value{
A \code{SpatRaster} or \code{SpatVector}.
}
\description{
Accepts an existing \code{terra} object or a file path and returns a \code{SpatRaster}
or \code{SpatVector}. Raster-like extensions are read with \code{\link[terra:rast]{terra::rast()}} and
vector-like extensions are read with \code{\link[terra:vect]{terra::vect()}}.
}
\examples{
r <- terra::rast(nrows = 2, ncols = 2, vals = 1)
read_spatial(r)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roads_conductance.R
\name{make_road_origins}
\alias{make_road_origins}
\title{Create road-based origin points inside the evacuation zone}
\usage{
make_road_origins(
  evac_grid,
  roads_buffer,
  hazard_zone = NULL,
  max_origins = NULL,
  seed = NULL
)
}
\arguments{
\item{evac_grid}{Polygon grid from \code{make_evac_grid()}.}

\item{roads_buffer}{Buffered road/pathway network.}

\item{hazard_zone}{Optional hazard-zone polygon used to crop candidate
origins after intersecting the grid with the road buffer.}

\item{max_origins}{Optional maximum number of origin points to retain. Useful
for large regions or exploratory runs.}

\item{seed}{Random seed used when \code{max_origins} is supplied.}
}
\value{
A point \code{SpatVector} of road-based evacuation origins.
}
\description{
Intersects an evacuation grid with a buffered road network and converts the
resulting road-crossing cell geometry to points.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
grid <- make_evac_grid(terra::as.polygons(r, dissolve = TRUE), resolution = 1)
roads <- terra::vect(matrix(c(0, 2, 4, 2), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
roads_buffer <- terra::buffer(roads, 0.2)
make_road_origins(grid, roads_buffer, hazard_zone = terra::as.polygons(r, dissolve = TRUE),
  max_origins = 3, seed = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid_region_escape.R
\name{find_escape_points}
\alias{find_escape_points}
\title{Identify escape/safety points where roads cross the hazard-zone boundary}
\usage{
find_escape_points(
  hazard_zone,
  roads,
  study_area = NULL,
  region_buffer_m = 5000
)
}
\arguments{
\item{hazard_zone}{Hazard/evacuation zone.}

\item{roads}{Road/pathway network.}

\item{study_area}{Optional local study area used to crop candidate escape
points to a broader region around the study area.}

\item{region_buffer_m}{Buffer distance passed to \code{make_region_area()} when
\code{study_area} is supplied.}
}
\value{
A point \code{SpatVector} of candidate escape/safety points.
}
\description{
Intersects a road/pathway network with the boundary of the hazard zone and
converts the intersection geometry to points. These points represent candidate
exits from the hazard zone.
}
\examples{
r <- terra::rast(nrows = 4, ncols = 4, xmin = 0, xmax = 4, ymin = 0, ymax = 4,
  vals = 1, crs = "EPSG:3857")
zone <- terra::as.polygons(r, dissolve = TRUE)
roads <- terra::vect(matrix(c(-1, 2, 5, 2), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
find_escape_points(zone, roads)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_outputs.R
\name{write_evac_outputs}
\alias{write_evac_outputs}
\title{Write evacpath outputs to disk}
\usage{
write_evac_outputs(
  result,
  output_dir,
  prefix = "evacpath",
  overwrite = TRUE,
  include_inputs = FALSE
)
}
\arguments{
\item{result}{An object returned by \code{run_evacpath()}.}

\item{output_dir}{Output directory.}

\item{prefix}{Filename prefix.}

\item{overwrite}{Logical. Overwrite existing files.}

\item{include_inputs}{Logical. Also write projected input layers.}
}
\value{
A named character vector of written file paths.
}
\description{
Writes the main spatial outputs in an \code{evacpath_result} list to GeoPackage and
GeoTIFF files. Non-spatial objects, including the conductance surface, are not
written.
}
\examples{
dem <- terra::rast(nrows = 7, ncols = 7, xmin = 0, xmax = 7, ymin = 0, ymax = 7,
  vals = 1, crs = "EPSG:3857")
hazard_raster <- terra::crop(dem, terra::ext(1, 6, 1, 6))
hazard <- terra::as.polygons(hazard_raster, dissolve = TRUE)
roads <- terra::vect(matrix(c(0, 3.5, 7, 3.5), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
result <- run_evacpath(
  hazard_zone = hazard,
  roads = roads,
  dem = dem,
  grid_resolution = 1,
  road_buffer_m = 0.2,
  escape_buffer_m = 0.3,
  final_road_buffer_m = 0.2,
  max_origins = 2,
  max_destinations = 2,
  seed = 1
)
out_dir <- file.path(tempdir(), "evacpath-example")
paths <- write_evac_outputs(result, output_dir = out_dir, prefix = "example")
unlink(out_dir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare.R
\name{prepare_evac_inputs}
\alias{prepare_evac_inputs}
\title{Read and project the core evacuation inputs}
\usage{
prepare_evac_inputs(
  hazard_zone,
  roads,
  dem,
  target_crs = NULL,
  hazard_as_polygon = TRUE,
  dissolve_hazard = TRUE,
  road_exclude = NULL
)
}
\arguments{
\item{hazard_zone}{Hazard/inundation zone as a \code{SpatRaster}, \code{SpatVector}, or file path.}

\item{roads}{Road/pathway network as a \code{SpatVector} or file path.}

\item{dem}{Elevation raster as a \code{SpatRaster} or file path.}

\item{target_crs}{Optional projected CRS in meters.}

\item{hazard_as_polygon}{Logical. Convert raster hazard zones to polygons.}

\item{dissolve_hazard}{Logical. Dissolve hazard polygon pieces.}

\item{road_exclude}{Optional road exclusion list passed to \code{clean_roads()}.}
}
\value{
A named list with \code{hazard_zone}, \code{roads}, and \code{dem}.
}
\description{
Read and project the core evacuation inputs
}
\examples{
dem <- terra::rast(nrows = 5, ncols = 5, xmin = 0, xmax = 5, ymin = 0, ymax = 5,
  vals = 1, crs = "EPSG:3857")
hazard <- terra::as.polygons(dem, dissolve = TRUE)
roads <- terra::vect(matrix(c(0, 2.5, 5, 2.5), ncol = 2, byrow = TRUE),
  type = "lines", crs = "EPSG:3857")
inputs <- prepare_evac_inputs(hazard, roads, dem)
names(inputs)
}
